@#define FORESIGHT_H   = 2
@#define FORESIGHT_Hm1 = 1
@#define FORESIGHT_Hp1 = 3

model(linear); 
@#include "NK_model_stst_v1.mod"

@#if SEPARATE_NX!=1
    #crhonx=crhog;
@#endif
@#if SEPARATE_TREND!=1
    #ctrendy=ctrend;
    #ctrendinv=ctrend;
    #ctrendw=ctrend;
    #ctrendg=ctrend;
@#endif

@#if TBILL_SHOCK==1
    #tbill=1;
@#else
    #tbill=0;
@#endif

@#if SMOOTH_TAX<-2
    @#if DO_TAX>0
        #cr_tl =crho_tl;
        #cr_tk =crho_tk;
        #cr_s=crho_s;
    @#else
        #cr_tl =0;
        #cr_tk =0;
        #cr_s=0;
    @#endif
    #cr_g=crhog;
@#endif

@#if CAP_U_IS_DEPRECIATION==1
    #no_cap_u=1;
@#else
    #no_cap_u=0;
@#endif

#cdelta1= crk;
#cdelta2= ( (czcap) / (1-czcap) ) * cdelta1;

#csadjcost_aux=1/(cgamma^2*csadjcost);

@#if LABOR_ONLY!=0
    #zcap=0;
    #rk=0;
    #k=0;
    #kp=0;
    #inve=0;
    #inve_obs=0;
    #pk=0;
    #dinv=0;
    #drev_k=0;
    #m_CI=0;
    #m_CI_lvl=0;
    #dtau_k=0;
@#endif

% ***********START OF MAIN MODEL EQUATIONS******************************************
    % Marginal cost  1
    mc =  calfa*rk+(1-calfa)*w  - a - f_a(-@{FORESIGHT_H});
    0 =  calfa*rkf+(1-calfa)*wf  - a - f_a(-@{FORESIGHT_H});

    @#if LABOR_ONLY==0
        % Capacity utilization 2
        @#if CAP_U_IS_DEPRECIATION==1
          zcap * cdelta2 / cdelta1
            =  
          rk  - pk / (1-ctau_k);

          zcapf * cdelta2 / cdelta1
            =  
          rkf  - pkf / (1-ctau_k);
        @#else
          % SW version
          zcap =  (1/(czcap/(1-czcap)))* rk  ;
          zcapf =  (1/(czcap/(1-czcap)))* rkf  ;
        @#endif

        % Factor demand: capital 3
        rk =  w+lab-k ;
        rkf =  wf+labf-kf ;

        % Effective supply of capital 4
        k =  kp(-1)+zcap ;
        kf =  kpf(-1)+zcapf ;
  
        % Capital LoM 5
        kp =  (1-cikbar)*kp(-1)+cikbar*inve - no_cap_u*zcap*cdelta1/cgamma+ (cikbar/csadjcost_aux)*qs ;
        kpf =  (1-cikbar)*kpf(-1)+cikbar*invef - no_cap_u*zcapf*cdelta1/cgamma+ (cikbar/csadjcost_aux)*qs ;

        % investment Euler equation 6
        inve = (1/(1+cbetabar*cgamma))* (  inve(-1) + cbetabar*cgamma*inve(1)+(csadjcost_aux)*pk ) +qs ;
        invef = (1/(1+cbetabar*cgamma))* (  invef(-1) + cbetabar*cgamma*invef(1)+(csadjcost_aux)*pkf ) +qs ;

        % Tobin's Q 7
        pk = -r+pinf(1)-tbill*b +(1/((1-chabb/cgamma)/(csigma*(1+chabb/cgamma))))*b*(1-tbill)
            +(1/(crk*(1-ctau_k)+(1-cdelta)+ctau_k*cdelta))*
            ( crk*(1-ctau_k) * rk(1) +  (1-cdelta)*pk(1) 
                    - (crk - cdelta) * dtau_k(+1));
        pkf = -rf-tbill*b +(1/((1-chabb/cgamma)/(csigma*(1+chabb/cgamma))))*b*(1-tbill)
            +(1/(crk*(1-ctau_k)+(1-cdelta)+ctau_k*cdelta))*
            ( crk*(1-ctau_k) * rkf(1) +  (1-cdelta)*pkf(1) 
                    - (crk - cdelta) * dtau_kf(+1));
    @#endif

    % Consumption Euler equation 8
    c*csigma*(1+chabb/cgamma) = csigma*(chabb/cgamma)*c(-1) + csigma*c(+1) +( (csigma-1)*cwhlc )*(lab-lab(+1)) 
                - (1-chabb/cgamma) *(r-pinf(+1)+tbill*b + dtau_c(+1)-dtau_c ) + b*(1-tbill) ;
    cf*csigma*(1+chabb/cgamma) = csigma*(chabb/cgamma)*cf(-1) + csigma*cf(+1) +( (csigma-1)*cwhlc )*(labf-labf(+1)) 
                - (1-chabb/cgamma) *(rf+tbill*b + dtau_cf(+1)-dtau_cf ) + b*(1-tbill) ;

    % Resource constraint 9
    y = ccy*c+ciy*inve + gc + f_g(-@{FORESIGHT_H}) + nx  +  (1-no_cap_u)*crkky*zcap ;
    yf = ccy*cf+ciy*invef + gcf + f_g(-@{FORESIGHT_H}) + nx  +  (1-no_cap_u)*crkky*zcapf ;

    % production function 10
    y  = cfc*( calfa*k+(1-calfa)*lab ) + a + f_a(-@{FORESIGHT_H}) ;
    yf = cfc*( calfa*kf+(1-calfa)*labf ) + a + f_a(-@{FORESIGHT_H}) ;

    % Phillips Curve 11
    pinf =  (1/(1+cbetabar*cgamma*cindp)) * ( cbetabar*cgamma*pinf(1) +cindp*pinf(-1) 
       +((1-cprobp)*(1-cbetabar*cgamma*cprobp)/cprobp)/((cfc-1)*curvp+1)*(mc)  )  + spinf ; 

    % Wage setting 12
    w =  (1/(1+cbetabar*cgamma))*w(-1)
       +(cbetabar*cgamma/(1+cbetabar*cgamma))*w(1)
       +(cindw/(1+cbetabar*cgamma))*pinf(-1)
       -(1+cbetabar*cgamma*cindw)/(1+cbetabar*cgamma)*pinf
       +(cbetabar*cgamma)/(1+cbetabar*cgamma)*pinf(1)
       +(1-cprobw)*(1-cbetabar*cgamma*cprobw)/((1+cbetabar*cgamma)*cprobw)*(1/((clandaw-1)*curvw+1))*
       (csigl*lab + (1/(1-chabb/cgamma))*c - ((chabb/cgamma)/(1-chabb/cgamma))*c(-1) -w 
       + dtau_c / (1+ctau_c) + (dtau_l + f_t_l(-@{FORESIGHT_H}) )/ (1-ctau_l)) + 1*sw ;
    wf =  csigl*labf + (1/(1-chabb/cgamma))*cf - ((chabb/cgamma)/(1-chabb/cgamma))*cf(-1)
       + dtau_cf / (1+ctau_c) + (dtau_lf + f_t_l(-@{FORESIGHT_H}) )/ (1-ctau_l) ;

    % FFR Taylor rule 13
    r =  crpi*(1-crr)*pinf +cry*(1-crr)*(y-yf) +crdy*(y-yf-(y(-1)-yf(-1)))
           +crr*r(-1) + ms + f_m(-@{FORESIGHT_H}) +
        + (1-crr)*(crpi_f*Ex_inf +cry_f*Ex_y + crdy_f*(Ex_y-y))  ;

    % Government Spending fiscal rule 14
    gc = cr_g*gc(-1)-(1-cr_g)*(cpsi_g_b*cby*debt(-@{DEBT_LAGS}) +cpsi_g_y*y(-@{Y_LAGS})*1)+u_g;
    gcf = cr_g*gcf(-1)-(1-cr_g)*(cpsi_g_b*cby*debtf(-@{DEBT_LAGS}) +cpsi_g_y*yf(-@{Y_LAGS})*1)+u_g;

    % Taylor rule: Labor tax 15
    clandaw*cwhlc * ccy  *dtau_l = cr_tl * clandaw*cwhlc * ccy  *dtau_l(-1) + (1-cr_tl)*(cpsi_l_y*(y(-@{Y_LAGS}))*1 + cpsi_l_b*cby*debt(-@{DEBT_LAGS}))  + ct_l_k * u_t_k + ct_l_c * u_t_c + u_t_l;
    clandaw*cwhlc * ccy  *dtau_lf = cr_tl * clandaw*cwhlc * ccy  *dtau_lf(-1) + (1-cr_tl)*(cpsi_l_y*(yf(-@{Y_LAGS}))*1 + cpsi_l_b*cby*debtf(-@{DEBT_LAGS}))  + ct_l_k * u_t_k + ct_l_c * u_t_c + u_t_l;
    % Taylor rule: Capital tax 16
    @#if LABOR_ONLY==0
        cky*(crk-cdelta) * dtau_k = cr_tk*cky*(crk-cdelta) * dtau_k(-1) + (1-cr_tk)*(cpsi_k_y*(y(-@{Y_LAGS}))*1 + cpsi_k_b*cby*debt(-@{DEBT_LAGS})) + ct_k_l * u_t_l + ct_k_c * u_t_c + u_t_k;
        cky*(crk-cdelta) * dtau_kf = cr_tk*cky*(crk-cdelta) * dtau_kf(-1) + (1-cr_tk)*(cpsi_k_y*(yf(-@{Y_LAGS}))*1 + cpsi_k_b*cby*debtf(-@{DEBT_LAGS})) + ct_k_l * u_t_l + ct_k_c * u_t_c + u_t_k;
    @#endif
    % Taylor rule: Transfers 17
    s=cr_s*s(-1)-(1-cr_s)*(cpsi_s_y*(y(-@{Y_LAGS}))*1 + cpsi_s_b * cby*debt(-@{DEBT_LAGS})+ nx) + u_s;
    sf=cr_s*sf(-1)-(1-cr_s)*(cpsi_s_y*(yf(-@{Y_LAGS}))*1 + cpsi_s_b * cby*debtf(-@{DEBT_LAGS})+ nx) + u_s;
    % Taylor rule: Consumption tax 18
    ccy  *dtau_c = ct_c_k * u_t_k + ct_c_l * u_t_l + u_t_c;
    ccy  *dtau_cf = ct_c_k * u_t_k + ct_c_l * u_t_l + u_t_c;

    % Government budget constraint 19
    (cby/cr)*(debt - r - tbill * b) 
        @#if LABOR_ONLY==0
            + ctau_k*cky*(crk-cdelta) * (dtau_k / ctau_k + (crk / (crk - cdelta)) * rk + kp(-1))
        @#endif
            + ctau_l * clandaw*cwhlc * ccy  * ( (dtau_l + f_t_l(-@{FORESIGHT_H}) ) / ctau_l + w + lab) 
            + ctau_c * ccy * (dtau_c / ctau_c + c) 
                = 
        gc + f_g(-@{FORESIGHT_H}) +s + (1 / cpie) * (cby/cgamma)*(debt(-1) - pinf) ;

    (cby/cr)*(debtf - rf - tbill * b) 
        @#if LABOR_ONLY==0
            + ctau_k*cky*(crk-cdelta) * (dtau_kf / ctau_k + (crk / (crk - cdelta)) * rkf + kpf(-1))
        @#endif
            + ctau_l * clandaw*cwhlc * ccy  * ( (dtau_lf + f_t_l(-@{FORESIGHT_H}) ) / ctau_l + wf + labf) 
            + ctau_c * ccy * (dtau_cf / ctau_c + cf) 
                = 
        gcf + f_g(-@{FORESIGHT_H}) +sf + (1 / cpie) * (cby/cgamma)*debtf(-1)  ;
% ***********END OF MAIN MODEL EQUATIONS******************************************

% ***********START OF AUXILIARY VARIABLES******************************************
    % real interest rate 20
    real_r=r-pinf(+1);
    % labor tax revenue 21
    rev_l = ctau_l * clandaw*cwhlc * ccy  * ( (dtau_l + f_t_l(-@{FORESIGHT_H}) ) / ctau_l + w  + lab ) ;
    % expected government consumption 22
    Egc = gc(+1) + f_g(-@{FORESIGHT_Hm1}) ;
% ***********END OF AUXILIARY VARIABLES******************************************


% ***********START OF MEASUREMENT EQUATIONS******************************************
    % GDP growth 23
    dy=y-y(-1)+ctrendy;
    % Private consumption growth 24
    dg=gc-gc(-1) + f_g(-@{FORESIGHT_H}) - f_g(-@{FORESIGHT_Hp1}) +ctrendy;
    % Government consumption growth 25
    dc=c-c(-1)+ctrend;
    % Investment growth 26
    @#if LABOR_ONLY==0
        dinv=inve-inve(-1)+ctrendinv;
    @#endif
    % Wage growth 27
    dw=w-w(-1)+ctrendw;
    % Inflation 28
    infobs = 1*(pinf) + constepinf;
    % Nominal interest rate 29
    tb3 =    1*(r) + conster;
    % labor tax rate 30
    dtau_l_obs= 1*dtau_l + f_t_l(-@{FORESIGHT_H}) + 100*ctau_l;
    % labor tax rate 30
    tauP = 1*dtau_l + f_t_l(-@{FORESIGHT_H}) + 100*ctau_l;
    % hours worked 31
    labobs = 1*lab + constelab;
    % capital tax revenue growth 32
    @#if LABOR_ONLY==0
        drev_k = ctau_k*cky*(crk-cdelta) * ( (dtau_k)  / ctau_k + (crk / (crk - cdelta)) * (rk) + kp(-1) - y)+se_t_k*err_k+consterev_k;
    @#endif
    % capital tax revenue growth 33
    drev_l = ctau_l * clandaw*cwhlc * ccy  * ( (dtau_l + f_t_l(-@{FORESIGHT_H}) ) / ctau_l + w  + lab - y) +se_t_l*err_l + consterev_l;
    % transfer revenue growth 34
    drev_c = ctau_c * ccy * ( (dtau_c) / ctau_c + c - y) + consterev_c;
    % debt to GDP 35
    // in the model, the debt to GDP ratio is at quarterly frequency
    // however, here everything else is also measured at quarterly rates, or at least all NIPA data is.
    debt_obs= cby * debt - cby*y+ 100*constedebt + se_s*err_debt;
    % debt to GDP in logs 36
    DEBT_Y = debt - y+ 100*log(constedebt) + se_s*err_debt;
    % Observation equation: Government spending 37
    gc_obs=1*gc+f_g(-@{FORESIGHT_H})+log(cg)*100+cy*100;
    % Observation equation: GDP 38
    y_obs=1*y+(cy)*100;
    % Observation equation: consumption 39
    c_obs=1*c+log(ccy)*100+cy*100;
    % Observation equation: Investment 40
    @#if LABOR_ONLY==0
        inve_obs=1*inve+log(ciy)*100+cy*100; 
    @#endif
% ***********END OF MEASUREMENT EQUATIONS******************************************

% ***********START OF NARRATIVE INSTRUMENTS******************************************
    % monetary policy observation equation 41
    m_signal = c_m_signal*em + se_m_signal*u_m_signal
    @#if CHECK_LOADINGS>=1 && (SMALL_SE==1 || SMALL_SE==3)
        +c_m_signal_a*se_a*ea+c_m_signal_b*se_b*eb
        +c_m_signal_g*se_g*eg+c_m_signal_nx*se_nx*enx
        +c_m_signal_qs*se_qs*eqs+c_m_signal_pinf*se_pinf*epinf
        +c_m_signal_w*se_w*ew
    @#endif
        ;

    % TFP observation equation 42
    a_signal = c_a_signal*(a-a(-1)) + se_a_signal*u_a_signal
    @#if CHECK_LOADINGS>=1 && (SMALL_SE==1 || SMALL_SE==5)
        +c_a_signal_g*se_g*eg+c_a_signal_b*se_b*eb
        +c_a_signal_m*se_m*em+c_a_signal_nx*se_nx*enx
        +c_a_signal_qs*se_qs*eqs+c_a_signal_pinf*se_pinf*epinf
        +c_a_signal_w*se_w*ew
    @#endif
        ;

    % government spending observation equation 43
    g_signal = c_g_signal*(gc + f_g(-@{FORESIGHT_H}) -Egc(-1)) + se_g_signal*u_g_signal
    @#if CHECK_LOADINGS>=1 && (SMALL_SE==1 || SMALL_SE==3)
        +c_g_signal_a*se_a*ea+c_g_signal_b*se_b*eb
        +c_g_signal_m*se_m*em+c_g_signal_nx*se_nx*enx
        +c_g_signal_qs*se_qs*eqs+c_g_signal_pinf*se_pinf*epinf
        +c_g_signal_w*se_w*ew
    @#endif
        ;

    % Defense spending: only a quarter of overall government spending 43
    g_signal_def_lvl = (c_g_signal_def*(gc-Egc(-1))/4 + se_g_signal*se_g*u_g_signal)/(cg/4)
    @#if CHECK_LOADINGS>=1 && (SMALL_SE==1 || SMALL_SE==3)
        +c_g_signal_a*se_a*ea+c_g_signal_b*se_b*eb
        +c_g_signal_m*se_m*em+c_g_signal_nx*se_nx*enx
        +c_g_signal_qs*se_qs*eqs+c_g_signal_pinf*se_pinf*epinf
        +c_g_signal_w*se_w*ew
    @#endif
        ;

    % Personal income tax observation 44
    m_PI = c_PI_signal * e_t_l + se_PI_signal * u_PI_signal;
    % Capital income tax observation
    @#if LABOR_ONLY==0
        m_CI = c_PI_CI_signal * e_t_l + c_CI_signal * e_t_k + se_CI_signal * u_CI_signal;
    @#endif

    % allow for the estimation using different observables 45
        m_signal_futures = m_signal;
    % allow for the estimation using different observables 46
        m_signal_lvl = m_signal; 
    % allow for the estimation using different observables 47
        m_signal_futures_lvl = m_signal;
    % allow for the estimation using different observables 48
        a_signal_lvl = a_signal;
    % allow for the estimation using different observables 49
        g_signal_lvl = g_signal;
    % allow for the estimation using different observables 50
        m_PI_lvl = (c_PI_signal * se_t_l*e_t_l 
            @#if LABOR_ONLY==0
                +c_PI_CI_signal * se_t_k*e_t_k *(clandaw*cwhlc*ccy) / ( cky*(crk-cdelta)) 
            @#endif
            + se_t_l*se_PI_signal * u_PI_signal) / (clandaw*cwhlc*ccy);
    % allow for the estimation using different observables 51
    @#if LABOR_ONLY==0
        m_CI_lvl = 0+(c_CI_signal * se_t_k*e_t_k + se_t_k*se_CI_signal * u_CI_signal)/ (cky*(crk-cdelta))+0; 
    @#endif
% ***********END OF NARRATIVE INSTRUMENTS******************************************

% ***********START OF EXOGENOUS PROCESSES******************************************
    % Exogenous processes: Residual GDP (net exports + inventories) 52
    nx= crhonx*(nx(-1)) + cnx_a*se_a*ea + cnx_b*se_b*eb + se_nx*enx;
    % Exogenous processes: TFP 53
    a = crhoa*a(-1)  + se_a*ea;
    % Exogenous processes: DF 54
    b = crhob*b(-1) + se_b*eb;
    % Exogenous processes: Government spending 55
    u_g= crhog*u_g(-1) + se_g*eg ;
    % Exogenous processes: Investment technology 56
    qs = crhoqs*qs(-1) + se_qs*eqs;
    % Exogenous processes: Monetary policy 57
    ms = crhoms*ms(-1) + se_m*em  + cov_m_g * eg + cov_m_t_l * e_t_l ;
    % Exogenous processes: Price markup 58
    spinf = crhopinf*spinf(-1) + se_pinf*epinf;
    % Exogenous processes: Wage markup 59
    sw = crhow*sw(-1) + se_w*ew;
    % Exogenous processes: Labor taxes 60
    u_t_l=crho_tl*u_t_l(-1) + se_t_l*e_t_l + cov_t_l_g * eg ;
    % Exogenous processes: Capital taxes 61
    u_t_k=crho_tk*u_t_k(-1) + se_t_k*e_t_k;
    % Exogenous processes: Consumption taxes 62
    u_t_c=crho_tc*u_t_c(-1) + se_t_c*e_t_c;
    % Exogenous processes: Transfers 63
    u_s=crho_s*u_s(-1) + se_s*e_s;

    % Exogenous processes: Forward looking G 64
    f_g = crho_f_g*f_g(-1)-(1-crho_f_g)*(cpsi_f_g_b*cby*debt +cpsi_f_g_y*y) + se_f_g*e_f_g ;
    % Exogenous processes: Forward looking Tax 65
    clandaw*cwhlc * ccy  *f_t_l = crho_f_t_l * clandaw*cwhlc * ccy  *f_t_l(-1) + (1-crho_f_t_l)*(cpsi_f_l_y*y + cpsi_f_l_b*cby*debt)  + clandaw*cwhlc * ccy  *se_f_t_l*e_f_t_l;
    % Exogenous processes: Forward looking productivity 66
    f_a =crho_f_a *f_a(-1) + se_f_a*e_f_a;
    % Exogenous processes: Forward guidance shocks 66
    f_m =crho_f_m *f_m(-1) + (1-crho_f_m) * 0 + se_f_m*e_f_m;
% ***********END OF EXOGENOUS PROCESSES******************************************

% ***********START OF EXPECTATIONS FORMATION*************************************
    % observation equation: future G 67
    Ex_g = gc(+4) + f_g; 
    % observation equation: future taxes 68
    Ex_dtau_l = dtau_l(+4) + f_t_l;
    % observation equation: future G 69
    Ex_y = y(+4); 
    % observation equation: future taxes 70
    Ex_c = c(+4);
    % observation equation: future taxes 70
    Ex_r = r(+4);
    % observation equation: future inflation 71
    Ex_inf = (pinf(+4)+pinf(+3)+pinf(+2)+pinf(+1))/4;
% ***********END OF EXOGENOUS PROCESSES******************************************
end; 

