function [residual, g1, g2, g3] = NK_narrative_v7_dynamic(y, x, params, steady_state, it_)
%
% Status : Computes dynamic model for Dynare
%
% Inputs :
%   y         [#dynamic variables by 1] double    vector of endogenous variables in the order stored
%                                                 in M_.lead_lag_incidence; see the Manual
%   x         [nperiods by M_.exo_nbr] double     matrix of exogenous variables (in declaration order)
%                                                 for all simulation periods
%   steady_state  [M_.endo_nbr by 1] double       vector of steady state values
%   params    [M_.param_nbr by 1] double          vector of parameter values in declaration order
%   it_       scalar double                       time period for exogenous variables for which to evaluate the model
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the dynamic model equations in order of 
%                                          declaration of the equations.
%                                          Dynare may prepend auxiliary equations, see M_.aux_vars
%   g1        [M_.endo_nbr by #dynamic variables] double    Jacobian matrix of the dynamic model equations;
%                                                           rows: equations in order of declaration
%                                                           columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%   g2        [M_.endo_nbr by (#dynamic variables)^2] double   Hessian matrix of the dynamic model equations;
%                                                              rows: equations in order of declaration
%                                                              columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%   g3        [M_.endo_nbr by (#dynamic variables)^3] double   Third order derivative matrix of the dynamic model equations;
%                                                              rows: equations in order of declaration
%                                                              columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

%
% Model equations
%

residual = zeros(99, 1);
cpie__ = 1+params(41)/100;
cgamma__ = 1+params(40)/100;
cbeta__ = 1/(1+params(39)/100);
cby__ = params(51);
clandap__ = params(7);
cbetabar__ = cbeta__*cgamma__^(-params(3));
cr__ = cpie__/(cbeta__*cgamma__^(-params(3)));
crk__ = (cbeta__^(-1)*cgamma__^params(3)-(1-params(44)*(1-params(43))))/(1-params(43));
cw__ = (params(4)^params(4)*(1-params(4))^(1-params(4))/(clandap__*crk__^params(4)))^(1/(1-params(4)));
cikbar__ = 1-(1-params(44))/cgamma__;
cik__ = cgamma__*(1-(1-params(44))/cgamma__);
clk__ = (1-params(4))/params(4)*crk__/cw__;
cky__ = params(7)*clk__^(params(4)-1);
ciy__ = cik__*cky__;
ccy__ = 1-params(52)-cik__*cky__;
cwhlc__ = cky__*crk__*(1-params(4))*1/params(47)/params(4)/ccy__;
crkky__ = crk__*cky__;
conster__ = 100*(cr__-1);
crhonx__ = params(94);
ctrendy__ = params(40);
ctrendinv__ = params(40);
ctrendw__ = params(40);
tbill__ = 0;
no_cap_u__ = 0;
cdelta1__ = crk__;
csadjcost_aux__ = 1/(cgamma__^2*params(2));
T125 = 1/(params(1)/(1-params(1)));
T157 = 1/(1+cgamma__*cbetabar__);
T176 = params(13)/cgamma__;
T181 = 1/((1-T176)/(params(3)*(1+T176)));
T190 = 1/(1-params(44)+(1-params(43))*crk__+params(44)*params(43));
T257 = 1/(1+cgamma__*cbetabar__*params(5));
T272 = (1-params(6))*(1-cgamma__*cbetabar__*params(6))/params(6)/(1+(params(7)-1)*params(8));
T281 = cgamma__*cbetabar__/(1+cgamma__*cbetabar__);
T308 = (1-params(10))*(1-cgamma__*cbetabar__*params(10))/((1+cgamma__*cbetabar__)*params(10))*1/(1+(params(47)-1)*params(11));
T462 = cby__/cr__;
T470 = y(5)+y(51)/params(43)+y(41)*crk__/(crk__-params(44));
T489 = 1/cpie__*cby__/cgamma__;
T654 = params(52)/4;
lhs =y(35);
rhs =params(4)*y(41)+(1-params(4))*(y(53)+y(54))-y(90)-y(30);
residual(1)= lhs-rhs;
lhs =y(40);
rhs =y(41)*T125;
residual(2)= lhs-rhs;
lhs =y(41);
rhs =y(53)+y(58)-y(42);
residual(3)= lhs-rhs;
lhs =y(42);
rhs =y(40)+y(5);
residual(4)= lhs-rhs;
lhs =y(43);
rhs =y(5)*(1-cikbar__)+cikbar__*y(44)-cdelta1__*y(40)*no_cap_u__/cgamma__+cikbar__/csadjcost_aux__*y(93);
residual(5)= lhs-rhs;
lhs =y(44);
rhs =y(93)+T157*(y(6)+cgamma__*cbetabar__*y(138)+csadjcost_aux__*y(45));
residual(6)= lhs-rhs;
lhs =y(45);
rhs =(-y(54))+y(141)-tbill__*y(91)+y(91)*T181*(1-tbill__)+T190*((1-params(43))*crk__*y(137)+(1-params(44))*y(139)-(crk__-params(44))*y(140));
residual(7)= lhs-rhs;
lhs =(1+T176)*params(3)*y(36);
rhs =params(3)*T176*y(1)+params(3)*y(134)+(params(3)-1)*cwhlc__*(y(58)-y(145))-(1-T176)*(tbill__*y(91)+y(54)-y(141)+y(146)-y(59))+y(91)*(1-tbill__);
residual(8)= lhs-rhs;
lhs =y(37);
rhs =y(40)*(1-no_cap_u__)*crkky__+y(39)+ccy__*y(36)+y(44)*ciy__+y(38)+y(31);
residual(9)= lhs-rhs;
lhs =y(37);
rhs =y(90)+params(7)*(params(4)*y(42)+(1-params(4))*y(58))+y(30);
residual(10)= lhs-rhs;
lhs =y(52);
rhs =T257*(cgamma__*cbetabar__*y(141)+params(5)*y(8)+y(35)*T272)+y(95);
residual(11)= lhs-rhs;
lhs =y(53);
rhs =y(96)+T157*y(9)+T281*y(142)+y(8)*params(9)/(1+cgamma__*cbetabar__)-y(52)*(1+cgamma__*cbetabar__*params(9))/(1+cgamma__*cbetabar__)+y(141)*T281+T308*(y(58)*params(12)+y(36)*1/(1-T176)-y(1)*T176/(1-T176)-y(53)+y(59)/(1+params(49))+(y(55)+y(32))/(1-params(46)));
residual(12)= lhs-rhs;
lhs =y(54);
rhs =(1-params(17))*(params(20)*y(109)+params(18)*y(107)+params(19)*(y(107)-y(37)))+y(52)*params(14)*(1-params(17))+y(37)*(1-params(17))*params(15)+params(16)*(y(37)-y(2))+params(17)*y(10)+y(94)+y(33);
residual(13)= lhs-rhs;
lhs =y(38);
rhs =params(21)*y(3)-(1-params(21))*(cby__*params(22)*y(57)+y(37)*params(23))+y(92);
residual(14)= lhs-rhs;
lhs =y(55)*ccy__*params(47)*cwhlc__;
rhs =ccy__*cwhlc__*params(47)*params(24)*y(11)+(1-params(24))*(y(37)*params(25)+y(57)*cby__*params(26))+params(27)*y(97)+params(28)*y(99)+y(98);
residual(15)= lhs-rhs;
lhs =cky__*(crk__-params(44))*y(51);
rhs =y(97)+(crk__-params(44))*cky__*params(29)*y(7)+(1-params(29))*(y(37)*params(30)+y(57)*cby__*params(31))+y(98)*params(32)+y(99)*params(33);
residual(16)= lhs-rhs;
lhs =y(56);
rhs =params(34)*y(12)-(1-params(34))*(y(39)+y(37)*params(35)+y(57)*cby__*params(36))+y(100);
residual(17)= lhs-rhs;
lhs =ccy__*y(59);
rhs =y(99)+y(97)*params(37)+y(98)*params(38);
residual(18)= lhs-rhs;
lhs =ccy__*params(49)*(y(36)+y(59)/params(49))+T462*(y(57)-y(54)-tbill__*y(91))+params(43)*cky__*(crk__-params(44))*T470+ccy__*cwhlc__*params(47)*params(46)*(y(58)+y(53)+(y(55)+y(32))/params(46));
rhs =T489*(y(13)-y(52))+y(56)+y(38)+y(31);
residual(19)= lhs-rhs;
lhs =y(60);
rhs =y(54)-y(141);
residual(20)= lhs-rhs;
lhs =y(61);
rhs =ccy__*cwhlc__*params(47)*params(46)*(y(58)+y(53)+(y(55)+y(32))/params(46));
residual(21)= lhs-rhs;
lhs =y(62);
rhs =y(136)+y(26);
residual(22)= lhs-rhs;
lhs =y(63);
rhs =y(37)-y(2)+ctrendy__;
residual(23)= lhs-rhs;
lhs =y(65);
rhs =ctrendy__+y(38)-y(3)+y(31)-y(34);
residual(24)= lhs-rhs;
lhs =y(64);
rhs =params(40)+y(36)-y(1);
residual(25)= lhs-rhs;
lhs =y(46);
rhs =y(44)-y(6)+ctrendinv__;
residual(26)= lhs-rhs;
lhs =y(66);
rhs =y(53)-y(9)+ctrendw__;
residual(27)= lhs-rhs;
lhs =y(67);
rhs =params(41)+y(52);
residual(28)= lhs-rhs;
lhs =y(68);
rhs =y(54)+conster__;
residual(29)= lhs-rhs;
lhs =y(69);
rhs =100*params(46)+y(55)+y(32);
residual(30)= lhs-rhs;
lhs =y(70);
rhs =100*params(46)+y(55)+y(32);
residual(31)= lhs-rhs;
lhs =y(71);
rhs =y(58)+params(42);
residual(32)= lhs-rhs;
lhs =y(47);
rhs =params(43)*cky__*(crk__-params(44))*(T470-y(37))+params(107)*x(it_, 17)+params(45);
residual(33)= lhs-rhs;
lhs =y(72);
rhs =params(48)+params(105)*x(it_, 15)+ccy__*cwhlc__*params(47)*params(46)*(y(58)+y(53)+(y(55)+y(32))/params(46)-y(37));
residual(34)= lhs-rhs;
lhs =y(73);
rhs =ccy__*params(49)*(y(36)+y(59)/params(49)-y(37))+params(50);
residual(35)= lhs-rhs;
lhs =y(74);
rhs =cby__*y(57)-cby__*y(37)+100*params(51)+params(111)*x(it_, 19);
residual(36)= lhs-rhs;
lhs =y(75);
rhs =params(111)*x(it_, 19)+y(57)-y(37)+100*log(params(51));
residual(37)= lhs-rhs;
lhs =y(76);
rhs =100*params(53)+100*log(params(52))+y(38)+y(31);
residual(38)= lhs-rhs;
lhs =y(77);
rhs =y(37)+100*params(53);
residual(39)= lhs-rhs;
lhs =y(78);
rhs =100*params(53)+y(36)+100*log(ccy__);
residual(40)= lhs-rhs;
lhs =y(48);
rhs =100*params(53)+y(44)+100*log(ciy__);
residual(41)= lhs-rhs;
lhs =y(79);
rhs =params(54)*x(it_, 4)+params(55)*x(it_, 13);
residual(42)= lhs-rhs;
lhs =y(80);
rhs =params(63)*(y(90)-y(15))+params(64)*x(it_, 12);
residual(43)= lhs-rhs;
lhs =y(81);
rhs =params(74)*x(it_, 11)+params(72)*(y(38)+y(31)-y(14));
residual(44)= lhs-rhs;
lhs =y(82);
rhs =(params(73)*(y(38)-y(14))/4+x(it_, 11)*params(74)*params(95))/T654;
residual(45)= lhs-rhs;
lhs =y(83);
rhs =params(85)*x(it_, 14)+params(82)*x(it_, 9);
residual(46)= lhs-rhs;
lhs =y(49);
rhs =x(it_, 14)*params(83)+params(86)*x(it_, 16)+params(84)*x(it_, 10);
residual(47)= lhs-rhs;
lhs =y(84);
rhs =y(79);
residual(48)= lhs-rhs;
lhs =y(85);
rhs =y(79);
residual(49)= lhs-rhs;
lhs =y(86);
rhs =y(79);
residual(50)= lhs-rhs;
lhs =y(87);
rhs =y(80);
residual(51)= lhs-rhs;
lhs =y(88);
rhs =y(81);
residual(52)= lhs-rhs;
lhs =y(89);
rhs =(x(it_, 14)*params(105)*params(85)+ccy__*params(47)*cwhlc__*x(it_, 16)*params(107)*params(83)/(cky__*(crk__-params(44)))+x(it_, 9)*params(105)*params(82))/(ccy__*params(47)*cwhlc__);
residual(53)= lhs-rhs;
lhs =y(50);
rhs =(x(it_, 16)*params(107)*params(86)+x(it_, 10)*params(107)*params(84))/(cky__*(crk__-params(44)));
residual(54)= lhs-rhs;
lhs =y(39);
rhs =crhonx__*y(4)+params(87)*params(91)*x(it_, 1)+params(88)*params(93)*x(it_, 2)+params(89)*x(it_, 8);
residual(55)= lhs-rhs;
lhs =y(90);
rhs =y(15)*params(90)+params(91)*x(it_, 1);
residual(56)= lhs-rhs;
lhs =y(91);
rhs =params(92)*y(16)+params(93)*x(it_, 2);
residual(57)= lhs-rhs;
lhs =y(92);
rhs =params(94)*y(17)+params(95)*x(it_, 7);
residual(58)= lhs-rhs;
lhs =y(93);
rhs =params(96)*y(18)+params(97)*x(it_, 3);
residual(59)= lhs-rhs;
lhs =y(94);
rhs =params(98)*y(19)+x(it_, 4)*params(99)+x(it_, 7)*params(125)+x(it_, 14)*params(126);
residual(60)= lhs-rhs;
lhs =y(95);
rhs =params(100)*y(20)+params(101)*x(it_, 5);
residual(61)= lhs-rhs;
lhs =y(96);
rhs =params(102)*y(21)+params(103)*x(it_, 6);
residual(62)= lhs-rhs;
lhs =y(98);
rhs =params(104)*y(23)+params(105)*x(it_, 14)+x(it_, 7)*params(124);
residual(63)= lhs-rhs;
lhs =y(97);
rhs =params(106)*y(22)+params(107)*x(it_, 16);
residual(64)= lhs-rhs;
lhs =y(99);
rhs =params(108)*y(24)+params(109)*x(it_, 20);
residual(65)= lhs-rhs;
lhs =y(100);
rhs =params(110)*y(25)+params(111)*x(it_, 18);
residual(66)= lhs-rhs;
lhs =y(101);
rhs =y(26)*params(114)-(1-params(114))*(y(57)*cby__*params(120)+y(37)*params(121))+params(115)*x(it_, 21);
residual(67)= lhs-rhs;
lhs =ccy__*params(47)*cwhlc__*y(102);
rhs =ccy__*cwhlc__*params(47)*params(112)*y(27)+(1-params(112))*(y(37)*params(123)+y(57)*cby__*params(122))+ccy__*params(47)*cwhlc__*params(113)*x(it_, 22);
residual(68)= lhs-rhs;
lhs =y(103);
rhs =params(116)*y(28)+params(117)*x(it_, 23);
residual(69)= lhs-rhs;
lhs =y(104);
rhs =params(118)*y(29)+params(119)*x(it_, 24);
residual(70)= lhs-rhs;
lhs =y(105);
rhs =y(101)+y(149);
residual(71)= lhs-rhs;
lhs =y(106);
rhs =y(102)+y(152);
residual(72)= lhs-rhs;
lhs =y(107);
rhs =y(155);
residual(73)= lhs-rhs;
lhs =y(108);
rhs =y(158);
residual(74)= lhs-rhs;
lhs =y(110);
rhs =y(161);
residual(75)= lhs-rhs;
lhs =y(109);
rhs =(y(141)+y(162)+y(163)+y(164))/4;
residual(76)= lhs-rhs;
lhs =y(111);
rhs =y(136);
residual(77)= lhs-rhs;
lhs =y(112);
rhs =y(147);
residual(78)= lhs-rhs;
lhs =y(113);
rhs =y(148);
residual(79)= lhs-rhs;
lhs =y(114);
rhs =y(144);
residual(80)= lhs-rhs;
lhs =y(115);
rhs =y(150);
residual(81)= lhs-rhs;
lhs =y(116);
rhs =y(151);
residual(82)= lhs-rhs;
lhs =y(117);
rhs =y(135);
residual(83)= lhs-rhs;
lhs =y(118);
rhs =y(153);
residual(84)= lhs-rhs;
lhs =y(119);
rhs =y(154);
residual(85)= lhs-rhs;
lhs =y(120);
rhs =y(134);
residual(86)= lhs-rhs;
lhs =y(121);
rhs =y(156);
residual(87)= lhs-rhs;
lhs =y(122);
rhs =y(157);
residual(88)= lhs-rhs;
lhs =y(123);
rhs =y(143);
residual(89)= lhs-rhs;
lhs =y(124);
rhs =y(159);
residual(90)= lhs-rhs;
lhs =y(125);
rhs =y(160);
residual(91)= lhs-rhs;
lhs =y(126);
rhs =y(141);
residual(92)= lhs-rhs;
lhs =y(127);
rhs =y(162);
residual(93)= lhs-rhs;
lhs =y(128);
rhs =y(163);
residual(94)= lhs-rhs;
lhs =y(129);
rhs =y(28);
residual(95)= lhs-rhs;
lhs =y(130);
rhs =y(26);
residual(96)= lhs-rhs;
lhs =y(131);
rhs =y(27);
residual(97)= lhs-rhs;
lhs =y(132);
rhs =y(29);
residual(98)= lhs-rhs;
lhs =y(133);
rhs =y(31);
residual(99)= lhs-rhs;
if nargout >= 2,
  g1 = zeros(99, 188);

  %
  % Jacobian matrix
  %

T3 = (-1);
T1127 = ccy__*cwhlc__*params(47)*params(46)*1/params(46);
  g1(1,35)=1;
  g1(1,41)=(-params(4));
  g1(1,53)=(-(1-params(4)));
  g1(1,54)=(-(1-params(4)));
  g1(1,90)=1;
  g1(1,30)=1;
  g1(2,40)=1;
  g1(2,41)=(-T125);
  g1(3,41)=1;
  g1(3,42)=1;
  g1(3,53)=T3;
  g1(3,58)=T3;
  g1(4,40)=T3;
  g1(4,42)=1;
  g1(4,5)=T3;
  g1(5,40)=cdelta1__*no_cap_u__/cgamma__;
  g1(5,5)=(-(1-cikbar__));
  g1(5,43)=1;
  g1(5,44)=(-cikbar__);
  g1(5,93)=(-(cikbar__/csadjcost_aux__));
  g1(6,6)=(-T157);
  g1(6,44)=1;
  g1(6,138)=(-(cgamma__*cbetabar__*T157));
  g1(6,45)=(-(csadjcost_aux__*T157));
  g1(6,93)=T3;
  g1(7,137)=(-((1-params(43))*crk__*T190));
  g1(7,45)=1;
  g1(7,139)=(-((1-params(44))*T190));
  g1(7,140)=(-(T190*(-(crk__-params(44)))));
  g1(7,141)=T3;
  g1(7,54)=1;
  g1(7,91)=(-((-tbill__)+T181*(1-tbill__)));
  g1(8,1)=(-(params(3)*T176));
  g1(8,36)=params(3)*(1+T176);
  g1(8,134)=(-params(3));
  g1(8,141)=(-(1-T176));
  g1(8,54)=1-T176;
  g1(8,58)=(-((params(3)-1)*cwhlc__));
  g1(8,145)=(params(3)-1)*cwhlc__;
  g1(8,59)=(-(1-T176));
  g1(8,146)=1-T176;
  g1(8,91)=(-(1-tbill__+(-(tbill__*(1-T176)))));
  g1(9,36)=(-ccy__);
  g1(9,37)=1;
  g1(9,38)=T3;
  g1(9,39)=T3;
  g1(9,40)=(-((1-no_cap_u__)*crkky__));
  g1(9,44)=(-ciy__);
  g1(9,31)=T3;
  g1(10,37)=1;
  g1(10,42)=(-(params(7)*params(4)));
  g1(10,58)=(-(params(7)*(1-params(4))));
  g1(10,90)=T3;
  g1(10,30)=T3;
  g1(11,35)=(-(T257*T272));
  g1(11,8)=(-(params(5)*T257));
  g1(11,52)=1;
  g1(11,141)=(-(cgamma__*cbetabar__*T257));
  g1(11,95)=T3;
  g1(12,1)=(-(T308*(-(T176/(1-T176)))));
  g1(12,36)=(-(T308*1/(1-T176)));
  g1(12,8)=(-(params(9)/(1+cgamma__*cbetabar__)));
  g1(12,52)=(1+cgamma__*cbetabar__*params(9))/(1+cgamma__*cbetabar__);
  g1(12,141)=(-T281);
  g1(12,9)=(-T157);
  g1(12,53)=1-(-T308);
  g1(12,142)=(-T281);
  g1(12,55)=(-(T308*1/(1-params(46))));
  g1(12,58)=(-(T308*params(12)));
  g1(12,59)=(-(T308*1/(1+params(49))));
  g1(12,96)=T3;
  g1(12,32)=(-(T308*1/(1-params(46))));
  g1(13,2)=params(16);
  g1(13,37)=(-((1-params(17))*(-params(19))+(1-params(17))*params(15)+params(16)));
  g1(13,52)=(-(params(14)*(1-params(17))));
  g1(13,10)=(-params(17));
  g1(13,54)=1;
  g1(13,94)=T3;
  g1(13,107)=(-((1-params(17))*(params(18)+params(19))));
  g1(13,109)=(-((1-params(17))*params(20)));
  g1(13,33)=T3;
  g1(14,37)=(1-params(21))*params(23);
  g1(14,3)=(-params(21));
  g1(14,38)=1;
  g1(14,57)=(1-params(21))*cby__*params(22);
  g1(14,92)=T3;
  g1(15,37)=(-((1-params(24))*params(25)));
  g1(15,11)=(-(ccy__*cwhlc__*params(47)*params(24)));
  g1(15,55)=ccy__*params(47)*cwhlc__;
  g1(15,57)=(-((1-params(24))*cby__*params(26)));
  g1(15,97)=(-params(27));
  g1(15,98)=T3;
  g1(15,99)=(-params(28));
  g1(16,37)=(-((1-params(29))*params(30)));
  g1(16,7)=(-((crk__-params(44))*cky__*params(29)));
  g1(16,51)=cky__*(crk__-params(44));
  g1(16,57)=(-((1-params(29))*cby__*params(31)));
  g1(16,97)=T3;
  g1(16,98)=(-params(32));
  g1(16,99)=(-params(33));
  g1(17,37)=(1-params(34))*params(35);
  g1(17,39)=1-params(34);
  g1(17,12)=(-params(34));
  g1(17,56)=1;
  g1(17,57)=(1-params(34))*cby__*params(36);
  g1(17,100)=T3;
  g1(18,59)=ccy__;
  g1(18,97)=(-params(37));
  g1(18,98)=(-params(38));
  g1(18,99)=T3;
  g1(19,36)=ccy__*params(49);
  g1(19,38)=T3;
  g1(19,41)=params(43)*cky__*(crk__-params(44))*crk__/(crk__-params(44));
  g1(19,5)=params(43)*cky__*(crk__-params(44));
  g1(19,51)=params(43)*cky__*(crk__-params(44))*1/params(43);
  g1(19,52)=T489;
  g1(19,53)=ccy__*cwhlc__*params(47)*params(46);
  g1(19,54)=(-T462);
  g1(19,55)=T1127;
  g1(19,56)=T3;
  g1(19,13)=(-T489);
  g1(19,57)=T462;
  g1(19,58)=ccy__*cwhlc__*params(47)*params(46);
  g1(19,59)=ccy__*params(49)*1/params(49);
  g1(19,91)=T462*(-tbill__);
  g1(19,31)=T3;
  g1(19,32)=T1127;
  g1(20,141)=1;
  g1(20,54)=T3;
  g1(20,60)=1;
  g1(21,53)=(-(ccy__*cwhlc__*params(47)*params(46)));
  g1(21,55)=(-T1127);
  g1(21,58)=(-(ccy__*cwhlc__*params(47)*params(46)));
  g1(21,61)=1;
  g1(21,32)=(-T1127);
  g1(22,136)=T3;
  g1(22,62)=1;
  g1(22,26)=T3;
  g1(23,2)=1;
  g1(23,37)=T3;
  g1(23,63)=1;
  g1(24,3)=1;
  g1(24,38)=T3;
  g1(24,65)=1;
  g1(24,31)=T3;
  g1(24,34)=1;
  g1(25,1)=1;
  g1(25,36)=T3;
  g1(25,64)=1;
  g1(26,6)=1;
  g1(26,44)=T3;
  g1(26,46)=1;
  g1(27,9)=1;
  g1(27,53)=T3;
  g1(27,66)=1;
  g1(28,52)=T3;
  g1(28,67)=1;
  g1(29,54)=T3;
  g1(29,68)=1;
  g1(30,55)=T3;
  g1(30,69)=1;
  g1(30,32)=T3;
  g1(31,55)=T3;
  g1(31,70)=1;
  g1(31,32)=T3;
  g1(32,58)=T3;
  g1(32,71)=1;
  g1(33,37)=params(43)*cky__*(crk__-params(44));
  g1(33,41)=(-(params(43)*cky__*(crk__-params(44))*crk__/(crk__-params(44))));
  g1(33,5)=(-(params(43)*cky__*(crk__-params(44))));
  g1(33,47)=1;
  g1(33,51)=(-(params(43)*cky__*(crk__-params(44))*1/params(43)));
  g1(33,181)=(-params(107));
  g1(34,37)=ccy__*cwhlc__*params(47)*params(46);
  g1(34,53)=(-(ccy__*cwhlc__*params(47)*params(46)));
  g1(34,55)=(-T1127);
  g1(34,58)=(-(ccy__*cwhlc__*params(47)*params(46)));
  g1(34,72)=1;
  g1(34,179)=(-params(105));
  g1(34,32)=(-T1127);
  g1(35,36)=(-(ccy__*params(49)));
  g1(35,37)=ccy__*params(49);
  g1(35,59)=(-(ccy__*params(49)*1/params(49)));
  g1(35,73)=1;
  g1(36,37)=cby__;
  g1(36,57)=(-cby__);
  g1(36,74)=1;
  g1(36,183)=(-params(111));
  g1(37,37)=1;
  g1(37,57)=T3;
  g1(37,75)=1;
  g1(37,183)=(-params(111));
  g1(38,38)=T3;
  g1(38,76)=1;
  g1(38,31)=T3;
  g1(39,37)=T3;
  g1(39,77)=1;
  g1(40,36)=T3;
  g1(40,78)=1;
  g1(41,44)=T3;
  g1(41,48)=1;
  g1(42,79)=1;
  g1(42,168)=(-params(54));
  g1(42,177)=(-params(55));
  g1(43,80)=1;
  g1(43,15)=params(63);
  g1(43,90)=(-params(63));
  g1(43,176)=(-params(64));
  g1(44,38)=(-params(72));
  g1(44,14)=params(72);
  g1(44,81)=1;
  g1(44,175)=(-params(74));
  g1(44,31)=(-params(72));
  g1(45,38)=(-(params(73)/4/T654));
  g1(45,14)=(-((-params(73))/4/T654));
  g1(45,82)=1;
  g1(45,175)=(-(params(74)*params(95)/T654));
  g1(46,83)=1;
  g1(46,173)=(-params(82));
  g1(46,178)=(-params(85));
  g1(47,49)=1;
  g1(47,174)=(-params(84));
  g1(47,178)=(-params(83));
  g1(47,180)=(-params(86));
  g1(48,79)=T3;
  g1(48,84)=1;
  g1(49,79)=T3;
  g1(49,85)=1;
  g1(50,79)=T3;
  g1(50,86)=1;
  g1(51,80)=T3;
  g1(51,87)=1;
  g1(52,81)=T3;
  g1(52,88)=1;
  g1(53,89)=1;
  g1(53,173)=(-(params(105)*params(82)/(ccy__*params(47)*cwhlc__)));
  g1(53,178)=(-(params(105)*params(85)/(ccy__*params(47)*cwhlc__)));
  g1(53,180)=(-(ccy__*params(47)*cwhlc__*params(107)*params(83)/(cky__*(crk__-params(44)))/(ccy__*params(47)*cwhlc__)));
  g1(54,50)=1;
  g1(54,174)=(-(params(107)*params(84)/(cky__*(crk__-params(44)))));
  g1(54,180)=(-(params(107)*params(86)/(cky__*(crk__-params(44)))));
  g1(55,4)=(-crhonx__);
  g1(55,39)=1;
  g1(55,165)=(-(params(87)*params(91)));
  g1(55,166)=(-(params(88)*params(93)));
  g1(55,172)=(-params(89));
  g1(56,15)=(-params(90));
  g1(56,90)=1;
  g1(56,165)=(-params(91));
  g1(57,16)=(-params(92));
  g1(57,91)=1;
  g1(57,166)=(-params(93));
  g1(58,17)=(-params(94));
  g1(58,92)=1;
  g1(58,171)=(-params(95));
  g1(59,18)=(-params(96));
  g1(59,93)=1;
  g1(59,167)=(-params(97));
  g1(60,19)=(-params(98));
  g1(60,94)=1;
  g1(60,168)=(-params(99));
  g1(60,171)=(-params(125));
  g1(60,178)=(-params(126));
  g1(61,20)=(-params(100));
  g1(61,95)=1;
  g1(61,169)=(-params(101));
  g1(62,21)=(-params(102));
  g1(62,96)=1;
  g1(62,170)=(-params(103));
  g1(63,23)=(-params(104));
  g1(63,98)=1;
  g1(63,171)=(-params(124));
  g1(63,178)=(-params(105));
  g1(64,22)=(-params(106));
  g1(64,97)=1;
  g1(64,180)=(-params(107));
  g1(65,24)=(-params(108));
  g1(65,99)=1;
  g1(65,184)=(-params(109));
  g1(66,25)=(-params(110));
  g1(66,100)=1;
  g1(66,182)=(-params(111));
  g1(67,37)=(1-params(114))*params(121);
  g1(67,57)=(1-params(114))*cby__*params(120);
  g1(67,26)=(-params(114));
  g1(67,101)=1;
  g1(67,185)=(-params(115));
  g1(68,37)=(-((1-params(112))*params(123)));
  g1(68,57)=(-((1-params(112))*cby__*params(122)));
  g1(68,27)=(-(ccy__*cwhlc__*params(47)*params(112)));
  g1(68,102)=ccy__*params(47)*cwhlc__;
  g1(68,186)=(-(ccy__*params(47)*cwhlc__*params(113)));
  g1(69,28)=(-params(116));
  g1(69,103)=1;
  g1(69,187)=(-params(117));
  g1(70,29)=(-params(118));
  g1(70,104)=1;
  g1(70,188)=(-params(119));
  g1(71,101)=T3;
  g1(71,105)=1;
  g1(71,149)=T3;
  g1(72,102)=T3;
  g1(72,106)=1;
  g1(72,152)=T3;
  g1(73,107)=1;
  g1(73,155)=T3;
  g1(74,108)=1;
  g1(74,158)=T3;
  g1(75,110)=1;
  g1(75,161)=T3;
  g1(76,141)=(-0.25);
  g1(76,109)=1;
  g1(76,162)=(-0.25);
  g1(76,163)=(-0.25);
  g1(76,164)=(-0.25);
  g1(77,136)=T3;
  g1(77,111)=1;
  g1(78,147)=T3;
  g1(78,112)=1;
  g1(79,148)=T3;
  g1(79,113)=1;
  g1(80,144)=T3;
  g1(80,114)=1;
  g1(81,150)=T3;
  g1(81,115)=1;
  g1(82,151)=T3;
  g1(82,116)=1;
  g1(83,135)=T3;
  g1(83,117)=1;
  g1(84,153)=T3;
  g1(84,118)=1;
  g1(85,154)=T3;
  g1(85,119)=1;
  g1(86,134)=T3;
  g1(86,120)=1;
  g1(87,156)=T3;
  g1(87,121)=1;
  g1(88,157)=T3;
  g1(88,122)=1;
  g1(89,143)=T3;
  g1(89,123)=1;
  g1(90,159)=T3;
  g1(90,124)=1;
  g1(91,160)=T3;
  g1(91,125)=1;
  g1(92,141)=T3;
  g1(92,126)=1;
  g1(93,162)=T3;
  g1(93,127)=1;
  g1(94,163)=T3;
  g1(94,128)=1;
  g1(95,28)=T3;
  g1(95,129)=1;
  g1(96,26)=T3;
  g1(96,130)=1;
  g1(97,27)=T3;
  g1(97,131)=1;
  g1(98,29)=T3;
  g1(98,132)=1;
  g1(99,31)=T3;
  g1(99,133)=1;

if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],99,35344);
if nargout >= 4,
  %
  % Third order derivatives
  %

  g3 = sparse([],[],[],99,6644672);
end
end
end
end
