function [residual, g1, g2, g3] = NK_narrative_v7_static(y, x, params)
%
% Status : Computes static model for Dynare
%
% Inputs : 
%   y         [M_.endo_nbr by 1] double    vector of endogenous variables in declaration order
%   x         [M_.exo_nbr by 1] double     vector of exogenous variables in declaration order
%   params    [M_.param_nbr by 1] double   vector of parameter values in declaration order
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the static model equations 
%                                          in order of declaration of the equations.
%                                          Dynare may prepend or append auxiliary equations, see M_.aux_vars
%   g1        [M_.endo_nbr by M_.endo_nbr] double    Jacobian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%   g2        [M_.endo_nbr by (M_.endo_nbr)^2] double   Hessian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%   g3        [M_.endo_nbr by (M_.endo_nbr)^3] double   Third derivatives matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

residual = zeros( 99, 1);

%
% Model equations
%

cpie__ = 1+params(41)/100;
cgamma__ = 1+params(40)/100;
cbeta__ = 1/(1+params(39)/100);
cby__ = params(51);
clandap__ = params(7);
cbetabar__ = cbeta__*cgamma__^(-params(3));
cr__ = cpie__/(cbeta__*cgamma__^(-params(3)));
crk__ = (cbeta__^(-1)*cgamma__^params(3)-(1-params(44)*(1-params(43))))/(1-params(43));
cw__ = (params(4)^params(4)*(1-params(4))^(1-params(4))/(clandap__*crk__^params(4)))^(1/(1-params(4)));
cikbar__ = 1-(1-params(44))/cgamma__;
cik__ = cgamma__*(1-(1-params(44))/cgamma__);
clk__ = (1-params(4))/params(4)*crk__/cw__;
cky__ = params(7)*clk__^(params(4)-1);
ciy__ = cik__*cky__;
ccy__ = 1-params(52)-cik__*cky__;
cwhlc__ = cky__*crk__*(1-params(4))*1/params(47)/params(4)/ccy__;
crkky__ = crk__*cky__;
conster__ = 100*(cr__-1);
crhonx__ = params(94);
ctrendy__ = params(40);
ctrendinv__ = params(40);
ctrendw__ = params(40);
tbill__ = 0;
no_cap_u__ = 0;
cdelta1__ = crk__;
csadjcost_aux__ = 1/(cgamma__^2*params(2));
T124 = 1/(params(1)/(1-params(1)));
T155 = 1/(1+cgamma__*cbetabar__);
T172 = params(13)/cgamma__;
T177 = 1/((1-T172)/(params(3)*(1+T172)));
T186 = 1/(1-params(44)+(1-params(43))*crk__+params(44)*params(43));
T242 = 1/(1+cgamma__*cbetabar__*params(5));
T256 = (1-params(6))*(1-cgamma__*cbetabar__*params(6))/params(6)/(1+(params(7)-1)*params(8));
T265 = cgamma__*cbetabar__/(1+cgamma__*cbetabar__);
T291 = (1-params(10))*(1-cgamma__*cbetabar__*params(10))/((1+cgamma__*cbetabar__)*params(10))*1/(1+(params(47)-1)*params(11));
T437 = cby__/cr__;
T445 = y(9)+y(17)/params(43)+y(7)*crk__/(crk__-params(44));
T459 = 1/cpie__*cby__/cgamma__;
lhs =y(1);
rhs =params(4)*y(7)+(1-params(4))*(y(19)+y(20))-y(56)-y(69);
residual(1)= lhs-rhs;
lhs =y(6);
rhs =y(7)*T124;
residual(2)= lhs-rhs;
lhs =y(7);
rhs =y(19)+y(24)-y(8);
residual(3)= lhs-rhs;
lhs =y(8);
rhs =y(6)+y(9);
residual(4)= lhs-rhs;
lhs =y(9);
rhs =y(9)*(1-cikbar__)+cikbar__*y(10)-cdelta1__*y(6)*no_cap_u__/cgamma__+cikbar__/csadjcost_aux__*y(59);
residual(5)= lhs-rhs;
lhs =y(10);
rhs =y(59)+T155*(y(10)+y(10)*cgamma__*cbetabar__+csadjcost_aux__*y(11));
residual(6)= lhs-rhs;
lhs =y(11);
rhs =(-y(20))+y(18)-tbill__*y(57)+y(57)*T177*(1-tbill__)+T186*(y(7)*(1-params(43))*crk__+(1-params(44))*y(11)-(crk__-params(44))*y(17));
residual(7)= lhs-rhs;
lhs =(1+T172)*params(3)*y(2);
rhs =params(3)*y(2)+y(2)*params(3)*T172-(1-T172)*(tbill__*y(57)+y(20)-y(18)+y(25)-y(25))+y(57)*(1-tbill__);
residual(8)= lhs-rhs;
lhs =y(3);
rhs =y(6)*(1-no_cap_u__)*crkky__+y(5)+ccy__*y(2)+y(10)*ciy__+y(4)+y(67);
residual(9)= lhs-rhs;
lhs =y(3);
rhs =y(56)+params(7)*(params(4)*y(8)+(1-params(4))*y(24))+y(69);
residual(10)= lhs-rhs;
lhs =y(18);
rhs =T242*(cgamma__*cbetabar__*y(18)+y(18)*params(5)+y(1)*T256)+y(61);
residual(11)= lhs-rhs;
lhs =y(19);
rhs =y(62)+y(19)*T155+y(19)*T265+y(18)*params(9)/(1+cgamma__*cbetabar__)-y(18)*(1+cgamma__*cbetabar__*params(9))/(1+cgamma__*cbetabar__)+y(18)*T265+T291*(y(24)*params(12)+y(2)*1/(1-T172)-y(2)*T172/(1-T172)-y(19)+y(25)/(1+params(49))+(y(21)+y(68))/(1-params(46)));
residual(12)= lhs-rhs;
lhs =y(20);
rhs =(1-params(17))*(params(20)*y(75)+params(18)*y(73)+params(19)*(y(73)-y(3)))+y(18)*(1-params(17))*params(14)+y(3)*(1-params(17))*params(15)+y(20)*params(17)+y(60)+y(70);
residual(13)= lhs-rhs;
lhs =y(4);
rhs =y(4)*params(21)-(1-params(21))*(cby__*params(22)*y(23)+y(3)*params(23))+y(58);
residual(14)= lhs-rhs;
lhs =y(21)*ccy__*params(47)*cwhlc__;
rhs =y(21)*ccy__*cwhlc__*params(47)*params(24)+(1-params(24))*(y(3)*params(25)+y(23)*cby__*params(26))+params(27)*y(63)+params(28)*y(65)+y(64);
residual(15)= lhs-rhs;
lhs =y(17)*cky__*(crk__-params(44));
rhs =y(63)+y(17)*(crk__-params(44))*cky__*params(29)+(1-params(29))*(y(3)*params(30)+y(23)*cby__*params(31))+y(64)*params(32)+y(65)*params(33);
residual(16)= lhs-rhs;
lhs =y(22);
rhs =y(22)*params(34)-(1-params(34))*(y(5)+y(3)*params(35)+y(23)*cby__*params(36))+y(66);
residual(17)= lhs-rhs;
lhs =ccy__*y(25);
rhs =y(65)+y(63)*params(37)+y(64)*params(38);
residual(18)= lhs-rhs;
lhs =ccy__*params(49)*(y(2)+y(25)/params(49))+T437*(y(23)-y(20)-tbill__*y(57))+params(43)*cky__*(crk__-params(44))*T445+ccy__*cwhlc__*params(47)*params(46)*(y(24)+y(19)+(y(21)+y(68))/params(46));
rhs =T459*(y(23)-y(18))+y(22)+y(4)+y(67);
residual(19)= lhs-rhs;
lhs =y(26);
rhs =y(20)-y(18);
residual(20)= lhs-rhs;
lhs =y(27);
rhs =ccy__*cwhlc__*params(47)*params(46)*(y(24)+y(19)+(y(21)+y(68))/params(46));
residual(21)= lhs-rhs;
lhs =y(28);
rhs =y(4)+y(67);
residual(22)= lhs-rhs;
lhs =y(29);
rhs =ctrendy__;
residual(23)= lhs-rhs;
lhs =y(31);
rhs =ctrendy__;
residual(24)= lhs-rhs;
lhs =y(30);
rhs =params(40);
residual(25)= lhs-rhs;
lhs =y(12);
rhs =ctrendinv__;
residual(26)= lhs-rhs;
lhs =y(32);
rhs =ctrendw__;
residual(27)= lhs-rhs;
lhs =y(33);
rhs =params(41)+y(18);
residual(28)= lhs-rhs;
lhs =y(34);
rhs =y(20)+conster__;
residual(29)= lhs-rhs;
lhs =y(35);
rhs =100*params(46)+y(21)+y(68);
residual(30)= lhs-rhs;
lhs =y(36);
rhs =100*params(46)+y(21)+y(68);
residual(31)= lhs-rhs;
lhs =y(37);
rhs =y(24)+params(42);
residual(32)= lhs-rhs;
lhs =y(13);
rhs =params(43)*cky__*(crk__-params(44))*(T445-y(3))+params(107)*x(17)+params(45);
residual(33)= lhs-rhs;
lhs =y(38);
rhs =params(48)+params(105)*x(15)+ccy__*cwhlc__*params(47)*params(46)*(y(24)+y(19)+(y(21)+y(68))/params(46)-y(3));
residual(34)= lhs-rhs;
lhs =y(39);
rhs =ccy__*params(49)*(y(2)+y(25)/params(49)-y(3))+params(50);
residual(35)= lhs-rhs;
lhs =y(40);
rhs =cby__*y(23)-cby__*y(3)+100*params(51)+params(111)*x(19);
residual(36)= lhs-rhs;
lhs =y(41);
rhs =params(111)*x(19)+y(23)-y(3)+100*log(params(51));
residual(37)= lhs-rhs;
lhs =y(42);
rhs =100*params(53)+y(4)+y(67)+100*log(params(52));
residual(38)= lhs-rhs;
lhs =y(43);
rhs =y(3)+100*params(53);
residual(39)= lhs-rhs;
lhs =y(44);
rhs =100*params(53)+y(2)+100*log(ccy__);
residual(40)= lhs-rhs;
lhs =y(14);
rhs =100*params(53)+y(10)+100*log(ciy__);
residual(41)= lhs-rhs;
lhs =y(45);
rhs =params(54)*x(4)+params(55)*x(13);
residual(42)= lhs-rhs;
lhs =y(46);
rhs =params(64)*x(12);
residual(43)= lhs-rhs;
lhs =y(47);
rhs =params(74)*x(11)+params(72)*(y(4)+y(67)-y(28));
residual(44)= lhs-rhs;
lhs =y(48);
rhs =(params(73)*(y(4)-y(28))/4+x(11)*params(74)*params(95))/(params(52)/4);
residual(45)= lhs-rhs;
lhs =y(49);
rhs =params(85)*x(14)+params(82)*x(9);
residual(46)= lhs-rhs;
lhs =y(15);
rhs =x(14)*params(83)+params(86)*x(16)+params(84)*x(10);
residual(47)= lhs-rhs;
lhs =y(50);
rhs =y(45);
residual(48)= lhs-rhs;
lhs =y(51);
rhs =y(45);
residual(49)= lhs-rhs;
lhs =y(52);
rhs =y(45);
residual(50)= lhs-rhs;
lhs =y(53);
rhs =y(46);
residual(51)= lhs-rhs;
lhs =y(54);
rhs =y(47);
residual(52)= lhs-rhs;
lhs =y(55);
rhs =(x(14)*params(105)*params(85)+ccy__*params(47)*cwhlc__*x(16)*params(107)*params(83)/(cky__*(crk__-params(44)))+x(9)*params(105)*params(82))/(ccy__*params(47)*cwhlc__);
residual(53)= lhs-rhs;
lhs =y(16);
rhs =(x(16)*params(107)*params(86)+x(10)*params(107)*params(84))/(cky__*(crk__-params(44)));
residual(54)= lhs-rhs;
lhs =y(5);
rhs =y(5)*crhonx__+params(87)*params(91)*x(1)+params(88)*params(93)*x(2)+params(89)*x(8);
residual(55)= lhs-rhs;
lhs =y(56);
rhs =y(56)*params(90)+params(91)*x(1);
residual(56)= lhs-rhs;
lhs =y(57);
rhs =y(57)*params(92)+params(93)*x(2);
residual(57)= lhs-rhs;
lhs =y(58);
rhs =params(94)*y(58)+params(95)*x(7);
residual(58)= lhs-rhs;
lhs =y(59);
rhs =y(59)*params(96)+params(97)*x(3);
residual(59)= lhs-rhs;
lhs =y(60);
rhs =y(60)*params(98)+x(4)*params(99)+x(7)*params(125)+x(14)*params(126);
residual(60)= lhs-rhs;
lhs =y(61);
rhs =y(61)*params(100)+params(101)*x(5);
residual(61)= lhs-rhs;
lhs =y(62);
rhs =y(62)*params(102)+params(103)*x(6);
residual(62)= lhs-rhs;
lhs =y(64);
rhs =y(64)*params(104)+params(105)*x(14)+x(7)*params(124);
residual(63)= lhs-rhs;
lhs =y(63);
rhs =y(63)*params(106)+params(107)*x(16);
residual(64)= lhs-rhs;
lhs =y(65);
rhs =y(65)*params(108)+params(109)*x(20);
residual(65)= lhs-rhs;
lhs =y(66);
rhs =y(66)*params(110)+params(111)*x(18);
residual(66)= lhs-rhs;
lhs =y(67);
rhs =y(67)*params(114)-(1-params(114))*(y(23)*cby__*params(120)+y(3)*params(121))+params(115)*x(21);
residual(67)= lhs-rhs;
lhs =ccy__*params(47)*cwhlc__*y(68);
rhs =y(68)*ccy__*cwhlc__*params(47)*params(112)+(1-params(112))*(y(3)*params(123)+y(23)*cby__*params(122))+ccy__*params(47)*cwhlc__*params(113)*x(22);
residual(68)= lhs-rhs;
lhs =y(69);
rhs =y(69)*params(116)+params(117)*x(23);
residual(69)= lhs-rhs;
lhs =y(70);
rhs =y(70)*params(118)+params(119)*x(24);
residual(70)= lhs-rhs;
lhs =y(71);
rhs =y(4)+y(67);
residual(71)= lhs-rhs;
lhs =y(72);
rhs =y(21)+y(68);
residual(72)= lhs-rhs;
lhs =y(73);
rhs =y(3);
residual(73)= lhs-rhs;
lhs =y(74);
rhs =y(2);
residual(74)= lhs-rhs;
lhs =y(76);
rhs =y(20);
residual(75)= lhs-rhs;
lhs =y(75);
rhs =(y(18)+y(18)+y(18)+y(18))/4;
residual(76)= lhs-rhs;
lhs =y(77);
rhs =y(4);
residual(77)= lhs-rhs;
lhs =y(78);
rhs =y(4);
residual(78)= lhs-rhs;
lhs =y(79);
rhs =y(4);
residual(79)= lhs-rhs;
lhs =y(80);
rhs =y(21);
residual(80)= lhs-rhs;
lhs =y(81);
rhs =y(21);
residual(81)= lhs-rhs;
lhs =y(82);
rhs =y(21);
residual(82)= lhs-rhs;
lhs =y(83);
rhs =y(3);
residual(83)= lhs-rhs;
lhs =y(84);
rhs =y(3);
residual(84)= lhs-rhs;
lhs =y(85);
rhs =y(3);
residual(85)= lhs-rhs;
lhs =y(86);
rhs =y(2);
residual(86)= lhs-rhs;
lhs =y(87);
rhs =y(2);
residual(87)= lhs-rhs;
lhs =y(88);
rhs =y(2);
residual(88)= lhs-rhs;
lhs =y(89);
rhs =y(20);
residual(89)= lhs-rhs;
lhs =y(90);
rhs =y(20);
residual(90)= lhs-rhs;
lhs =y(91);
rhs =y(20);
residual(91)= lhs-rhs;
lhs =y(92);
rhs =y(18);
residual(92)= lhs-rhs;
lhs =y(93);
rhs =y(18);
residual(93)= lhs-rhs;
lhs =y(94);
rhs =y(18);
residual(94)= lhs-rhs;
lhs =y(95);
rhs =y(69);
residual(95)= lhs-rhs;
lhs =y(96);
rhs =y(67);
residual(96)= lhs-rhs;
lhs =y(97);
rhs =y(68);
residual(97)= lhs-rhs;
lhs =y(98);
rhs =y(70);
residual(98)= lhs-rhs;
lhs =y(99);
rhs =y(67);
residual(99)= lhs-rhs;
if ~isreal(residual)
  residual = real(residual)+imag(residual).^2;
end
if nargout >= 2,
  g1 = zeros(99, 99);

  %
  % Jacobian matrix
  %

T3 = (-1);
T1086 = ccy__*cwhlc__*params(47)*params(46)*1/params(46);
  g1(1,1)=1;
  g1(1,7)=(-params(4));
  g1(1,19)=(-(1-params(4)));
  g1(1,20)=(-(1-params(4)));
  g1(1,56)=1;
  g1(1,69)=1;
  g1(2,6)=1;
  g1(2,7)=(-T124);
  g1(3,7)=1;
  g1(3,8)=1;
  g1(3,19)=T3;
  g1(3,24)=T3;
  g1(4,6)=T3;
  g1(4,8)=1;
  g1(4,9)=T3;
  g1(5,6)=cdelta1__*no_cap_u__/cgamma__;
  g1(5,9)=1-(1-cikbar__);
  g1(5,10)=(-cikbar__);
  g1(5,59)=(-(cikbar__/csadjcost_aux__));
  g1(6,10)=1-(1+cgamma__*cbetabar__)*T155;
  g1(6,11)=(-(csadjcost_aux__*T155));
  g1(6,59)=T3;
  g1(7,7)=(-((1-params(43))*crk__*T186));
  g1(7,11)=1-(1-params(44))*T186;
  g1(7,17)=(-(T186*(-(crk__-params(44)))));
  g1(7,18)=T3;
  g1(7,20)=1;
  g1(7,57)=(-((-tbill__)+T177*(1-tbill__)));
  g1(8,2)=params(3)*(1+T172)-(params(3)+params(3)*T172);
  g1(8,18)=(-(1-T172));
  g1(8,20)=1-T172;
  g1(8,57)=(-(1-tbill__+(-(tbill__*(1-T172)))));
  g1(9,2)=(-ccy__);
  g1(9,3)=1;
  g1(9,4)=T3;
  g1(9,5)=T3;
  g1(9,6)=(-((1-no_cap_u__)*crkky__));
  g1(9,10)=(-ciy__);
  g1(9,67)=T3;
  g1(10,3)=1;
  g1(10,8)=(-(params(7)*params(4)));
  g1(10,24)=(-(params(7)*(1-params(4))));
  g1(10,56)=T3;
  g1(10,69)=T3;
  g1(11,1)=(-(T242*T256));
  g1(11,18)=1-T242*(cgamma__*cbetabar__+params(5));
  g1(11,61)=T3;
  g1(12,2)=(-(T291*(1/(1-T172)-T172/(1-T172))));
  g1(12,18)=(-(T265+params(9)/(1+cgamma__*cbetabar__)-(1+cgamma__*cbetabar__*params(9))/(1+cgamma__*cbetabar__)));
  g1(12,19)=1-(T155+T265-T291);
  g1(12,21)=(-(T291*1/(1-params(46))));
  g1(12,24)=(-(T291*params(12)));
  g1(12,25)=(-(T291*1/(1+params(49))));
  g1(12,62)=T3;
  g1(12,68)=(-(T291*1/(1-params(46))));
  g1(13,3)=(-((1-params(17))*params(15)+(1-params(17))*(-params(19))));
  g1(13,18)=(-((1-params(17))*params(14)));
  g1(13,20)=1-params(17);
  g1(13,60)=T3;
  g1(13,70)=T3;
  g1(13,73)=(-((1-params(17))*(params(18)+params(19))));
  g1(13,75)=(-((1-params(17))*params(20)));
  g1(14,3)=(1-params(21))*params(23);
  g1(14,4)=1-params(21);
  g1(14,23)=(1-params(21))*cby__*params(22);
  g1(14,58)=T3;
  g1(15,3)=(-((1-params(24))*params(25)));
  g1(15,21)=ccy__*params(47)*cwhlc__-ccy__*cwhlc__*params(47)*params(24);
  g1(15,23)=(-((1-params(24))*cby__*params(26)));
  g1(15,63)=(-params(27));
  g1(15,64)=T3;
  g1(15,65)=(-params(28));
  g1(16,3)=(-((1-params(29))*params(30)));
  g1(16,17)=cky__*(crk__-params(44))-(crk__-params(44))*cky__*params(29);
  g1(16,23)=(-((1-params(29))*cby__*params(31)));
  g1(16,63)=T3;
  g1(16,64)=(-params(32));
  g1(16,65)=(-params(33));
  g1(17,3)=(1-params(34))*params(35);
  g1(17,5)=1-params(34);
  g1(17,22)=1-params(34);
  g1(17,23)=(1-params(34))*cby__*params(36);
  g1(17,66)=T3;
  g1(18,25)=ccy__;
  g1(18,63)=(-params(37));
  g1(18,64)=(-params(38));
  g1(18,65)=T3;
  g1(19,2)=ccy__*params(49);
  g1(19,4)=T3;
  g1(19,7)=params(43)*cky__*(crk__-params(44))*crk__/(crk__-params(44));
  g1(19,9)=params(43)*cky__*(crk__-params(44));
  g1(19,17)=params(43)*cky__*(crk__-params(44))*1/params(43);
  g1(19,18)=T459;
  g1(19,19)=ccy__*cwhlc__*params(47)*params(46);
  g1(19,20)=(-T437);
  g1(19,21)=T1086;
  g1(19,22)=T3;
  g1(19,23)=T437-T459;
  g1(19,24)=ccy__*cwhlc__*params(47)*params(46);
  g1(19,25)=ccy__*params(49)*1/params(49);
  g1(19,57)=T437*(-tbill__);
  g1(19,67)=T3;
  g1(19,68)=T1086;
  g1(20,18)=1;
  g1(20,20)=T3;
  g1(20,26)=1;
  g1(21,19)=(-(ccy__*cwhlc__*params(47)*params(46)));
  g1(21,21)=(-T1086);
  g1(21,24)=(-(ccy__*cwhlc__*params(47)*params(46)));
  g1(21,27)=1;
  g1(21,68)=(-T1086);
  g1(22,4)=T3;
  g1(22,28)=1;
  g1(22,67)=T3;
  g1(23,29)=1;
  g1(24,31)=1;
  g1(25,30)=1;
  g1(26,12)=1;
  g1(27,32)=1;
  g1(28,18)=T3;
  g1(28,33)=1;
  g1(29,20)=T3;
  g1(29,34)=1;
  g1(30,21)=T3;
  g1(30,35)=1;
  g1(30,68)=T3;
  g1(31,21)=T3;
  g1(31,36)=1;
  g1(31,68)=T3;
  g1(32,24)=T3;
  g1(32,37)=1;
  g1(33,3)=params(43)*cky__*(crk__-params(44));
  g1(33,7)=(-(params(43)*cky__*(crk__-params(44))*crk__/(crk__-params(44))));
  g1(33,9)=(-(params(43)*cky__*(crk__-params(44))));
  g1(33,13)=1;
  g1(33,17)=(-(params(43)*cky__*(crk__-params(44))*1/params(43)));
  g1(34,3)=ccy__*cwhlc__*params(47)*params(46);
  g1(34,19)=(-(ccy__*cwhlc__*params(47)*params(46)));
  g1(34,21)=(-T1086);
  g1(34,24)=(-(ccy__*cwhlc__*params(47)*params(46)));
  g1(34,38)=1;
  g1(34,68)=(-T1086);
  g1(35,2)=(-(ccy__*params(49)));
  g1(35,3)=ccy__*params(49);
  g1(35,25)=(-(ccy__*params(49)*1/params(49)));
  g1(35,39)=1;
  g1(36,3)=cby__;
  g1(36,23)=(-cby__);
  g1(36,40)=1;
  g1(37,3)=1;
  g1(37,23)=T3;
  g1(37,41)=1;
  g1(38,4)=T3;
  g1(38,42)=1;
  g1(38,67)=T3;
  g1(39,3)=T3;
  g1(39,43)=1;
  g1(40,2)=T3;
  g1(40,44)=1;
  g1(41,10)=T3;
  g1(41,14)=1;
  g1(42,45)=1;
  g1(43,46)=1;
  g1(44,4)=(-params(72));
  g1(44,28)=params(72);
  g1(44,47)=1;
  g1(44,67)=(-params(72));
  g1(45,4)=(-(params(73)/4/(params(52)/4)));
  g1(45,28)=(-((-params(73))/4/(params(52)/4)));
  g1(45,48)=1;
  g1(46,49)=1;
  g1(47,15)=1;
  g1(48,45)=T3;
  g1(48,50)=1;
  g1(49,45)=T3;
  g1(49,51)=1;
  g1(50,45)=T3;
  g1(50,52)=1;
  g1(51,46)=T3;
  g1(51,53)=1;
  g1(52,47)=T3;
  g1(52,54)=1;
  g1(53,55)=1;
  g1(54,16)=1;
  g1(55,5)=1-crhonx__;
  g1(56,56)=1-params(90);
  g1(57,57)=1-params(92);
  g1(58,58)=1-params(94);
  g1(59,59)=1-params(96);
  g1(60,60)=1-params(98);
  g1(61,61)=1-params(100);
  g1(62,62)=1-params(102);
  g1(63,64)=1-params(104);
  g1(64,63)=1-params(106);
  g1(65,65)=1-params(108);
  g1(66,66)=1-params(110);
  g1(67,3)=(1-params(114))*params(121);
  g1(67,23)=(1-params(114))*cby__*params(120);
  g1(67,67)=1-params(114);
  g1(68,3)=(-((1-params(112))*params(123)));
  g1(68,23)=(-((1-params(112))*cby__*params(122)));
  g1(68,68)=ccy__*params(47)*cwhlc__-ccy__*cwhlc__*params(47)*params(112);
  g1(69,69)=1-params(116);
  g1(70,70)=1-params(118);
  g1(71,4)=T3;
  g1(71,67)=T3;
  g1(71,71)=1;
  g1(72,21)=T3;
  g1(72,68)=T3;
  g1(72,72)=1;
  g1(73,3)=T3;
  g1(73,73)=1;
  g1(74,2)=T3;
  g1(74,74)=1;
  g1(75,20)=T3;
  g1(75,76)=1;
  g1(76,18)=T3;
  g1(76,75)=1;
  g1(77,4)=T3;
  g1(77,77)=1;
  g1(78,4)=T3;
  g1(78,78)=1;
  g1(79,4)=T3;
  g1(79,79)=1;
  g1(80,21)=T3;
  g1(80,80)=1;
  g1(81,21)=T3;
  g1(81,81)=1;
  g1(82,21)=T3;
  g1(82,82)=1;
  g1(83,3)=T3;
  g1(83,83)=1;
  g1(84,3)=T3;
  g1(84,84)=1;
  g1(85,3)=T3;
  g1(85,85)=1;
  g1(86,2)=T3;
  g1(86,86)=1;
  g1(87,2)=T3;
  g1(87,87)=1;
  g1(88,2)=T3;
  g1(88,88)=1;
  g1(89,20)=T3;
  g1(89,89)=1;
  g1(90,20)=T3;
  g1(90,90)=1;
  g1(91,20)=T3;
  g1(91,91)=1;
  g1(92,18)=T3;
  g1(92,92)=1;
  g1(93,18)=T3;
  g1(93,93)=1;
  g1(94,18)=T3;
  g1(94,94)=1;
  g1(95,69)=T3;
  g1(95,95)=1;
  g1(96,67)=T3;
  g1(96,96)=1;
  g1(97,68)=T3;
  g1(97,97)=1;
  g1(98,70)=T3;
  g1(98,98)=1;
  g1(99,67)=T3;
  g1(99,99)=1;
  if ~isreal(g1)
    g1 = real(g1)+2*imag(g1);
  end
if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],99,9801);
if nargout >= 4,
  %
  % Third order derivatives
  %

  g3 = sparse([],[],[],99,970299);
end
end
end
end
