function [Shocks_smoother, Shocks_filter, Errors_smoother, Errors_filter, Pt_smoother, Mean_smoother]=Koopman_KF_KS_fast(Y_data, Coeff, ModelStructure, States,CoreList, ShockIndex, DO_VERBOSE, STATIONARY_INITIAL)
% Filter / smoother
% Y = H xi + eps, Var[eps]=Sigma
% xi = F xi(t-1) + eta, Var[eta]=Phi

% DSGE
% X = A X(t-1) + B e
% Y = C X(t-1) + D e

% Mapping
% xi = [X; Y]
% H = [0 I];
% F = [A 0; C 0] 
% Phi_half = [B; D]
% Sigma=0

if ~exist('DO_VERBOSE','var')
    DO_VERBOSE=0;
end
if ~exist('STATIONARY_INITIAL','var')
    STATIONARY_INITIAL=0;
end

Coeff.B=Coeff.B(:,ShockIndex(1:ModelStructure.mY));
Coeff.By=Coeff.By(:,ShockIndex(1:ModelStructure.mY));

ModelStructure.mY_effective=ModelStructure.mY;

ExoProcess_H = zeros(ModelStructure.mY,ModelStructure.mX);
for vv=1:ModelStructure.mY
    ExoProcess_H(vv,strcmp(CoreList{vv,4}, States.Names))=1;
end

Sigma = Coeff.B*Coeff.B';
Phi = Coeff.By*Coeff.By';
Cov = Coeff.B*Coeff.By';

if max(abs(eig(Coeff.A - (Coeff.B / Coeff.By) * Coeff.Ay)))<1 
    P0 = zeros(ModelStructure.mX,ModelStructure.mX);

    Ft = Coeff.Ay * P0 * Coeff.Ay' + Coeff.By * Coeff.By' ;
    Kt = (Coeff.A * P0 * Coeff.Ay' + Coeff.B * Coeff.By') / Ft ;    
    
    STATIONARY_INITIAL=1;
else
    if ~STATIONARY_INITIAL
        vec_VarX = (eye( (ModelStructure.mX)^2)- kron(Coeff.A,Coeff.A)) \ reshape(Sigma, ModelStructure.mX^2,1);
        P0 = reshape(vec_VarX,ModelStructure.mX,ModelStructure.mX);
        
        Ft = Coeff.Ay * P0 * Coeff.Ay' + Coeff.By * Coeff.By' ;
        Kt = (Coeff.A * P0 * Coeff.Ay' + Coeff.B * Coeff.By') / Ft ;

    else
        ChangeTol=1e-10;
        MaxIter=1e4;
        AbsChange=100;
        tt=0;
        Pt_warmup=Sigma;
        while tt<MaxIter && AbsChange>ChangeTol
            tt=tt+1;
            Ft = Coeff.Ay * Pt_warmup * Coeff.Ay' + Coeff.By * Coeff.By' ;
            Kt = (Coeff.A * Pt_warmup * Coeff.Ay' + Coeff.B * Coeff.By') / Ft ;
            Lt = Coeff.A - Kt * Coeff.Ay;
            Pt = Coeff.A*Pt_warmup * Lt' + Coeff.B*(Coeff.B' - Coeff.By' * Kt');
            AbsChange=norm(Pt - Pt_warmup ); %/norm(Pt_warmup);
            if DO_VERBOSE
                fprintf(num2str(max( abs( eig( Coeff.A - Kt  * Coeff.Ay ) ) ), '%1.4f ::'));
                fprintf(num2str(norm(Ft), '%1.4f ::'));
                fprintf(num2str(norm(Pt_warmup  ), '%1.8f ::'));
                fprintf(num2str(norm(Pt - Pt_warmup), '%1.4f ::'));
                fprintf(num2str( AbsChange , '%1.4f ::'));
                fprintf('\n');
            end        
            Pt_warmup = Pt;
        end
        P0=Pt;
    end
end

TT=size(Y_data,2);
NObs=size(Coeff.Ay,1);
NStates=size(Coeff.Ay,2);
NN=sum(diag(Phi).^2>0);

Kt_filter = zeros(ModelStructure.mX,ModelStructure.mY, TT+1);
Pt_filter = zeros(ModelStructure.mX,ModelStructure.mX, TT+1);
Pt_smoother = zeros(ModelStructure.mX,ModelStructure.mX, TT+1);
Ft_filter = zeros(ModelStructure.mY,ModelStructure.mY, TT+1);
Nt_smoother = zeros(ModelStructure.mX,ModelStructure.mX, TT+1);
Mean_filter = zeros(ModelStructure.mX,TT+1);
rt_smoother = zeros(ModelStructure.mX, TT+1);
Errors_filter = zeros(ModelStructure.mY,TT);
Errors_smoother = zeros(ModelStructure.mY,TT);
Shocks_filter = zeros(ModelStructure.mY,TT);
Shocks_smoother = zeros(ModelStructure.mY,TT);

Pt_filter(:,:,1) = P0;
Mean_filter(:,1) = zeros(ModelStructure.mX,1);

for tt=1:TT
    if ~STATIONARY_INITIAL
        Ft_filter(:, :, tt+1) = Coeff.Ay * Pt_filter(:,:,tt) * Coeff.Ay' + Coeff.By * Coeff.By' ;
        Kt_filter(:, :, tt+1) =(Coeff.A * Pt_filter(:,:,tt) * Coeff.Ay' + Coeff.B * Coeff.By') / Ft_filter(:, :, tt+1) ;
        Lt = Coeff.A - Kt_filter(:,:,tt+1) * Coeff.Ay;
        Pt_filter(:,:,tt+1) = Coeff.A*Pt_filter(:,:,tt)* Lt' + Coeff.B*(Coeff.B' - Coeff.By' * Kt_filter(:,:,tt+1)');
    else
        Ft_filter(:,:,tt+1) = Ft;
        Kt_filter(:,:,tt+1) = Kt;
        Pt_filter(:,:,tt+1)  = P0;
    end

    Errors_filter(:,tt+1)= Y_data(:,tt) - Coeff.Ay* Mean_filter(:,tt);
    Shocks_filter(:,tt+1)=Coeff.By \ Errors_filter(:,tt+1);
    Mean_filter(:,tt+1)= Coeff.A * Mean_filter(:,tt) + Kt_filter(:,:,tt) * Errors_filter(:,tt+1);
    
    if DO_VERBOSE
        fprintf( num2str( max(abs(eig( Coeff.A - Kt_filter(:,:,tt) * Coeff.Ay ))), '%1.4f ::'));
        fprintf( num2str( norm(Ft_filter(:, :, tt+1)), '%1.4f ::'));
        fprintf( num2str( norm(Pt_filter(:,:,tt+1)), '%1.4f ::'));
        fprintf( num2str( norm(Pt_filter(:,:,tt+1) - Pt_filter(:,:,tt)), '%1.4f ::'));
        fprintf( num2str( norm(Pt_filter(:,:,tt+1) - Pt_filter(:,:,tt)) / norm( Pt_filter(:,:,tt+1) ), '%1.4f ::'));
        fprintf( '\n' );
    end
end
if DO_VERBOSE
    spy(round(1e3*Pt_filter(:,:,tt+1))/1e3);
end
for tt=TT:-1:1
    Qt = inv(Ft_filter(:, :, tt+1)) + Kt_filter(:, :, tt+1)' * Nt_smoother(:,:,tt+1) * Kt_filter(:, :, tt+1);
    [V,D]=eig( Qt );
    D(D<0)=0;
    if max(abs(imag(diag(D))))<1e-10
        Qt = real( V*D*V' );
    else
        error('imaginary covariance');
    end

    M_1 = [Coeff.Ay', Coeff.A'; Coeff.By', Coeff.B'];
    M_2 = [Qt, -Kt_filter(:,:,tt+1)' * Nt_smoother(:,:,tt+1); -Nt_smoother(:,:,tt+1)' * Kt_filter(:,:,tt+1), Nt_smoother(:,:,tt+1)];
    
    M_prod = M_1 * M_2 * M_1';
    Nt_smoother(:,:,tt) = M_prod(1:ModelStructure.mX,1:ModelStructure.mX);
    Rt_smoother(:,:,tt+1) = M_prod(ModelStructure.mX+1:end,ModelStructure.mX+1:end);    
    
    Nt_smoother(:, :, tt) = Coeff.Ay' * Qt * Coeff.Ay - Coeff.A' * Nt_smoother(:,:,tt+1) * Kt_filter(:,:,tt+1) * Coeff.Ay ...
                                                     -(Coeff.A' * Nt_smoother(:,:,tt+1) * Kt_filter(:,:,tt+1) * Coeff.Ay)' ...
                                                     + Coeff.A' * Nt_smoother(:,:,tt+1) * Coeff.A;

	et = inv(Ft_filter(:,:,tt+1) ) * Errors_filter(:,tt+1) - Kt_filter(:,:,tt+1)' * rt_smoother(:,tt+1);
    rt_smoother(: ,tt) = Coeff.Ay' * et + Coeff.A' * rt_smoother(: ,tt+1);
    ut = Coeff.By' * et + Coeff.B' * rt_smoother(:,tt+1);

    Errors_smoother(:,tt+1)=Coeff.By'\ut;
    Shocks_smoother(:,tt+1)= ut;
    
end
Mean_smoother=[];
Pt_smoother = Rt_smoother;
