%% Marginal data density and signal-to-noise ratio plots
clear all;

% Which dimension to put on the horizontal axis
WhichDim=1;
% where to center the horizontal axis
PlotCenter = 5;

% Average different parallel chains/different seed?
DO_AVERAGE=true;

% Provide file name pattern
MyQuery = dir('Data_GtFYDII_*.mat');

% Arrange data file names
DataFiles = cell(length(MyQuery),1);
for ll=1:length(MyQuery)
    DataFiles{ll} = MyQuery(ll).name;
end


if ~isempty(strfind( DataFiles{1}, 'eec_'))
    if isempty(strfind( DataFiles{1}, 'update'))
        Suffix='_news';
    else
        Suffix='_news_update';
    end
else
    if ~isempty(strfind( DataFiles{1}, 'SPF_RR_MR_IV'))
        if ~isempty(strfind( DataFiles{1}, 'update'))
            Suffix='_update';
        else
            Suffix='_SPF_only';
        end
    else
        Suffix='_SPF_only';
    end
end


SNR_DATA = nan(length(DataFiles),3,5);
CI_lvl=[68 90];
DD_mz=load(DataFiles{1}, 'mz');
MDensity_Data=zeros(length(DataFiles),5);
ShockCorr_Data=NaN(length(DataFiles),3,DD_mz.mz);
IV_ShockCorr_Data=NaN(length(DataFiles),3,DD_mz.mz,2);
for dd_idx=1:length(DataFiles)

    disp(dd_idx);
    clear CurrentData;
    try
        CurrentData=load(DataFiles{dd_idx}, 'log_p_Y', 'OriginalData', 'DSGE_lambda_S', 'DSGE_lambda', 'SEED', 'mat_draws', 'PriorStructure', 'ShockCorr', 'IV_Shock_Corr');
        disp(CurrentData.SEED);
    catch
        CurrentData=load(DataFiles{dd_idx}, 'log_p_Y', 'OriginalData', 'DSGE_lambda_S', 'DSGE_lambda', 'mat_draws', 'PriorStructure', 'ShockCorr', 'IV_Shock_Corr');
        CurrentData.SEED=15;
    end
    MDensity_Data(dd_idx,:) =  [CurrentData.DSGE_lambda, CurrentData.DSGE_lambda_S, CurrentData.OriginalData.k, CurrentData.log_p_Y, CurrentData.SEED];
    for ss=1:CurrentData.PriorStructure.c_signal_no 
        SNR_DATA(dd_idx,ss,:)=prctile( CurrentData.mat_draws.S(:,CurrentData.PriorStructure.param_no-CurrentData.PriorStructure.c_signal_no*2+ss+CurrentData.PriorStructure.c_signal_no)./CurrentData.mat_draws.S(:,CurrentData.PriorStructure.param_no-CurrentData.PriorStructure.c_signal_no*2+ss) ...
            ,[(50-CI_lvl(2)/2) (50-CI_lvl(1)/2) 50 (50+CI_lvl(1)/2) (50+CI_lvl(2)/2)]);
    end
    
    if ~isequal(Suffix,'_update') && ~isequal(Suffix,'_news_update') 
        ShockCorr_Data(dd_idx,:,:)=prctile(CurrentData.ShockCorr(:,:),[50 5 95]);
        IV_ShockCorr_Data(dd_idx,:,:,1)=prctile(squeeze(CurrentData.IV_Shock_Corr(:,:,1))',[50 5 95]);
        IV_ShockCorr_Data(dd_idx,:,:,2)=prctile(squeeze(CurrentData.IV_Shock_Corr(:,:,2))',[50 5 95]);
    end
    
end

if isempty(strfind(DataFiles{1}, 'GtFYDIeec'))
    my_IDX{1} = round(100*MDensity_Data(:,2))/100 ==1;
    my_IDX{2} = round(100*MDensity_Data(:,2))/100 ==1/2;
    my_IDX{3} = round(100*MDensity_Data(:,2))/100 ==1/5 ;
else
    my_IDX{1} = round(100*MDensity_Data(:,2))/100 ==1;
    my_IDX{2} = round(100*MDensity_Data(:,2))/100 ==1/4 ;
end

%%
close all
figure;
PlotValues = MDensity_Data(:,4)-max(MDensity_Data(:,4))*0;
Label_MDD = MDensity_Data(:,WhichDim);
if ~isempty(strfind(DataFiles{1}, 'GtFYDIeec'))
    MDensity_Data(MDensity_Data(:,WhichDim)==1/4,WhichDim)=1/5;
    MDensity_Data(MDensity_Data(:,WhichDim)==1/2,WhichDim)=.6;
end
Plot_MDD = MDensity_Data(:,WhichDim);

cmap = get(gca, 'colororder');
my_marker = {'x';'o';'v';'d'};
for ii = 1:length(my_IDX)
    if ~DO_AVERAGE
        scatter(MDensity_Data(my_IDX{ii},WhichDim)./(PlotCenter+MDensity_Data(my_IDX{ii},WhichDim)), ...
            -log10(-PlotValues(my_IDX{ii})), 50, 'LineWidth', 2, 'marker', my_marker{ii});
    else
        DATA=NaN(length(unique(MDensity_Data(my_IDX{ii},WhichDim))), size(MDensity_Data,2));
        tmp_data=MDensity_Data(my_IDX{ii},:);
        jj=0;
        for uu=unique(MDensity_Data(my_IDX{ii},WhichDim))'
            jj=jj+1;
            DATA(jj,:)=mean(tmp_data( tmp_data(:,WhichDim)==uu ,:),1);
        end
        scatter(DATA(:,WhichDim)./(PlotCenter+DATA(:,WhichDim)), ...
            -log10(-DATA(:,4)), 50, 'LineWidth', 2, 'marker', my_marker{ii});
    end
    hold all;
    disp('Shock correlation: DSGE vs DSGE-VAR');
    for mm=1:DD_mz.mz
        disp([ii, mm]);
        dispData=[MDensity_Data(my_IDX{ii},[1 2]), ShockCorr_Data(my_IDX{ii}, :,mm)];
        [~,disp_IDX]=sort(dispData(:,1),1);
        disp(num2str(dispData(disp_IDX,:), '%1.2f'));
    end
    disp('Shock correlation: DSGE-VAR vs IV');
    for mm=1:DD_mz.mz
        disp([ii, mm]);
        dispData=[MDensity_Data(my_IDX{ii},[1 2]), squeeze( IV_ShockCorr_Data(my_IDX{ii}, :,mm,1) )];
        [~,disp_IDX]=sort(dispData(:,1),1);
        disp(num2str(dispData(disp_IDX,:), '%1.2f'));
    end
    disp('Shock correlation: DSGE vs IV');
    for mm=1:DD_mz.mz
        disp([ii, mm]);
        dispData=[MDensity_Data(my_IDX{ii},[1 2]), squeeze( IV_ShockCorr_Data(my_IDX{ii}, :,mm,2) )];
        [~,disp_IDX]=sort(dispData(:,1),1);
        disp(num2str(dispData(disp_IDX,:), '%1.2f'));
    end
end

set(gca, 'Xtick', unique(Plot_MDD./(PlotCenter+Plot_MDD)));
if isempty(strfind( DataFiles{1}, 'eec_'))
    set(gca, 'Ytick', -log10(-((floor(min(MDensity_Data(:,4))/100)*100):50:(ceil(max(MDensity_Data(:,4))/100)*100))' ) );
else
    set(gca, 'Ytick', -log10(-((floor(min(MDensity_Data(:,4))/100)*100):100:(ceil(max(MDensity_Data(:,4))/100)*100))' ) );
end
set(gca, 'YTicklabels', num2str(-10.^(-get(gca, 'Ytick')'), '%1.0f'));
CurrentLabels=cellstr(num2str(unique(Label_MDD), '%1.2g'));
CurrentLabels=regexprep(CurrentLabels, '0.', '.');
CurrentLabels=regexprep(CurrentLabels, ' ', '');
CurrentLabels{end}='inf';
set(gca, 'XTickLabels', CurrentLabels)
grid on;

xlabel('T_0^B / T')
ylabel('marginal data density');

if isempty(strfind(DataFiles{1}, 'GtFYDIeec'))
    legend({'T_0^V = T';'T_0^V = T / 2 '; 'T_0^V = T / 5' },'Location', 'Best', 'Orientation','vertical');
else
    legend({'T_0^V = T';'T_0^V = T / 4'},'Location', 'Best', 'Orientation','vertical');
end
        set(findall(gcf,'type','axes'),'fontsize',16);
        set(findall(gcf,'type','text'),'fontSize',16);        

% return;
if DO_AVERAGE
    print('-depsc', ['Graphs/New_MDD' Suffix '.eps'])
end

%%
figure;
PlotValues = MDensity_Data(:,4)-max(MDensity_Data(:,4))*0;
cmap = get(gca, 'colororder');
my_marker = {'x';'o';'v';'d'};


ylim_list=NaN(length(my_IDX),2);
for ii = 1:length(my_IDX)
    figure(1+ii);
    tmp_SNR_DATA=SNR_DATA(my_IDX{ii},:,:);
    [SortedX, IDX]=sort(MDensity_Data(my_IDX{ii},WhichDim)./(PlotCenter+MDensity_Data(my_IDX{ii},WhichDim)));
        DATA=NaN([length(unique(MDensity_Data(my_IDX{ii},WhichDim))), size(SNR_DATA,2), size(SNR_DATA,3)]);
        DensityData=MDensity_Data(my_IDX{ii},WhichDim);
        tmp_data=SNR_DATA(my_IDX{ii},:,:);
        jj=0;
        for uu=unique(MDensity_Data(my_IDX{ii},WhichDim))'
            jj=jj+1;
            DATA(jj,:,:)=mean(tmp_data( DensityData ==uu ,:,:),1);
        end
    
    hndl=plot(unique(SortedX), squeeze(DATA(:,:,3)), 'LineWidth', 3);
    hold all;
    new_hndl=plot(unique(SortedX), squeeze(DATA(:,:,1)), '--', 'LineWidth', 2);
    set(new_hndl, {'Color'}, get(hndl, 'Color'));
    new_hndl=plot(unique(SortedX), squeeze(DATA(:,:,5)), '--', 'LineWidth', 2);
    set(new_hndl, {'Color'}, get(hndl, 'Color'));

    set(gca, 'Xtick', unique(MDensity_Data(:,WhichDim)./(PlotCenter+MDensity_Data(:,WhichDim))));
    CurrentLabels=cellstr(num2str(unique(MDensity_Data(:,WhichDim)), '%1.2g'));
    CurrentLabels=regexprep(CurrentLabels, '0.', '.');
    CurrentLabels=regexprep(CurrentLabels, ' ', '');    
    CurrentLabels{end}='inf';
    set(gca, 'XTickLabels', CurrentLabels)
    grid on;

    xlabel('T_0^B / T')
    ylabel('signal to noise ratio');

    if isempty(strfind(DataFiles{1}, 'GtFYDIeec'))
        legend(hndl,{'G';'tax'; 'FFR'},'Location', 'Best');
    else
        legend(hndl,{'G';'FFR'},'Location', 'Best');
    end
            set(findall(gcf,'type','axes'),'fontsize',16);
            set(findall(gcf,'type','text'),'fontSize',16);   
            
    ylim_list(ii,:)=ylim;
end
for ii=1:length(my_IDX)
    figure(1+ii)
    axis([xlim 0 max(ylim_list(:))]);
    print('-depsc', ['Graphs/SNR' num2str(ii)  Suffix '.eps'])
end
