% Code by Thorsten Drautzburg
% This Version: November 2019

clear variables global;
clc;
close all;
fclose all;


global my_draw my_log_prior

% on a PC, don't use GPU capability. Set Path to Dynare
% disable this code if using a Mac/Linux machine without GPU capability
if ispc
  addpath(regexprep(pwd, '\VAR',''));
  addpath(genpath('../../v4.4.3')); % Dynare path
  copyfile('../my_VAR_Moments_CPU.m', '../my_VAR_Moments.m');
else
    copyfile('../my_VAR_Moments_GPU.m', '../my_VAR_Moments.m');
    maxNumCompThreads=4;    
    addpath('/dynare/matlab'); % Dynare path
    addpath(regexprep(pwd, '/VAR',''));
end

DO_PRINT=1; % save results and produce graphs (when JVM is enabled)
DO_SMALL = 1; % small sample of draws

if ~DO_PRINT
        disp('*******************************************************************************');
        fprintf(2, '                          RESULTS ARE NOT SAVED \n');
        disp('*******************************************************************************');
end

NO_TREND = 1; % using detrended data
SEED = 1; % set seed for random number generator
rng(SEED);

% for reporting and computations
h=21; % steps for the IRF (here: h quarters + impact period)
CI_lvl=[68 90]; % coverage probabiltiy of Credible sets 

DO_IV = 5;
    % 5 - all three, baseline
    % 1 - SPF/GB only
    % 2 - SPF + RR only
    % 3 - RR only (30: RR update only)
    % 4  - MR + RR only
    % 6  - MR only
    % 8  - SPF/GB + MR only
USE_FF=0; % 1 - use Fed Funds Futures data instead

DSGE_lambda_S= 0.2;
DSGE_lambda= 4;
    clear current* R_* R IV_* IRF_* DoResumeSampling mat_draws log_p_Y ConvergencePlot
    
    DO_IV_CHOL_RES=0;
    DO_COST_CHANNEL=1;
    WhichChol=1; % 0 - cholesky of a11a11', 1 - cholesky of s11s11', 2 - MR lower, 3 - MR upper
    Post1982 = 0; % use only data post 1982
    Post1982_IV=0; % use only IV data for identification post 1982
    Post1966=0;
    Post66k8smple=0;
    DO_SEPARATE=0;
    DO_MANUAL_SUR=1;
    DO_IMPUTATION=1;

    DO_POLICY_RULES=1;
    if DO_POLICY_RULES==0
        disp('*******************************************************************************');
        disp('                           Policy rules disabled                               ');
        disp('*******************************************************************************');
    end
    SIMUL_T=240;
    WHICH_MODEL='';
%    WHICH_MODEL='NEW_IV';
%    WHICH_MODEL='FLEX'; % or empty ('') or 'NEW_IV'
    OLD_IV_SAMPLE=0; % 0 (+ all other)= full sample available; 1 = old sample for all variables; 2=old sample for G only, -1: drop pre 1966 tax instruments
    WHICH_DATA=-9;

    DO_SMALL_SAMPLE=0;
    DO_DISPLAY=1;
    DO_CHOL=1;
    DO_CPI=-2;
    
    NO_IRF=0;
    if NO_IRF==1
        disp('*******************************************************************************');
        disp('                           IRF computation disabled                            ');
        disp('*******************************************************************************');
    end
    MY_PRIOR=2; 
        %0: Flat prior. 
        %1: Minnesota prior. 
        %2: DSGE Prior 
        %33: DSGE mean prior
        %34: Dummy prior with simulated data
    DO_PRIOR_MEAN=1;
    NO_TRUNCATION=1;
    ESTIMATE_IV_ERR=3;
    if ~exist('DSGE_lambda_S','var')
        if ~exist('DSGE_lambda','var')
                DSGE_lambda=2.00; % mC*n_lags+2=26 in baseline case. With T=236, this requires DSGE_LAMBDA>=0.1102
                DSGE_lambda_S=0.20; % mC*n_lags+2=26 in baseline case. With T=236, this requires DSGE_LAMBDA>=0.1102
        else
            DSGE_lambda_S=DSGE_lambda;
        end
    end
    UseStudentDist=1;
    StudentDF=15;
    DoRandomBlocking=1;
    
    if MY_PRIOR~=2
        if DO_PRIOR_MEAN
            switch upper(WHICH_MODEL)
                case 'FORESIGHT+K'
                    ModFile='NK_narrative_v7.mod';

                    SourceModFile='NK_priors_v1_loose_SD_rules_capital_foresight_k.mod';
                    OverWriteModFile='../NK_override_v2.mod';
                otherwise
                    ModFile='NK_narrative_v7.mod';

                    SourceModFile='NK_priors_v1_loose_SD_rules_capital.mod';
                    OverWriteModFile='../NK_override_v2.mod';
            end
        else
            error('Model option currently not specified');
        end
    else
        switch upper(WHICH_MODEL)
            case 'FORESIGHT+K'
                ModFile='NK_narrative_v7.mod'; %without flex price economy in the background
                SourceModFile='NK_priors_v1_loose_SD_rules_capital_foresight_k.mod';
                OverWriteModFile='../NK_override_v2.mod';
            case 'FLEX'
                ModFile='NK_narrative_Flex_v7.mod'; % with flex price economy in the background 
                SourceModFile='NK_priors_v1_loose_SD_rules_capital.mod';
                OverWriteModFile='../NK_override_v2.mod';                
            otherwise
                ModFile='NK_narrative_v7.mod'; %without flex price economy in the background
                SourceModFile='NK_priors_v1_loose_SD_rules_capital.mod';
                OverWriteModFile='../NK_override_v2.mod';
        end
    end     
        PICK_VARS={'lvl_g';'tauP';'tb3';'lvl_y';'DEBT_Y';'lvl_inv'; 'infobs'; 'GB_g_lvl'; 'GB_y_lvl'; 'GB_inf_lvl'; 'GB_FedR_lvl'};
        Labels={'G';'tax'; 'FFR';'Y';'Debt';'Inv';'Inf';'expected G'; 'expected Y'; 'expected inf'; 'expected rev'};
        CHOL_ORDER={'G',1;'tax',2;'Y',3;'Debt',4;'FFR',7;'Inf',6;'Inv',5; 'expected G', 8; 'expected Y', 9; 'expected inf', 10; 'expected rev', 11};

        switch upper(WHICH_MODEL)
            case 'FORESIGHT'
                EXCLUDE_VARS={'Rev';'C';'Inv';'hours'; 'real wage'; 'expected inf'; 'expected rev'; 'capital'; 'spread'; 'oil'};
            case 'FORESIGHT+REV'
                EXCLUDE_VARS={'Rev';'C';'Inv';'hours'; 'real wage'; 'expected inf'; 'spread'; 'oil'};
            case 'FORESIGHT+INF'
                EXCLUDE_VARS={'Rev';'C';'Inv';'hours'; 'real wage'; 'expected rev'; 'spread'; 'oil'};
            case 'FORESIGHT+K'
                EXCLUDE_VARS={'Rev';'C';'hours'; 'Inv';'real wage'; 'expected inf';'expected rev'; 'spread'; 'oil'};
            case 'FORESIGHT+I'
                EXCLUDE_VARS={'Rev';'C';'hours'; 'real wage'; 'expected inf';'expected rev'; 'capital'; 'spread'; 'oil'};
            otherwise
                EXCLUDE_VARS={'expected G'; 'expected Y'; 'expected inf'; 'expected rev'; 'capital'; 'spread'; 'oil'};
        end

    % Format: DSGE variable name, VAR variable name, Trend (1) or no trend (0), corresponding ``own'' shock in DSGE model

        DSGE_VAR_MAP={'gc_obs','G',1,'eg', 'u_g'; ...
                      'tauP','tax',0,'e_t_l', 'u_t_l'; ...
                      'y_obs','Y',1,'ea', 'a'; ...
                      'DEBT_Y','Debt',1,'e_s', 'u_s'; ...
                      'inve','Inv',1,'eqs', 'qs'; ...
                      'tb3','FFR',0,'em', 'ms';  ...
                      'infobs','Inf',0,'epinf', 'spinf';  ...
                      'Ex_g', 'expected G', 1, 'e_f_g', 'f_g';  ...
                      'Ex_y', 'expected Y', 1, 'e_f_a', 'f_a'; ...
                      'Ex_rev', 'expected rev', 1, 'e_f_t_l', 'f_t_l'; ...
                      'Ex_inf', 'expected inf', 1, 'e_f_m', 'f_m'; ...
                      };
                         
  
    rhomax=1.03; % maximum root allowed
    rhomax_attempts=1e2; % maximum number of draws to get smaller root (given same distribution) before taking a draw of DSGE parameters from prior

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%% MCMC CONFIG %%%%%%%% MCMC CONFIG %%%%%%%% MCMC CONFIG %%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    switch MY_PRIOR
        case 2 % with DSGE-VAR prior
            if ~DO_PRINT
                GibbsBurnIn=5e2;
                GibbsSkip=2;
                TargetScale=0.3; % target acceptance rate
                DD=2.5e2;% number of draws from posterior
            else
                if ~ispc && ~DO_SMALL
                    GibbsBurnIn=2.5e3;
                    GibbsSkip=20;
                    TargetScale=0.3; % target acceptance rate
                    DD=5e3;% number of draws from posterior
                else
                    GibbsBurnIn=2.5e2;
                    GibbsSkip=20;
                    TargetScale=0.3; % target acceptance rate
                    DD=5e3;% number of draws from posterior                    
                end
            end
        otherwise %no DSGE-VAR prior
            if ~DO_PRINT
                GibbsSkip=3 ;   
                GibbsBurnIn=1e3; 
                DD=1e3;% number of draws from posterior    
            else
                GibbsSkip=10 ;   
                GibbsBurnIn=2e3; 
                DD=5e3;% number of draws from posterior    
            end
    end
    if WHICH_DATA~=-1
        switch upper(WHICH_MODEL) 
            case {'FORESIGHT', 'FORESIGHT+REV', 'FORESIGHT+INF'}
                OriginalData.k=4; %number of lags
            case {'FORESIGHT+K'}
                OriginalData.k=4; %number of lags
            otherwise
                OriginalData.k=4; %number of lags
        end
    else
        switch upper(WHICH_MODEL) 
            case {'FORESIGHT', 'FORESIGHT+REV'}
                OriginalData.k=12; %number of lags
            otherwise
                OriginalData.k=12; %number of lags
        end
    end

    % Alternatively, Uhlig-style max FEVD to identify shocks
DO_UHLIG_FACTOR = 0;
    min_h_Uhlig=4;
    max_h_Uhlig=19;
    Uhlig_VAR='G';

    StationaryPrior=1-cell2mat(DSGE_VAR_MAP(:,3))';
    Units={'%'; 'p.p.'; '%'; '%'; '%'; 'p.p.'; '%'; 'p.p.'; 'p.p.'; '%'; '%'; '%'; '%'; '%';'%';'%';'%';'%'};

    DO_IV_PRIOR=0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%% READ VARIABLES %%%%%%%%%%%%%%% READ VARIABLES %%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    OriginalLabels=Labels;
    my_Exclusions=find(cellfun(@(x)(any(~cellfun(@isempty,regexp(x,EXCLUDE_VARS)))),Labels))';
    S_idx_Included=setdiff(1:length(PICK_VARS), my_Exclusions);
    PICK_VARS=PICK_VARS([S_idx_Included, my_Exclusions]);
    Units=Units([S_idx_Included, my_Exclusions]);
    Labels=Labels([S_idx_Included, my_Exclusions],:);
    StationaryPrior=StationaryPrior([S_idx_Included, my_Exclusions] );
    my_Exclusions=length(S_idx_Included)+1:length(Labels);

    
        PICK_VARS(my_Exclusions)=[];

        % CHOL_ORDER is in abritrary order in terms of the variables, thus need to
        % go through them one by one.
        CHOL_EXCLUDED=nan(length(my_Exclusions),1);
        for ll=1:length(my_Exclusions)
            CHOL_EXCLUDED(ll,1)=CHOL_ORDER{ strcmp(Labels(my_Exclusions(ll)),CHOL_ORDER(:,1)),2};
            CHOL_ORDER( strcmp(Labels(my_Exclusions(ll)),CHOL_ORDER(:,1)) ,:)=[];
        end
        CHOL_EXCLUDED=sort(CHOL_EXCLUDED);
        subtractions=0;
        for ll=1:length(CHOL_EXCLUDED)
            tmp_idx=find(cell2mat(CHOL_ORDER(:,2))>CHOL_EXCLUDED(ll)-subtractions)';
            for ii=tmp_idx
                CHOL_ORDER{ ii,2} =CHOL_ORDER{ ii,2}-1;
            end
            if ~isempty(tmp_idx)
                subtractions=subtractions+1;
            end
        end
        clear tmp_idx subtractions CHOL_EXCLUDED;
        Labels(my_Exclusions)=[];
        StationaryPrior(my_Exclusions)=[];
        my_Exclusions=[];
        S_idx_Included=1:length(Labels);


    m_all=length(PICK_VARS);
    m=length(S_idx_Included);
    m_p=m_all-m;

    
    run my_VAR_config

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%% CHOLESKI ROTATION %%%%%%%%%%%%% CHOLESKI ROTATION %%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if length(DO_IV_CHOL_RES)*(DO_IV_CHOL_RES>0)>0
        CholToIV=[find(strcmp(Labels,CHOL_ORDER{DO_IV_CHOL_RES,1})) CholToIV];
    end

    ImpliedCholOrder=NaN( (length(Labels)-length(my_Exclusions)),1);
    for mm=1:(length(Labels)-length(my_Exclusions))
        ImpliedCholOrder(mm,1)=CHOL_ORDER{strcmp(Labels{mm},CHOL_ORDER(:,1)),2} ...
            - ( CHOL_ORDER{strcmp(Labels{mm},CHOL_ORDER(:,1)),2} > m)*length(my_Exclusions) ;
    end
    while ~isempty(find([1; diff(sort(ImpliedCholOrder))]>1,1))
        SortedOrder=sort(ImpliedCholOrder);
        ImpliedCholOrder( (ImpliedCholOrder>=SortedOrder(find([1; diff(sort(ImpliedCholOrder))]>1,1),1))) = ...
            ImpliedCholOrder( (ImpliedCholOrder>=SortedOrder(find([1; diff(sort(ImpliedCholOrder))]>1,1),1))) ...
            -(SortedOrder(find([1; diff(sort(ImpliedCholOrder))]>1,1),1)-SortedOrder(find(diff(sort(ImpliedCholOrder))>1,1),1)-1);
    end

    CholRotatePre=zeros(length(Labels)-length(my_Exclusions));
    for mm=1:(length(Labels)-length(my_Exclusions))
        CholRotatePre(mm, ImpliedCholOrder(mm) )=1;
    end
    CholRotatePost=zeros(length(Labels)-length(my_Exclusions));
    for mm=1:(length(Labels)-length(my_Exclusions))
        CholRotatePost( ImpliedCholOrder(mm) ,mm)=1;
    end
    if ~isequal(CholRotatePost*CholRotatePost',eye(size(CholRotatePost)))
        error('Problem with Choleski ordering');
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%% SET SUFFIX %%%%%%%%%%%%%%%%% SET SUFFIX %%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        if MY_PRIOR>1
            VAR_suffix=[VAR_suffix '-' SourceModFile(strfind(SourceModFile,'priors')+length('priors_v1_'):end-4) ];
            VAR_suffix=[VAR_suffix '-l' regexprep(num2str(DSGE_lambda, '%1.2f'),'\.','-')];
            if DSGE_lambda_S~=DSGE_lambda
                VAR_suffix=[VAR_suffix '-lS' regexprep(num2str(DSGE_lambda_S, '%1.2f'),'\.','-')];
            end
        end

    if MY_PRIOR~=2 && MY_PRIOR>0
        if ~isequal(SourceModFile, ModFile)
            VAR_suffix=[VAR_suffix '_prior'];
        end
    end
    if MY_PRIOR==2 
        VAR_suffix=[VAR_suffix '_EIV' num2str(ESTIMATE_IV_ERR) ];
    end
        
    if DO_SMALL==1
        VAR_suffix=[VAR_suffix '_sm'];
    end
    
    if DO_IMPUTATION~=1
        VAR_suffix=[VAR_suffix '_xImp'];
    end
    
    VAR_suffix = [VAR_suffix '_s' num2str(SEED, '%1.0f')];
        
    if MY_PRIOR==0
        DSGE_lambda=0;
        DSGE_lambda_S=0;
    end
    
    my_VAR_module_GibbsPlus;

    save tmp
    my_VAR_Plot_Save;
	