function [Yresponse, Coeff, ModelStructure, States]= my_DSGE_IRF(HH, oo_ ,M_,CoreList, InstrumentList, ForecastErrIdx,ModelStructure,CholToIV, DO_NORMALIZE)
% Compute Dynare DSGE model based IRFs for variables in CoreList and Shocks in InstrumentList using the indexing in
% ForecastErrIdx

% if nargin<=2
%     error('Need to provide both the decision rule and either indices or variable and shock names');
% end;
% if nargin==2
%     if ~isnumeric(varargin{2})
%         error('Need to provide both the horizon and some data');
%     else
%         DoAreaPlot=false;
%     end;
% else
    
if ~exist('DO_NORMALIZE', 'var')
    DO_NORMALIZE=true;
end
%     Code ripped from my_Dynare_DSGE_VAR_Prior_fcn_v2

if isempty(strmatch('nstatic',fieldnames(oo_.dr)))
    oo_.dr.nstatic=M_.nstatic;
    oo_.dr.npred=M_.npred;
    oo_.dr.nboth=M_.nboth;
    oo_.dr.nfwrd=M_.nfwrd;
end


States.Names=cellstr(M_.endo_names(oo_.dr.order_var(oo_.dr.nstatic+(1:(oo_.dr.npred+oo_.dr.nboth))),:));
States.Index=oo_.dr.nstatic+(1:(oo_.dr.npred+oo_.dr.nboth));


% A are the States.Index x all elements as dynare only includes states
Coeff.A=oo_.dr.ghx(States.Index, :);
% A are the States.Index x all elements 
Coeff.B=oo_.dr.ghu(States.Index, :);

ModelStructure.mX=size(Coeff.A,1);
%     ModelStructure.mY=size(Coeff.Ay,1);

%     % ShockIndex{jj} contains the position of the jj'th shock (in ShockList) in M_.exo_names and thus also the corresponding
%     % column number of Coeff.B and Coeff.By below
%     ShockIndex=zeros(length(ShockList),1);
%     for jj=1:length(ShockList)
%     %     w0=[[1; 0; 0], zeros(3, 30)];
%         % need to change the above!
%         ShockIndex(jj)=strmatch(ShockList{jj}, M_.exo_names, 'exact');
%     end;

VariableIndex=NaN(size(CoreList,1),1);
%     Coeff.By=zeros(size(CoreList,1), size(oo_.dr.ghu,2));
for ll=1:ModelStructure.m
    VariableIndex(ll)=oo_.dr.inv_order_var(strmatch(CoreList{ll,1}, M_.endo_names, 'exact'));


    % A are the States.Index x all elements as dynare only includes states
    Coeff.Ay(ll,:)=oo_.dr.ghx(VariableIndex(ll), :);
    % A are the States.Index x all elements 
    Coeff.By(ll,:)=oo_.dr.ghu(VariableIndex(ll), :);
    Coeff.Mean(ll,:)=oo_.dr.ys(strmatch(CoreList{ll,1}, M_.endo_names, 'exact'), :); %note: this is in declaration order, not DR rule order!
end
% total numbers of endogenous variables
ModelStructure.mY=size(Coeff.Ay,1);
% How many shocks are there prior to adding measurement error?
mShocksExisting=size(Coeff.By,2);
% add columns for noise to state and observation equations
ModelStructure.mZ=size(InstrumentList,1);
Coeff.B=[Coeff.B zeros(ModelStructure.mX,ModelStructure.mz)];
Coeff.By=[Coeff.By zeros(ModelStructure.m,ModelStructure.mz)];

%     Compute IRF at horizon h as X(t+h) = A^h Xt + B \sum(hh=1:HH)bA^hh for the state
%     and Y(t+h) = Ay X(t+h-1) + By * (h==0)

if HH>0
    Xresponse = NaN(ModelStructure.mX,HH+1,ModelStructure.mz);
    Yresponse = NaN(ModelStructure.mY,HH+1,ModelStructure.mz);

    % ForecastErrIdx
    Xresponse(:,1,:)=Coeff.B(:,ForecastErrIdx(:,2)');
    Yresponse(:,1,:)=Coeff.By(:,ForecastErrIdx(:,2)');
    if DO_NORMALIZE
        for ll=1:ModelStructure.mz
            Xresponse(:,1,ll)=Xresponse(:,1,ll)/Yresponse(CholToIV(ll),1,ll);
            Yresponse(:,1,ll)=Yresponse(:,1,ll)/Yresponse(CholToIV(ll),1,ll);
        end
    end
    for hh=2:HH+1
        Xresponse(: , hh,:)=Coeff.A*squeeze(Xresponse(: , hh-1,:));
        Yresponse(: , hh,:)=Coeff.Ay*squeeze(Xresponse(: , hh-1,:));
    end
else
    Yresponse=[];
end