function new_draw=my_RW_MH(old_draw, proposal_var, scale)
% Draw from Random Walk Metropolis-Hastings with normal density

% Variance: Typically coming from Hessian of log posterior = log(m-dens) + log(prior)
% First draw centered at posterior mode

% Draw from DSGE model prior and translate into VAR prior

floor_T=floor(T*(DSGE_lambda));

draw_idx=0;
NewDrawNeeded=true;
while draw_idx<10 && NewDrawNeeded
    draw_idx=draw_idx+1;
    run('../my_Dynare_DSGEVAR_Prior_m.m');
    if NewDrawNeeded
        disp('Problem with VAR representation of prior DSGE draw, trying again');
    end
end

Y_dum=[sqrt(floor_T)*B_prior zeros(size(NXY,1), mz)];
X_dum=sqrt(floor_T)*eye(size(NXX));
Y=[Y_dum; [OriginalData.Y(:,1:m) OriginalData.IV]];
Xy=[X_dum; OriginalData.Xy];
Xz=[zeros(size(X_dum,1),NoXz); OriginalData.Xz];

%         inv(DumX_l(:,1:(end-2))'*DumX_l(:,1:(end-2)))*(DumX_l(:,1:(end-2))'*DumY_l(:,:))
X_0=sparse([kron(eye(m), X_dum), zeros(m*NoParamsC,mz); zeros(mz*NoParamsC,m*NoParamsC), kron(eye(mz),ones(size(X_dum,1),NoXz))]);
Y_0=Y_dum(:);
try
    invCholS=inv(chol(S_prior));
catch
    if min(eig(S_prior))>0
        invCholS=(S_prior)^(-1/2);    
    else
        error('negative definite prior variance-covariance');
    end
end
%%
X_0_tilde=sparse(kron(invCholS, eye(NoParamsC))*X_0);
Y_0_tilde=sparse(kron(invCholS, eye(NoParamsC))*Y_0);
N_0_tilde=(X_0_tilde'*X_0_tilde);
beta_0=full(N_0_tilde\(X_0_tilde'*Y_0_tilde));

% (Y(1:length(Y_dum),:)-Xy(1:length(Y_dum),:)*B_check)'*(Y(1:length(Y_dum),:)-Xy(1:length(Y_dum),:)*B_check)/( length(Y_dum)*floor_T);
Z=Y(:,(m+1):end);  
Y(:,(m+1):end)=[];
if norm(reshape(beta_0(1:NoParamsC*m),NoParamsC,m)-B_prior)> 1e-8
    if 1-nancorr(beta_0(1:NoParamsC*m),B_prior(:))>1e-8
        error('Dummy variables don''t implement the prior correctly');
    end
end

% Make sure the timing of the instruments is aligned
%                 IV=[NaN(1+m+preLambda(3)*m+k*m,size(IV,2)); IV];
IV=Y(:,end-(mz-1):end);

nu_prior=floor_T; %subtract one so that the posterior DF can just be the updated with the length of Z and gives; posterior DF=#(obs)+prior DF
% incorporates Jeffrey's prior


% Constant terms can be omitted
log_mdens_ex_cons = @(T_all, T_prior, m, n, SSR_all, SSR_prior, N_all, N_prior) ...
        ( (T_all-n)/2 ) * 2 * sum(-log(diag(chol(SSR_all)))) - ( (T_prior-n)/2 ) * 2 * sum(-log(diag(chol(SSR_prior)))) ...
        - (m/2) * 2 * (sum(-log(diag(chol(N_all))))  - sum(-log(diag(chol(N_prior)))) ) ;
    
log_mdens = @(T_all, T_prior, m, n, SSR_all, SSR_prior, N_all, N_prior) ...
    (-m*(T_all-T_prior) / 2) * log(pi) + ...
    ( (T_all-n)/2 ) * 2 * sum(-log(diag(chol(SSR_all)))) - ( (T_prior-n)/2 ) * 2 * sum(-log(diag(chol(SSR_prior)))) ...
        + logMvGamma( (T_all-n) / 2,m) - logMvGamma( (T_prior-n) / 2,m) ...
        - (m/2) * 2 * (sum(-log(diag(chol(N_all))))  - sum(-log(diag(chol(N_prior)))) ) ;