% Gibbs-Sampler for drawing variance and coefficients, given data and, potentially, prior information

rho=rhomax+1; % restrict the posterior to ensure the roots drawn are not too explosive
U_inv=inv(chol(S_0));

if DSGE_lambda>=100 % check if prior is close to putting all weight on the DSGE model
    Bc=reshape(prior_beta(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m);
    K_core=[reshape(Bc(1:m*OriginalData.k, :)', m, m*OriginalData.k); eye((OriginalData.k-1)*m), zeros((OriginalData.k-1)*m, m)];
    rho=abs(max(eig(K_core))); %same as modulus
    vecB_1=prior_beta;
    rr=1;
else
    Vinv_yy=inv(S_0(1:m,1:m)-( S_0(1:m,m+(1:mz))/(S_0(m+(1:mz),m+(1:mz))) )*S_0(m+(1:mz),1:m));
    Vinv_zz=inv(S_0(m+(1:mz),m+(1:mz))-( S_0(m+(1:mz),1:m)/(S_0(1:m,1:m)) )*S_0(1:m,m+(1:mz)));
    Vinv_yz=-S_0(1:m,1:m)\(S_0(1:m,m+(1:mz))*Vinv_zz);
    % Vinv_yz=-inv(S_prior(1:m,1:m))*S_prior(1:m,m+(1:mz))*Vinv_zz;
    Vinv_zy=-Vinv_zz*S_0(m+(1:mz),1:m)/S_0(1:m,1:m);
    Vinv=[Vinv_yy, Vinv_yz; Vinv_zy Vinv_zz];

    NXyXy=OriginalData.Xy'*OriginalData.Xy;
    NXyXz=OriginalData.Xy'*OriginalData.Xz;
    NXzXz=OriginalData.Xz'*OriginalData.Xz;
% 
    XkronX=[kron(Vinv_yy,NXyXy) kron(Vinv_yz,NXyXz ); ...
        kron(Vinv_zy,NXyXz') kron(Vinv_zz,NXzXz)];
    XkronY=zeros(size(X_Gibbs,2), 1);
    % loop over all forecast errors and instruments
    for m_idx1=1:(m+mz)
        % loop over all forecast errors and instruments
        % forecast error
        if m_idx1<=m
            % forecast error x forecast error
                XkronY(NoParamsC*(m_idx1-1)+(1:NoParamsC),1) = ...
                    XkronY(NoParamsC*(m_idx1-1)+(1:NoParamsC),1) +OriginalData.Xy'*OriginalData.Y*Vinv(m_idx1,1:m)';
            % forecast error x instrument
                XkronY(NoParamsC*(m_idx1-1)+(1:NoParamsC),1) = ...
                    XkronY(NoParamsC*(m_idx1-1)+(1:NoParamsC),1) +OriginalData.Xy'*OriginalData.IV*Vinv(m_idx1,m+1:end)';
        else
            % instrument x forecast error
            XkronY(NoParamsC*m+NoXz*(m_idx1-m-1)+(1:NoXz),1) = ...
                XkronY(NoParamsC*m+NoXz*(m_idx1-m-1)+(1:NoXz),1) +OriginalData.Xz'*OriginalData.Y*Vinv(m_idx1,1:m)';
            % instrument x instrument
            XkronY(NoParamsC*m+NoXz*(m_idx1-m-1)+(1:NoXz),1) = ...
                XkronY(NoParamsC*m+NoXz*(m_idx1-m-1)+(1:NoXz),1) +OriginalData.Xz'*OriginalData.IV*Vinv(m_idx1,m+1:end)';
        end
    end
    XkronX=XkronX+prior_N0;
    XkronY=XkronY+prior_N0*prior_beta;
    B_hat=XkronX\XkronY;
    CholXkronX = chol( XkronX ,'lower');

    rr=0;
    while rho>rhomax && rr<rhomax_attempts
        rr=rr+1;
        vecB_1=B_hat(:)+CholXkronX'\randn( NoParamsBeta0 ,1);
        Bc=reshape(vecB_1(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m);
        K_core=[reshape(Bc(1:m*OriginalData.k, :)', m, m*OriginalData.k); eye((OriginalData.k-1)*m), zeros((OriginalData.k-1)*m, m)];
        rho=max(abs(eig(K_core))); %same as modulus
    end            
end
Bz=reshape(vecB_1( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz);

try
    DS=chol(inv(S_posterior(Bc,Bz))/nu_posterior);
catch
    fprintf(2, 'Problem with inverting V \n');                             
    eig(S_posterior(Bc,Bz))
    chol(S_posterior(Bc,Bz))
end

if DSGE_lambda_S>=100 && MY_PRIOR==2 % check if prior is close to putting all weight on the DSGE model
    S_1 = proposal_S_prior;
else
    S_1 = inv(wishrnd(DS'*DS,nu_posterior, DS));
end
if d>NoBurnIns
    GibbsCorr((d-NoBurnIns-1)*GibbsSkip+gg,:)=[corr(vecB_1,vecB_0), corr(S_0(:),S_1(:))];
end
if rr<rhomax_attempts
    S_0=S_1;
    vecB_0=vecB_1;
else
    gg=gg-1;
end
