% Draw structural parameters for a given partition into blocks.
% The parameters already need to be initialized

% Inputs: 
% - current_draw
% - vecB_0
% - S_0
% - prior_beta

% Outputs:
% - proposal_draw;
% - current_draw
% - density_at_current=density_at_proposal;
% - AcceptDraw (percentage accepted)
% - log_prior
% - Prior_rho=proposal_Prior_rho;
% - Prior_eig_S_Prior
% - prior_N0
% - prior_beta
% - S_posterior function


if DSGE_lambda>=100 && DSGE_lambda_S>=100
    % update theta independent of the current value of B, V
    [density_at_current,~,~,~,~,current_S_prior]=...
        my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
            CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, current_draw, ...
            reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, -1, ZeroMean, 0);
elseif DSGE_lambda>=100 
    [density_at_current,~,~,~,~,current_S_prior]=...
        my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
            CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, current_draw, ...
            reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, -2, ZeroMean, 0);
elseif DSGE_lambda_S>=100 
    [density_at_current,~,~,~,~,current_S_prior]=...
        my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
            CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, current_draw, ...
            reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, -3, ZeroMean, 0);
else
    [density_at_current,~,~,~,~,current_S_prior]=...
        my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
            CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, current_draw, ...
            reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, 0, ZeroMean, 0);
end

for bb=1:NoBlocks
    if UseStudentDist
        if d==1
            proposal_draw(BlockIndices{bb})=current_draw(BlockIndices{bb}) + ...
                diag(max([PriorStructure.prior_s(BlockIndices{bb})'*0.1,abs(draw_sol(BlockIndices{bb}))*0.1]')/MH_ScaleFactor)*...
                        randn(numel(BlockIndices{bb}),1)/sqrt(matlab_gamrnd(StudentDF/2, 2, 1, 1) / StudentDF);
        else
            % After Burn-In, use RW proposal density with correlated increments.
            proposal_draw(BlockIndices{bb})=current_draw(BlockIndices{bb})+...
                (Proposal_Cov_chol(BlockIndices{bb},BlockIndices{bb})/MH_ScaleFactor)*randn(numel(BlockIndices{bb}),1)...
                /sqrt(matlab_gamrnd(StudentDF/2, 2, 1, 1) / StudentDF);
        end

    else  
        if d==1
            proposal_draw(BlockIndices{bb})=current_draw(BlockIndices{bb}) + ...
                diag(max([PriorStructure.prior_s(BlockIndices{bb})'*0.1,abs(draw_sol(BlockIndices{bb}))*0.1]')/MH_ScaleFactor)*...
                        randn(numel(BlockIndices{bb}),1);
        else
            % After Burn-In, use RW proposal density with correlated increments.
            proposal_draw(BlockIndices{bb})=current_draw(BlockIndices{bb})+...
                (Proposal_Cov_chol(BlockIndices{bb},BlockIndices{bb})/MH_ScaleFactor)*randn(numel(BlockIndices{bb}),1);
        end
    end
    if DSGE_lambda>=100 && DSGE_lambda_S>=100
        % update theta independent of the current value of B, V
        [density_at_proposal, ~, proposal_prior, proposal_N0, proposal_beta, proposal_S_prior, proposal_Prior_rho, proposal_Prior_eig_S_Prior]=...
            my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
                    CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, proposal_draw, ...
                    reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, -1, ZeroMean,0);
    elseif DSGE_lambda>=100 
        [density_at_proposal, ~, proposal_prior, proposal_N0, proposal_beta, proposal_S_prior, proposal_Prior_rho, proposal_Prior_eig_S_Prior]=...
            my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
                    CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, proposal_draw, ...
                    reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, -2, ZeroMean,0);
    elseif DSGE_lambda_S>=100
        [density_at_proposal, ~, proposal_prior, proposal_N0, proposal_beta, proposal_S_prior, proposal_Prior_rho, proposal_Prior_eig_S_Prior]=...
            my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
                    CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, proposal_draw, ...
                    reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, -3, ZeroMean,0);
    else
        [density_at_proposal, ~, proposal_prior, proposal_N0, proposal_beta, proposal_S_prior, proposal_Prior_rho, proposal_Prior_eig_S_Prior]=...
            my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
                    CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, proposal_draw, ...
                    reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, 0, ZeroMean,0);
    end

    % note that with RW proposal density only the target density matters.
    % Accept new draw with probability=min{1, ratio of densities}
    AcceptDrawPr=min(1, exp(density_at_proposal-density_at_current));                    

    if rand(1)<=AcceptDrawPr  && ~isempty(proposal_N0)
        my_draw=proposal_draw;
        current_draw=proposal_draw;
        density_at_current=density_at_proposal;
        % Update quantities entering the Gibbs sampling: Implicitly this uses that these are sufficient statistics
        % Both for drawing and for computing the LLhood
        AcceptDraw=AcceptDraw+numel(BlockIndices{bb})/PriorStructure.param_no;

        current_S_prior=proposal_S_prior;
        log_prior=proposal_prior;
        Prior_rho=proposal_Prior_rho;
        Prior_eig_S_Prior=proposal_Prior_eig_S_Prior;
        prior_N0=proposal_N0;
        prior_beta=proposal_beta;
        S_posterior=@(By,Bz) (nu_prior/nu_posterior)*proposal_S_prior + (T/nu_posterior)*(1/T)*[((OriginalData.Y-OriginalData.Xy*By)'*(OriginalData.Y-OriginalData.Xy*By)), ...
            ((OriginalData.Y-OriginalData.Xy*By)'*(OriginalData.IV-OriginalData.Xz*Bz)); ((OriginalData.IV-OriginalData.Xz*Bz)'*(OriginalData.Y-OriginalData.Xy*By)), ...
            ((OriginalData.IV-OriginalData.Xz*Bz)'*(OriginalData.IV-OriginalData.Xz*Bz))];
    else
        proposal_draw=current_draw;
        my_draw=current_draw;
    end

if isempty(proposal_N0) && isempty(prior_N0)
	keyboard;
end;

end

if (DSGE_lambda >=100 || DSGE_lambda_S>=100) 
    % this is needed to make sure the rejected proposal does not set the prior that is used subsequently. 
    [~, ~, proposal_prior, proposal_N0, proposal_beta, proposal_S_prior, proposal_Prior_rho, proposal_Prior_eig_S_Prior]=...
        my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile, M_,oo_, OriginalData, ModelStructure, PriorStructure,DSGE_lambda, DSGE_lambda_S, ...
                CoreList(1:(ModelStructure.m+ModelStructure.mP),:), InstrumentList, ShockList, my_draw, ...
                reshape(vecB_0(1:m*(m*OriginalData.k+ConstantTerms),1), m*OriginalData.k+ConstantTerms, m), reshape(vecB_0( (m*(m*OriginalData.k+ConstantTerms)+1):end,1), NoXz, mz), S_0, -1, ZeroMean,0);
end
