function IRFs=my_DynareIRF(DR,MM, hh, VariableList, ShockList)
% DR is the Dynare decision rule DR
% MM is the Dynare structure M_
% hh is the horizon

% structure: Y = A X_{t-1} + B eps; X_t = CC X_{t-1} + DD eps;

%% Get coefficient matrices
% Identify state

if isempty(strmatch('nstatic',fieldnames(DR)))
    DR.nstatic=MM.nstatic;
    DR.npred=MM.npred;
    DR.nboth=MM.nboth;
    DR.nfwrd=MM.nfwrd;
end


States.Names=cellstr(MM.endo_names(DR.order_var(DR.nstatic+(1:(DR.npred+DR.nboth))),:));
States.Index=DR.nstatic+(1:(DR.npred+DR.nboth));

for ll=1:length(States.Names)
    % NOTE: ys is stored in declaration order, ghs2 isn't!
    Coeff.X0(ll, 1)=DR.ys(strmatch(States.Names{ll}, MM.endo_names, 'exact'));
end
% A are the States.Index x all elements as dynare only includes states
Coeff.A=DR.ghx(States.Index, :);
% A are the States.Index x all elements 
Coeff.B=DR.ghu(States.Index, :);

% First, find the indices of the observations, then get coefficient
% matrices from there
for ll=1:length(VariableList)
    try
    VariableIndex(ll)=DR.inv_order_var(strmatch(VariableList{ll}, MM.endo_names, 'exact'));
    
  
        % A are the States.Index x all elements as dynare only includes states
        Coeff.Ay(ll,:)=DR.ghx(VariableIndex(ll), :);
        % A are the States.Index x all elements 
        Coeff.By(ll,:)=DR.ghu(VariableIndex(ll), :);
    catch
        Coeff.Ay(ll,:)=DR.ghx(1, :)*0;
        % A are the States.Index x all elements 
        Coeff.By(ll,:)=DR.ghu(1, :)*0;        
    end
end

%%
%IRF computation
ShockIndex=zeros(length(ShockList),1);
% initialize the states and the variables of interest
IRFs=NaN(length(ShockList),length(VariableIndex),hh);
X=zeros(length(States.Index),hh);
for jj=1:length(ShockList)
%     w0=[[1; 0; 0], zeros(3, 30)];
    % need to change the above!
    ShockIndex(jj)=strmatch(ShockList{jj}, MM.exo_names, 'exact');
%     IRFs{jj}=zeros(length(VariableIndex),hh);
    X(:,1)=Coeff.B(:,ShockIndex(jj));
    IRFs(jj,:,1)=Coeff.By(:,ShockIndex(jj));
    for tt=2:hh
        X(:,tt)=Coeff.A*X(:,tt-1);
        IRFs(jj,:,tt)=Coeff.Ay*X(:,tt-1);
    end
end