function [log_post_kernel_hyper, NewDrawNeeded, my_log_prior, N_prior, beta_prior, S_prior, Prior_rho, Prior_eig_S_Prior, py_VAR]=...
    my_Dynare_DSGEVAR_Prior_fcn_v3( ModFile , M_,oo_, OriginalData, ModelStructure, PriorStructure, DSGE_lambda, ...
            DSGE_lambda_S, OriginalCoreList, InstrumentList, ShockList, NewDraw, Bc_old, Bz_old, V_old, WhichKernel, ZeroMean, DO_VAR)
% DR is the Dynare decision rule DR
% MM is the Dynare structure M_
% VariableList contains the list of variables in the VAR
% VariableList also marks, in column 2, whether the variables have an a priori trend
% OriginalCoreList lists the core and periphery variables
% ShockList list the shocks to be used in the Variance computation 
% InstrumentList lists the instruments and the measurement error in the instruments
% PriorStructure contains the names and characteristics of the structural parameters and their independent priors
% B?_old, V_old are the existing (Gibbs) draws of the VAR parameters

% structure: Y = A X_{t-1} + B eps; X_t = CC X_{t-1} + DD eps;
% VariableList={'y', 1;'lab', 0;'r', 0;'pinf', 0;'dtau_l', 0;'g', 1;'debt', 1};
%% Get coefficient matrices
global my_draw 
my_draw=NewDraw;

my_log_prior=my_DSGE_Parameter_Prior_density(PriorStructure, my_draw) ;

% Check for arguments, set defaults otherwise
if ~exist('OriginalCoreList','var')
    % 1st: variable, 2nd: Trend?, 3rd: Periphery?
    CoreList={'g', 1, 0;'dtau_l', 0, 0;'y', 1, 0;'debt', 1, 0;'r', 0, 0;'pinf', 0, 0; ...
                'lab', 0, 1;'c', 1, 1;'inve', 1, 1;'drev_l', 1, 1;'w',1 , 1  ...
                };
else 
    CoreList=OriginalCoreList;
end
if ~exist('WhichKernel', 'var')
    WhichKernel=0;
end
if ~exist('ZeroMean', 'var')
    ZeroMean=0;
end
if ~exist('DO_VAR', 'var')
    DO_VAR=0;
    py_VAR=NaN;
end

% Relevant only when using a Core-Periphery VAR
[~, idx]=sort(cell2mat(CoreList(:,3)));
CoreList=CoreList(idx,:);

% more defaults
if ~exist('ShockList','var')
    ShockList={'ea';'eg';'em';'epinf';'e_t_l';'eb'};
end
if ~exist('DoMean','var')
    DoMean=0;
end
if ~exist('DO_M_FILE_ONLY', 'var')
    DO_M_FILE_ONLY=0;
end
if ~exist('ModFile', 'var')
    ModFile='SW_narrative_estimation_example.mod';
end
if ~exist('DynareResults','var')
                       %     _ENDO_G_TAX_OBS1_noErr_LAGS000_Data2_g_obs_SigLFix_smooth
    DynareResults='TD_Results_ENDO_G_TAX_OBS1_noErr_LAGS000_Data2_g_obs_SigLFix_smooth';
end
if ~exist('InstrumentList','var')
    %1st entry: which shock. 2nd entry: which impact variable. 3rd entry: relative std of noise
    InstrumentList={'eg','u_g',0.5,1;'e_t_l','u_t_l',0.5,1;'em','ms',0.5,1};
end
if PriorStructure.se_signal_no==ModelStructure.mz
    InstrumentList(:,3)=num2cell(my_draw( PriorStructure.param_no-PriorStructure.se_signal_no-PriorStructure.c_signal_no+(1:PriorStructure.se_signal_no) ) );
end
if PriorStructure.c_signal_no==ModelStructure.mz
    InstrumentList(:,4)=num2cell(my_draw( PriorStructure.param_no-PriorStructure.c_signal_no+(1:PriorStructure.c_signal_no) )  );
end

% Dynare resets the random number generator - so save here and reset later
current_seed=rng;
try
    % clear Dynare input
    oo_.dr.ghx=[];
    % this is a safety-check that the dynare run actually works
    
    if isequal(ModFile(end-3:end),'.mod')
        evalc(['run ' ModFile(1:end-4) '.m']);
    else
        evalc(['run ' ModFile '.m']);
    end
    rng(current_seed);

    NewDrawNeeded=false;

    % % For convenience and clarity, redefine variables. Not necessary...
    if ~exist('n_lags','var')
        n_lags=4;
    end
    if isempty(strmatch('nstatic',fieldnames(oo_.dr)))
        oo_.dr.nstatic=M_.nstatic;
        oo_.dr.npred=M_.npred;
        oo_.dr.nboth=M_.nboth;
        oo_.dr.nfwrd=M_.nfwrd;
    end

    States.Names=cellstr(M_.endo_names(oo_.dr.order_var(oo_.dr.nstatic+(1:(oo_.dr.npred+oo_.dr.nboth))),:));
    States.Index=oo_.dr.nstatic+(1:(oo_.dr.npred+oo_.dr.nboth));

    for ll=1:length(States.Names)
        % NOTE: ys is stored in declaration order, ghs2 isn't!
        Coeff.X0(ll, 1)=oo_.dr.ys(strmatch(States.Names{ll}, M_.endo_names, 'exact'));
    end
    % A are the States.Index x all elements as dynare only includes states
    Coeff.A=oo_.dr.ghx(States.Index, :);
    % A are the States.Index x all elements 
    Coeff.B=oo_.dr.ghu(States.Index, :);

    % ShockIndex{jj} contains the position of the jj'th shock (in ShockList) in M_.exo_names and thus also the corresponding
    % column number of Coeff.B and Coeff.By below
    ShockIndex=zeros(length(ShockList),1);
    for jj=1:length(ShockList)
        ShockIndex(jj)=strmatch(ShockList{jj}, M_.exo_names, 'exact');
    end
    
    VariableIndex=NaN(size(CoreList,1),1);
    for ll=1:size(CoreList,1)
        VariableIndex(ll)=oo_.dr.inv_order_var(strmatch(CoreList{ll,1}, M_.endo_names, 'exact'));

        % A are the States.Index x all elements as dynare only includes states
        Coeff.Ay(ll,:)=oo_.dr.ghx(VariableIndex(ll), :);
        % A are the States.Index x all elements 
        Coeff.By(ll,:)=oo_.dr.ghu(VariableIndex(ll), :);
        Coeff.Mean(ll,:)=oo_.dr.ys(strmatch(CoreList{ll,1}, M_.endo_names, 'exact'), :); %note: this is in declaration order, not DR rule order!
    end
    % How many shocks are there prior to adding measurement error?
    mShocksExisting=size(Coeff.By,2);
    % add columns for noise to state and observation equations
    Coeff.B=[Coeff.B zeros(ModelStructure.mX,size(InstrumentList,1))];
    Coeff.By=[Coeff.By zeros(ModelStructure.m,size(InstrumentList,1))];

    % Add extra elements to ShockIndex to accommodate measurement error for the instruments
    ShockIndex=[ShockIndex;mShocksExisting+(1:size(InstrumentList,1))' ];
    mShocks=size(ShockIndex,1);

    ForecastErrIdx=NaN(ModelStructure.mz,2);
    ShockLoading = NaN(ModelStructure.mz,2);
    % append instruments to core list and add measurement errors
    for ll=1:ModelStructure.mz
        % instrument is unrelated to past states
        Coeff.Ay(end+1,:)=zeros(1,ModelStructure.mX);
        % instrument loads on the structural shock and measurement error - otherwise zero.
        Coeff.By(end+1,:)=zeros(1,size(Coeff.By,2));    
        Coeff.Mean(end+1,1)=0;
        % Here: set the non-zero elements of it.
        % To have a plausible prior for the standard deviation of the instrument, I pick the standard deviation corresponding to
        % the impact the "observed" shock has on the observed time series associated with the shock, e.g. "gc" for "eg"
        ShockLoading(ll,1) = oo_.dr.ghu(oo_.dr.inv_order_var(strmatch(InstrumentList{ll,2}, M_.endo_names, 'exact')), strmatch(InstrumentList{ll,1}, M_.exo_names, 'exact')) ...
                                    * InstrumentList{ll,4} ;
        ShockLoading(ll,2) = InstrumentList{ll,3} * oo_.dr.ghu(oo_.dr.inv_order_var(strmatch(InstrumentList{ll,2}, M_.endo_names, 'exact')), strmatch(InstrumentList{ll,1}, M_.exo_names, 'exact'));
        ForecastErrIdx(ll,:)=[ll, strmatch(InstrumentList{ll,1}, M_.exo_names, 'exact')];
        % Now add the instruments to the CoreList
        CoreList{end+1,1}=InstrumentList{ll,1};
        CoreList{end,2}=0;
        CoreList{end,3}=0;
    end

    % Add G and measurement error covariance matrix here at the end...
    if PriorStructure.NON_DIAGONAL_G
        % potentially non-diagonal "G" matrix, diagonal measurement error
        Coeff.By(end - ModelStructure.mz + (1:ModelStructure.mz),ForecastErrIdx(:,2))= ...
                   my_IV_G_matrix(ModelStructure.mz,PriorStructure, my_draw)*diag(ShockLoading(:,1)) ;
        Coeff.By(end - ModelStructure.mz + (1:ModelStructure.mz),mShocksExisting+(1:ModelStructure.mz)')= ...
                   diag(ShockLoading(:,2));
    else
        % diagonal "G" matrix, potentially non-diagonal measurement error
        Coeff.By(end - ModelStructure.mz + (1:ModelStructure.mz),ForecastErrIdx(:,2))= ...
                   diag(ShockLoading(:,1));
        Coeff.By(end - ModelStructure.mz + (1:ModelStructure.mz),mShocksExisting+(1:ModelStructure.mz)')= ...
                   chol( diag(ShockLoading(:,2)) *my_IV_corr_matrix(ModelStructure.mz,PriorStructure, my_draw)*diag(ShockLoading(:,2)) , 'lower') ;
    end
    
    Coeff.Trend=M_.params(strmatch('ctrend',M_.param_names))*cell2mat(CoreList(:,2)) * (ZeroMean~=0);
    
    if ZeroMean
        Coeff.Mean=Coeff.Mean*0;
    end

%% Get VAR moments given Dynare decision rule

if ~exist('DoPeriphery','var')
    DoPeriphery=uint8(ModelStructure.mP>0);
end

T=OriginalData.T;

lastwarn('');
MomentsOriginalData=OriginalData;
MomentsOriginalData.T=5;
if DoPeriphery
    [B_prior,Prior_rho, S_prior,Prior_eig_S_Prior, ~, NXX, ~, PeripheryB_prior,PeripheryS_prior, PeripheryNXY, PeripheryNXX]=...
        my_VAR_Moments(Coeff, ShockIndex, ForecastErrIdx, MomentsOriginalData, ModelStructure, 1);
else
    [B_prior,Prior_rho, S_prior,Prior_eig_S_Prior, ~, NXX]=...
                    my_VAR_Moments(Coeff, ShockIndex, ForecastErrIdx, MomentsOriginalData, ModelStructure, 0);
end

if ~isreal(eig(NXX)) || min(eig(S_prior))<0
    error('Prior implies negative definite PD matrix');
end

floor_T_B=floor(T*(DSGE_lambda));
floor_T_S=floor(T*(DSGE_lambda_S));
ImplicitXy=chol(NXX, 'upper');
ImplicitXz=ImplicitXy(:,ModelStructure.m*OriginalData.k+1);

n_y=size(ImplicitXy,1);
X_dum=sqrt(floor_T_B/n_y)*ImplicitXy;
Y_dum=[X_dum*B_prior zeros(OriginalData.NoParamsC, ModelStructure.mz)];
Xz_dum=sqrt(floor_T_B/size(ImplicitXz,1))*ImplicitXz;
Z_dum=Xz_dum*zeros(OriginalData.NoXz,ModelStructure.mz);

% Update data with dummy variables for the mean
Y=[Y_dum; [OriginalData.Y(:,1:ModelStructure.m) OriginalData.IV]];
Xy=[X_dum; OriginalData.Xy];
Xz=[Xz_dum; OriginalData.Xz];

%         inv(DumX_l(:,1:(end-2))'*DumX_l(:,1:(end-2)))*(DumX_l(:,1:(end-2))'*DumY_l(:,:))
X_0=sparse([kron(eye(ModelStructure.m), X_dum), zeros(ModelStructure.m*OriginalData.NoParamsC,ModelStructure.mz); ...
    zeros(ModelStructure.mz*OriginalData.NoParamsC,ModelStructure.m*OriginalData.NoParamsC), kron(eye(ModelStructure.mz),Xz_dum)]);
Y_0=Y_dum(:);
try
    invCholS=inv(chol(S_prior));
catch
    if min(eig(S_prior))>0
        invCholS=(S_prior)^(-1/2);    
    else
        error('negative definite prior variance-covariance');
    end
end
CholS=chol(S_prior, 'lower');
%% The code in this section is for testing only
% the actual output is not used when the test is passed
% when the test is not passed, the draw is rejected.

% piecewise inverse of V matrix
Vinv_yy=inv(S_prior(1:ModelStructure.m,1:ModelStructure.m)-( S_prior(1:ModelStructure.m,ModelStructure.m+(1:ModelStructure.mz)) ...
    /(S_prior(ModelStructure.m+(1:ModelStructure.mz),ModelStructure.m+(1:ModelStructure.mz))) )*S_prior(ModelStructure.m+(1:ModelStructure.mz),1:ModelStructure.m));
Vinv_zz=inv(S_prior(ModelStructure.m+(1:ModelStructure.mz),ModelStructure.m+(1:ModelStructure.mz))-...
    ( S_prior(ModelStructure.m+(1:ModelStructure.mz),1:ModelStructure.m)/(S_prior(1:ModelStructure.m,1:ModelStructure.m)) )*S_prior(1:ModelStructure.m,ModelStructure.m+(1:ModelStructure.mz)));
Vinv_yz=-S_prior(1:ModelStructure.m,1:ModelStructure.m)\(S_prior(1:ModelStructure.m,ModelStructure.m+(1:ModelStructure.mz))*Vinv_zz);
Vinv_zy=-Vinv_zz*S_prior(ModelStructure.m+(1:ModelStructure.mz),1:ModelStructure.m)/S_prior(1:ModelStructure.m,1:ModelStructure.m);
Vinv=[Vinv_yy, Vinv_yz; Vinv_zy, Vinv_zz];


[~, warning_id] = lastwarn;
if isequal(warning_id,'MATLAB:nearlySingularMatrix');
    fprintf(2, 'Nearly singular matrix \n');
end
warning('off','MATLAB:nearlySingularMatrix');

NXyXy0=X_dum'*X_dum;
NXyXz0=X_dum'*Xz_dum;
NXzXz0=Xz_dum'*Xz_dum;

N_prior=[kron(Vinv_yy,NXyXy0) kron(Vinv_yz,NXyXz0 ); ...
        kron(Vinv_zy,NXyXz0') kron(Vinv_zz,NXzXz0)];
XkronY=zeros(size(N_prior,1), 1);
% loop over all forecast errors and instruments
for m_idx1=1:(ModelStructure.m+ModelStructure.mz)
    % loop over all forecast errors and instruments
    % forecast error
    if m_idx1<=ModelStructure.m
        % forecast error x forecast error
            XkronY(OriginalData.NoParamsC*(m_idx1-1)+(1:OriginalData.NoParamsC),1) = ...
                XkronY(OriginalData.NoParamsC*(m_idx1-1)+(1:OriginalData.NoParamsC),1) +X_dum'*Y_dum(:,1:ModelStructure.m)*Vinv(m_idx1,1:ModelStructure.m)';
        % forecast error x instrument
            XkronY(OriginalData.NoParamsC*(m_idx1-1)+(1:OriginalData.NoParamsC),1) = ...
                XkronY(OriginalData.NoParamsC*(m_idx1-1)+(1:OriginalData.NoParamsC),1) +X_dum'*Z_dum*Vinv(m_idx1,ModelStructure.m+1:end)';
    else
        % instrument x forecast error
        XkronY(OriginalData.NoParamsC*ModelStructure.m+OriginalData.NoXz*(m_idx1-ModelStructure.m-1)+(1:OriginalData.NoXz),1) = ...
            XkronY(OriginalData.NoParamsC*ModelStructure.m+OriginalData.NoXz*(m_idx1-ModelStructure.m-1)+(1:OriginalData.NoXz),1) +Xz_dum'*Y_dum(:,1:ModelStructure.m)*Vinv(m_idx1,1:ModelStructure.m)';
        % instrument x instrument
        XkronY(OriginalData.NoParamsC*ModelStructure.m+OriginalData.NoXz*(m_idx1-ModelStructure.m-1)+(1:OriginalData.NoXz),1) = ...
            XkronY(OriginalData.NoParamsC*ModelStructure.m+OriginalData.NoXz*(m_idx1-ModelStructure.m-1)+(1:OriginalData.NoXz),1) +Xz_dum'*Z_dum*Vinv(m_idx1,ModelStructure.m+1:end)';
    end
end    

beta_prior=N_prior\XkronY;

lastwarn('');
warning('on','MATLAB:nearlySingularMatrix');

if norm(reshape(beta_prior(1:OriginalData.NoParamsC*ModelStructure.m),OriginalData.NoParamsC,ModelStructure.m)-B_prior)/sqrt(numel(B_prior))> 1e-8
    if 1-nancorr(beta_prior(1:OriginalData.NoParamsC*ModelStructure.m),B_prior(:))>1e-3
        fprintf([num2str(norm(reshape(beta_prior(1:OriginalData.NoParamsC*ModelStructure.m),OriginalData.NoParamsC,ModelStructure.m)-B_prior)/sqrt(numel(B_prior))) , ...
            ' :: ' num2str(1-nancorr(beta_prior(1:OriginalData.NoParamsC*ModelStructure.m),B_prior(:))) ' :: ' ]);
        if nancorr(beta_prior(1:OriginalData.NoParamsC*ModelStructure.m),B_prior(:))>=0.99
            fprintf(2, 'Dummy variables don''t implement the prior correctly \n');
        else
            error('Dummy variables don''t implement the prior correctly');            
        end
    end
end

%%
Z=Y(:,(ModelStructure.m+1):end);  
Y_dum(:,(ModelStructure.m+1):end)=[]; 
Y(:,(ModelStructure.m+1):end)=[];    

S_posterior=@(By,Bz) (floor_T_S/(OriginalData.T+floor_T_S))*S_prior + ...
            (OriginalData.T/(OriginalData.T+floor_T_S))*(1/OriginalData.T)*[((Y-Xy*By)'*(Y-Xy*By)), ...
                            ((Y-Xy*By)'*(Z-Xz*Bz)); ((Z-Xz*Bz)'*(Y-Xy*By)), ((Z-Xz*Bz)'*(Z-Xz*Bz))];

SSR_DSGE=@(By,Bz) ...
            [((Y_dum-X_dum*By)'*(Y_dum-X_dum*By)), ...
                            ((Y_dum-X_dum*By)'*(Z_dum-Xz_dum*Bz)); ((Z_dum-Xz_dum*Bz)'*(Y_dum-X_dum*By)), ((Z_dum-Xz_dum*Bz)'*(Z_dum-Xz_dum*Bz))];
% S_1: variance matrix of interest 
% Bc, Bz: coefficient matrices of interest
switch WhichKernel
    case 1
        myBy=(Xy'*Xy)\(Xy'*Y);
        myBz=(Xz'*Xz)\(Xz'*Z);
        V_old=S_posterior(myBy,myBz);
        log_post_kernel_hyper = sum(my_log_prior) + ...
            ( -(OriginalData.T/2)*(ModelStructure.m+ModelStructure.mz)*log(2*pi) ...
            - ((OriginalData.T+floor_T_S)/2)*2*sum(log(diag(chol(V_old)))) ...
            + ((floor_T_S+1+ModelStructure.m+ModelStructure.mz)/2)*2*sum(log(diag(chol(S_prior)))) ...
            - ((OriginalData.T+floor_T_S)/2)*(ModelStructure.m+ModelStructure.mz) );
    case 2
        myBy=(Xy'*Xy)\(Xy'*Y);
        myBz=(Xz'*Xz)\(Xz'*Z);
        V_old=S_posterior(myBy,myBz);
        S_posterior=@(By,Bz) ...
                    (OriginalData.T/(OriginalData.T+floor_T_S))*(1/OriginalData.T)*[((OriginalData.Y-OriginalData.Xy*By)'*(OriginalData.Y-OriginalData.Xy*By)), ...
                                    ((OriginalData.Y-OriginalData.Xy*By)'*(OriginalData.Z-OriginalData.Xz*Bz)); ...
                            ((OriginalData.Z-OriginalData.Xz*Bz)'*(OriginalData.Y-OriginalData.Xy*By)), ((OriginalData.Z-OriginalData.Xz*Bz)'*(OriginalData.Z-OriginalData.Xz*Bz))];
        log_post_kernel_hyper = sum(my_log_prior) + ...
            ( -(OriginalData.T/2)*(ModelStructure.m+ModelStructure.mz)*log(2*pi) ...
            + ((floor_T_S+1+ModelStructure.m+ModelStructure.mz)/2)*2*sum(log(diag(chol(S_prior)))) ...
            - ((OriginalData.T+floor_T_S)/2)*2*sum(log(diag(chol(V_old)))) ...
            - ((OriginalData.T+floor_T_S)/2)*(ModelStructure.m+ModelStructure.mz) );
    case -1
        % both B and V are fixed, so update via the likelihood only. 
        log_post_kernel_hyper = sum(my_log_prior) + ...
            - (OriginalData.T/2)*2*sum(log(diag(chol( S_prior )))) ...
            - (1/2) * trace(S_prior\([OriginalData.Y - OriginalData.Xy * B_prior , OriginalData.IV - OriginalData.Xz * zeros(OriginalData.NoXz, ModelStructure.mz)]'...
                *[OriginalData.Y - OriginalData.Xy * B_prior , OriginalData.IV - OriginalData.Xz * zeros(OriginalData.NoXz, ModelStructure.mz)])) ...
            ;
    case -2
        % B fixed, so update via the likelihood and V-prior only. 
            % from the likelihood: exponential enters, evaluated at B_prior
            % from the prior, all terms involving S_prior enter - except that S_0(B) equals zero
        log_post_kernel_hyper = sum(my_log_prior) + ...
            - (1/2) * trace(V_old\(floor_T_S*S_prior)) ... prior
            + (floor_T_S/2)*2*sum(log(diag(chol( floor_T_S*S_prior )))) ... prior
            - ((floor_T_S-ModelStructure.m-ModelStructure.mz-1)/2)*2*sum(log(diag(chol(V_old)))) ... likelihood
            - (1/2) * trace(V_old\([OriginalData.Y - OriginalData.Xy * B_prior , OriginalData.IV - OriginalData.Xz * zeros(OriginalData.NoXz, ModelStructure.mz)]'... likelihood
                *[OriginalData.Y - OriginalData.Xy * B_prior , OriginalData.IV - OriginalData.Xz * zeros(OriginalData.NoXz, ModelStructure.mz)])) ...
            ;

    case -3
        % V fixed, so update via the likelihood and B-prior only. 
            % from the likelihood: everything enters, but evaluated at B_old
            % from the prior, all terms involving S_prior enter - except that S_0(B) equals zero
        log_post_kernel_hyper = sum(my_log_prior) + ...
            - (1/2) * trace(S_prior\([Y_dum - X_dum * Bc_old, Z_dum - Xz_dum * Bz_old]'*[Y_dum - X_dum * Bc_old, Z_dum - Xz_dum * Bz_old])) ... prior: beta-component
            - (OriginalData.T/2)*2*sum(log(diag(chol( S_prior )))) ... likelihood: determinant-term at V=S_prior 
            - (1/2) * trace(S_prior\([OriginalData.Y - OriginalData.Xy * Bc_old , OriginalData.IV - OriginalData.Xz * Bz_old]'... likelihood: exponential at V=S_prior
                *[OriginalData.Y - OriginalData.Xy * Bc_old , OriginalData.IV - OriginalData.Xz * Bz_old])) ...
            ;
    case -999   
        log_post_kernel_hyper = [];
    otherwise 

        log_post_kernel_hyper = sum(my_log_prior) + ...
            ( -( length(beta_prior) / 2) * log(2*pi) ... inconsistent inclusion of this constant, exclusion of Wishart constant
            + (floor_T_S/2)*2*sum(log(diag(chol( floor_T_S*S_prior )))) ... ok
            - ((floor_T_S-ModelStructure.m-ModelStructure.mz-1)/2)*2*sum(log(diag(chol(V_old)))) ... ok
            + (1/2)*2*sum(log(diag(chol( X_0'*kron(Vinv,eye(n_y))*X_0 )))) ... % ok - to highlight the similarity with the trace algebra that is based on this expression
            - (1/2) * trace(V_old\(floor_T_S*S_prior)) ...
            - (1/2) * trace(S_prior\([Y_dum - X_dum * Bc_old, Z_dum - Xz_dum * Bz_old]'*[Y_dum - X_dum * Bc_old, Z_dum - Xz_dum * Bz_old])) ...
            );
end

if abs(Prior_rho)>OriginalData.rhomax
    log_post_kernel_hyper=log_post_kernel_hyper-(abs(Prior_rho)-OriginalData.rhomax)*max(abs(log_post_kernel_hyper),1e3);
end

% flag badly conditioned draws
[~, warning_id] = lastwarn;
if isequal(warning_id,'MATLAB:nearlySingularMatrix');
    fprintf(2, 'Nearly singular matrix \n');
end
lastwarn('');
if log_post_kernel_hyper==Inf
    log_post_kernel_hyper=NaN;
    error('infinite posterior density kernel');
end

if max(abs(eig( ...
    Coeff.A-Coeff.B(:,ShockIndex(1:ModelStructure.m)) * inv(Coeff.By(1:ModelStructure.m,ShockIndex(1:ModelStructure.m))) * Coeff.Ay(1:ModelStructure.m,:) ...
            )))>=1
    error('ABCD condition violated');
end

% Allow for Periphery
if DoPeriphery==1 || DoPeriphery==-1
    error('Periphery currently not implemented');
end

% ---------------------------------------------------------------------
%                      Compute VAR likelihood
% ---------------------------------------------------------------------

if DO_VAR
    TryJeffreys=1;

    % need to evaluate these at the posterior means
    VAR_By= (NXyXy0+ OriginalData.Xy'*OriginalData.Xy) \ ...
        (X_dum'*Y_dum + OriginalData.Xy'*OriginalData.Y);

    SSR_prior= floor_T_S*S_prior(1:ModelStructure.m,1:ModelStructure.m) ;
    SSR_all= SSR_prior + (Y_dum-X_dum*VAR_By)'*(Y_dum-X_dum*VAR_By)+ (OriginalData.Y-OriginalData.Xy*VAR_By)'*(OriginalData.Y-OriginalData.Xy*VAR_By) ;
    py_VAR = sum(my_log_prior) + ... same 
        - ( ModelStructure.m * T / 2) * log(2*pi) + (T * ModelStructure.m / 2) * log(2)  ... different: also includes data LL
            + logMvGamma( (T+floor_T_S) / 2 , ModelStructure.m) ... different: also includes prior info
            - logMvGamma( (floor_T_S) / 2 , ModelStructure.m) ... different: also includes prior info
            - (T+floor_T_S-OriginalData.NoParamsC*(TryJeffreys~=1))/2 * logdet(SSR_all)  ... different: V replaced by SSR_all and df differ
            + (floor_T_S-OriginalData.NoParamsC*(TryJeffreys~=1))/2 * logdet(SSR_prior)  ... same + prior beta-discrepancy
            - (ModelStructure.m/2) * logdet(NXyXy0+ OriginalData.Xy'*OriginalData.Xy) ... % posterior NXX
            + (ModelStructure.m/2) * logdet(NXyXy0); % prior NXX
end

    
catch err
    fprintf(2,['The following error occurred in ' mfilename ': ' err.message '\n']);
    log_post_kernel_hyper=-Inf;
    N_prior=[];
    beta_prior=[];
    S_prior=[];
    Prior_rho=[];
    Prior_eig_S_Prior=[];
    rng(current_seed);
    NewDrawNeeded=true;
end
