% [BB,SIG,NXX,PeripheryNXX]=my_Dynare_DSGEVAR_Prior( DynareResults,CoreList, ShockList, n_lags, T)
% DR is the Dynare decision rule DR
% MM is the Dynare structure M_
% VariableList contains the list of variables in the VAR
% VariableList also marks, in column 2, whether the variables have an a priori trend
% ShockList list the shocks to be used in the Variance computation 

% structure: Y = A X_{t-1} + B eps; X_t = CC X_{t-1} + DD eps;
% VariableList={'y', 1;'lab', 0;'r', 0;'pinf', 0;'dtau_l', 0;'g', 1;'debt', 1};
%% Get coefficient matrices
% Identify state
% clear variables

clear Coeff 

if ~exist('PriorStructure', 'var')
    load VAR/tmp;
end

if ~exist('m','var')
    % 1st: variable, 2nd: Trend?, 3rd: Periphery?
    CoreList={'g', 1, 0;'dtau_l', 0, 0;'y', 1, 0;'debt', 1, 0;'r', 0, 0;'pinf', 0, 0; ...
                'lab', 0, 1;'c', 1, 1;'inve', 1, 1;'drev_l', 1, 1;'w',1 , 1  ...
                };

    if ~exist('ShockList','var')
        ShockList={'ea';'eg';'em';'epinf';'e_t_l';'eb'};
    end
else
    
    CoreList=cell(m,4);
    ShockList=cell(m,1);
    InstrumentList=cell(length(CholToIV),3);
    % Use DSGE_VAR_MAP to sort variables into core and periphery, depending on their position in "Labels"
    for mm=1:m
        CoreList(mm,1:2)=DSGE_VAR_MAP( strcmp(Labels{mm},DSGE_VAR_MAP(:,2) ),[1 3]);
        CoreList{mm,3}=0;
        CoreList{mm,4}=DSGE_VAR_MAP( strcmp(Labels{mm},DSGE_VAR_MAP(:,2)),5);
        ShockList(mm,1)=DSGE_VAR_MAP( strcmp(Labels{mm},DSGE_VAR_MAP(:,2)),4);
    end
    for mm=m+(1:mP)
        CoreList(mm,1:2)=DSGE_VAR_MAP( strcmp(Labels{mm},DSGE_VAR_MAP(:,2) ),[1 3]);
        CoreList{mm,3}=1;
    end
    InstrumentList(:,1)=ShockList(CholToIV);
    InstrumentList(:,2)=[CoreList{CholToIV,4}];
    for mm=1:length(CholToIV)
        InstrumentList{mm,3}=0.5;    % noise to structural standard deviation ratio
        InstrumentList{mm,4}=1;      % loading on structural shock
    end
end
% Sort list to make sure that Periphery is cut off.
ModelStructure.mP=sum(cell2mat(CoreList(:,3)));
if ~exist('m', 'var')
    m=size(CoreList,1)-ModelStructure.mP;
end
ModelStructure.m=m;
ModelStructure.ConstantTerms=ConstantTerms;

[~, idx]=sort(cell2mat(CoreList(:,3)));
CoreList=CoreList(idx,:);

if ~exist('DoMean','var')
    DoMean=0;
end
if ~exist('DO_M_FILE_ONLY', 'var')
    DO_M_FILE_ONLY=0;
end
if ~exist('ModFile', 'var')
    ModFile='SW_narrative_estimation_example.mod';
end
if ~exist('DynareResults','var')
    DynareResults='TD_Results_ENDO_G_TAX_OBS1_noErr_LAGS000_Data2_g_obs_SigLFix_smooth';
end
if ~exist('InstrumentList','var')
    %1st entry: which shock. 2nd entry: which impact variable. 3rd entry: relative std of noise
    InstrumentList={'eg','g',0.5;'e_t_l','dtau_l',0.5;'em','r',0.5};
    ModelStructure.mz=size(InstrumentList,1);
end
if PriorStructure.se_signal_no==size(InstrumentList,1)
    InstrumentList(:,3)=num2cell(my_draw( PriorStructure.param_no-PriorStructure.se_signal_no-PriorStructure.c_signal_no+(1:PriorStructure.se_signal_no) ) );
end
if PriorStructure.c_signal_no==size(InstrumentList,1)
    InstrumentList(:,4)=num2cell(my_draw( PriorStructure.param_no-PriorStructure.c_signal_no+(1:PriorStructure.c_signal_no) )  );
end

if DoMean==1
    load(DynareResults);

    param_names=fieldnames(oo_.posterior_mode.parameters);
    oo_.prior.mean(strmatch('cmap',param_names,'exact'))=0;
    oo_.prior.mean(strmatch('cmaw',param_names,'exact'))=0;
    
    FileName='TD_Results_ENDO_G_TAX_OBS1_noErr_LAGS01_Data2_g_obs_SigLFix_smooth';
    mySubParamNames=param_names;
    mySubParamValues={oo_.prior.mean};
    UseMean=1;

    current_seed=rng;

    my_DynarePosterior_m;
    rng(current_seed);
else
    
    if ~exist('DONT_DRAW','var')
            my_DSGE_Parameter_Prior_draw_m;        
    else
        if DONT_DRAW~=1
            my_DSGE_Parameter_Prior_draw_m;
        end
    end

    % Dynare resets the random number generator - so save here and reset later
    current_seed=rng;
    no_error=false;
    while ~no_error
        try
            if DO_M_FILE_ONLY
                evalc(['run ' ModFile(1:end-4) '.m']);
            else
                evalc(['dynare ' ModFile ' noclearall nolog ']);
            end
            no_error=true;
        catch err
            display(err.message)
            rng(current_seed);
            my_DSGE_Parameter_Prior_draw_m;        
            current_seed=rng;
        end
    end
    rng(current_seed);
end
% set flag for outer loop
NewDrawNeeded=false;

% % For convenience and clarity, redefine variables. Not necessary...
if ~exist('n_lags','var')
    n_lags=4;
end
if isempty(strmatch('nstatic',fieldnames(oo_.dr)))
    oo_.dr.nstatic=M_.nstatic;
    oo_.dr.npred=M_.npred;
    oo_.dr.nboth=M_.nboth;
    oo_.dr.nfwrd=M_.nfwrd;
end


States.Names=cellstr(M_.endo_names(oo_.dr.order_var(oo_.dr.nstatic+(1:(oo_.dr.npred+oo_.dr.nboth))),:));
States.Index=oo_.dr.nstatic+(1:(oo_.dr.npred+oo_.dr.nboth));

for ll=1:length(States.Names)
    % NOTE: ys is stored in declaration order, ghs2 isn't!
    Coeff.X0(ll, 1)=oo_.dr.ys(strmatch(States.Names{ll}, M_.endo_names, 'exact'));
end
% A are the States.Index x all elements as dynare only includes states
Coeff.A=oo_.dr.ghx(States.Index, :);
% A are the States.Index x all elements 
Coeff.B=oo_.dr.ghu(States.Index, :);

ModelStructure.mX=size(Coeff.A,1);

% ShockIndex{jj} contains the position of the jj'th shock (in ShockList) in M_.exo_names and thus also the corresponding
% column number of Coeff.B and Coeff.By below
ShockIndex=zeros(length(ShockList),1);
for jj=1:length(ShockList)
    ShockIndex(jj)=strmatch(ShockList{jj}, M_.exo_names, 'exact');
end

VariableIndex=NaN(size(CoreList,1),1);
for ll=1:size(CoreList,1)
    VariableIndex(ll)=oo_.dr.inv_order_var(strmatch(CoreList{ll,1}, M_.endo_names, 'exact'));
    
    % A are the States.Index x all elements as dynare only includes states
    Coeff.Ay(ll,:)=oo_.dr.ghx(VariableIndex(ll), :);
    % A are the States.Index x all elements 
    Coeff.By(ll,:)=oo_.dr.ghu(VariableIndex(ll), :);
    Coeff.Mean(ll,:)=oo_.dr.ys(strmatch(CoreList{ll,1}, M_.endo_names, 'exact'), :); %note: this is in declaration order, not DR rule order!
end
% total numbers of endogenous variables
ModelStructure.m=size(Coeff.Ay,1);
% How many shocks are there prior to adding measurement error?
mShocksExisting=size(Coeff.By,2);
% add columns for noise to state and observation equations
Coeff.B=[Coeff.B zeros(ModelStructure.mX,size(InstrumentList,1))];
Coeff.By=[Coeff.By zeros(ModelStructure.m,size(InstrumentList,1))];



% Add extra elements to ShockIndex to accommodate measurement error for the instruments
ShockIndex=[ShockIndex;mShocksExisting+(1:size(InstrumentList,1))' ];
mShocks=size(ShockIndex,1);
ModelStructure.mz=size(InstrumentList,1);

% For each structural shock, list the corresponding forecast errors
ForecastErrIdx=NaN(ModelStructure.mz,2);
% append instruments to core list and add measurement errors
for ll=1:ModelStructure.mz
    % instrument is unrelated to past states
    Coeff.Ay(end+1,:)=zeros(1,ModelStructure.mX);
    % instrument loads on the structural shock and measurement error - otherwise zero.
    Coeff.By(end+1,:)=zeros(1,size(Coeff.By,2));    
    Coeff.Mean(end+1,1)=0;
    % Here: set the non-zero elements of it.
    Coeff.By(end,[strmatch(InstrumentList{ll,1}, M_.exo_names, 'exact') mShocksExisting+ll])= ...
        oo_.dr.ghu(oo_.dr.inv_order_var(strmatch(InstrumentList{ll,2}, M_.endo_names, 'exact')), strmatch(InstrumentList{ll,1}, M_.exo_names, 'exact'))*[1 InstrumentList{ll,3}];
    ForecastErrIdx(ll,:)=[ll, strmatch(InstrumentList{ll,1}, M_.exo_names, 'exact')];
    % Now add the instruments to the CoreList
    CoreList{end+1,1}=InstrumentList{ll,1};
    CoreList{end,2}=0;
    CoreList{end,3}=0;
end

ModelStructure.mY=size(Coeff.Ay,1);

Coeff.Trend=M_.params(strmatch('ctrend',M_.param_names))*cell2mat(CoreList(:,2)) * (ZeroMean~=0);

if ZeroMean
    Coeff.Mean=Coeff.Mean*0;
end


%%
%Prior computation

if ~exist('T','var')
    T=1e2;
end
if ~exist('DoPeriphery','var')
    DoPeriphery=uint8(ModelStructure.mP>0);
end
 
    [B_prior,Prior_rho, S_prior,Prior_eig_S_Prior, NXY, NXX, DSGE_Rotate]=my_VAR_Moments(Coeff,ShockIndex, ForecastErrIdx,OriginalData,ModelStructure, DoPeriphery);
    if ~isreal(eig(NXX)) || min(eig(S_prior))<0 
        NewDrawNeeded=true;
    end

% check ABCD condition
if max(abs(eig( ...
        Coeff.A-Coeff.B(:,ShockIndex(1:ModelStructure.m)) * inv(Coeff.By(1:ModelStructure.m,ShockIndex(1:ModelStructure.m))) * Coeff.Ay(1:ModelStructure.m,:) ...
                ))) >= 1
    NewDrawNeeded=true;
end

%%
DO_CHECKS=0;
if DO_CHECKS
    CoreIV_idx=[ Core_idx(1:m) (ModelStructure.m+ModelStructure.mP)+(1:ModelStructure.mz)];
    CoreLHS_idx=Core_idx;

    BurnIn=1e4;
    T_sim=1e5;
    rng(5);

    X_sim=zeros(ModelStructure.mX,T_sim+BurnIn+1);
    Y_sim=zeros(ModelStructure.m,T_sim+BurnIn+1);
    for tt=1:(T_sim+BurnIn)
        shocks=randn(ModelStructure.m,1);
        X_sim(:,tt+1) = Coeff.A*X_sim(:,tt) + Coeff.B(:,ShockIndex(1:ModelStructure.m))*shocks ;
        Y_sim(:,tt+1) = Coeff.Ay(Core_idx(1:ModelStructure.m),:)*X_sim(:,tt) +  Coeff.By(Core_idx(1:ModelStructure.m),ShockIndex(1:ModelStructure.m))*shocks;
    end

    Y_sim=Y_sim;
    switch n_lags
        case 4
            SimReg=[Y_sim(:,BurnIn+2-1:end-1)' Y_sim(:,BurnIn+2-2:end-2)' Y_sim(:,BurnIn+2-3:end-3)' Y_sim(:,BurnIn+2-4:end-4)' ones(T_sim,1) (1:T_sim)'];
        case 2
            SimReg=[Y_sim(:,BurnIn+2-1:end-1)' Y_sim(:,BurnIn+2-2:end-2)' ones(T_sim,1) (1:T_sim)' ];
        case 1
            SimReg=[Y_sim(:,BurnIn+2-1:end-1)' ones(T_sim,1) (1:T_sim)'];
        otherwise
            error('simulated code only supports 1 and 4 lags');
    end
    SiModelStructure.mXX=SimReg'*SimReg/T_sim;
    SiModelStructure.mXY=SimReg'*Y_sim(:,BurnIn+2:end)'/T_sim;
    SimBB=SiModelStructure.mXX\SiModelStructure.mXY
    SimK=[SimBB(1:ModelStructure.m*n_lags,:)'; eye(ModelStructure.m*(n_lags-1),ModelStructure.m*n_lags)];
    [sort(abs(eig(K))) sort(abs(eig(SimK)))]

    SimSIG=cov(Y_sim(:,BurnIn+2:end)'-SimReg*SimBB)
    
    if DoPeriphery
        error('Code not set up for Core-Periphery estimation')
    end
end
