function [B_prior,Prior_rho, S_prior,Prior_eig_S_Prior, NXY, NXX,DSGE_Rotate, PeripheryB_prior, PeripheryS_prior, PeripheryNXY, PeripheryNXX]=...
    my_VAR_Moments(Coeff, ShockIndex,ForecastErrIdx,OriginalData,ModelStructure,DoPeriphery)
% Simulate VAR moments given DSGE coefficients and information about the model structure
% - Coeff contains Dynare model solution
% - ShockIndex is a vector of the indices of structural shocks used in the estimation
% - ForecastErrIdx is a matrix [#structural shock, index of corresp. f'cast error; ...]
% - OriginalData contains information about the actual data set, such as number of observations
% - ModelStructure contains information about the number of core and periphery variables
% - DoPeriphery is an (optional) binary entry controlling whether Periphery is computed.
  
% for modeling deterministic components
SumTSq_T=@(m,n,T) (T+1) *(2*T+1) / 6 - (m+n)*(T+1)/2 +m*n; % sum( (0):(T-1))/T-SumT_T(1,T) 
SumT_T=@(m,T) (T+1-2*m) /2; % sum( ((-1):(T-2)).*((1):(T)) )/T-SumTSq_T(0,2,T) 

if ~exist('DoPeriphery','var')
    DoPeriphery=uint8(ModelStructure.mP>0);
end;

if DoPeriphery==1
    PeripheryNXX=[];
    % need to append measurement error shocks to avoid stochastic singularity
%     display('better: put measurement error in peripheral variables in DSGE model');
    ShockIndex=[ShockIndex; size(Coeff.B,2)+(1:ModelStructure.mP)'];
    Coeff.B=[Coeff.B zeros(ModelStructure.mX, ModelStructure.mP)];
    % Since loadings for non-0Periphery are zero, one can always use the entire "ShockIndex"
    Coeff.By=[Coeff.By [zeros(ModelStructure.m, ModelStructure.mP); diag(mean(abs(Coeff.By(ModelStructure.m+(1:ModelStructure.mP),:)),2)); zeros(ModelStructure.mz, ModelStructure.mP)]];    
else
    ModelStructure.mP=0; % Without periphery, there should be no peripheral variables to take care of
end;

% ABCD_ok=1;
for pp=0 : DoPeriphery
    if pp==0
%         if DO_SEPARATE_IV==0
%             CoreIV_idx=[1:ModelStructure.m (ModelStructure.m+ModelStructure.mP)+(1:ModelStructure.mz)];
%             Core_idx=CoreIV_idx;
%     %         CoreIV_idx=[(mC+ModelStructure.mP)+(1:ModelStructure.mz)];
%             CoreLHS_idx=CoreIV_idx;
%         else
            Core_idx=1:ModelStructure.m;
            CoreIV_idx=[1:ModelStructure.m (ModelStructure.m+ModelStructure.mP)+(1:ModelStructure.mz)];
    %         CoreIV_idx=[(mC+ModelStructure.mP)+(1:ModelStructure.mz)];
            CoreLHS_idx=Core_idx;
%         end;

% %             R=inv(chol)*ghuForecastErrIdx
%             Chol_cond_var=chol(Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2))*Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2))', 'lower');
%             DSGE_Rotate=Chol_cond_var\Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2));
    else
        % include everything as regressor: later add contemporaneous regressors
        Core_idx=1:(ModelStructure.m+ModelStructure.mP);
        CoreLHS_idx=ModelStructure.m+(1:ModelStructure.mP);
        CoreIV_idx=1:ModelStructure.mY;
    end;
%     error('try excluding the instruments from the regressors, but include them in the variance matrix calculation')
    CurrentY=length(Core_idx);
    LHS_Y=length(CoreLHS_idx);
    
    % Initialize variance of RHS variables and corresponding covariance
    RegressorVar=zeros(CurrentY*OriginalData.k+ModelStructure.ConstantTerms+pp*ModelStructure.m,CurrentY*OriginalData.k+ModelStructure.ConstantTerms+pp*ModelStructure.m);
    RegressorVarSimple=zeros(CurrentY*OriginalData.k + 0 +pp*ModelStructure.m, CurrentY*OriginalData.k + 0 +pp*ModelStructure.m);
    RegressorCov=zeros(CurrentY*OriginalData.k+ModelStructure.ConstantTerms+pp*ModelStructure.m,LHS_Y);
    RegressorCovSimple=zeros(CurrentY*OriginalData.k+ModelStructure.ConstantTerms+pp*ModelStructure.m,LHS_Y);    

    % Below, X is the hidden state of the model
    % vectorized unconditional variance of X: (I-A \otimes A)^{-1} vec(BB')
    vec_VarX = (eye(ModelStructure.mX^2)- kron(Coeff.A,Coeff.A)) \ reshape(Coeff.B(:,ShockIndex)*Coeff.B(:,ShockIndex)', ModelStructure.mX^2,1);
    % bring into normal shape
    VarX=reshape(vec_VarX,ModelStructure.mX,ModelStructure.mX);
    % unconditional variance of Y via unconditional variance of X and increment
    
%     % Check ABCD condition
%     if max(abs(eig(Coeff.A-Coeff.B(:,ShockIndex(1:ModelStructure.m)) * inv(Coeff.By(Core_idx,ShockIndex(1:ModelStructure.m))) * Coeff.Ay(Core_idx,:)))) >=1
%         ABCD_ok=0;
%     end;

    VarY=Coeff.Ay(Core_idx,:) * VarX * Coeff.Ay(Core_idx,:)' + Coeff.By(Core_idx,ShockIndex)*Coeff.By(Core_idx,ShockIndex)';
    % unconditional variance of instruments (no controls) including endogenous regressors
    if ModelStructure.mz>0 % && DO_SEPARATE_IV==1
        % first term only affects Y's since instruments have a zero loading via Coeff.Ay
        VarY_IV=Coeff.Ay(CoreIV_idx,:) * VarX * Coeff.Ay(CoreIV_idx,:)' + Coeff.By(CoreIV_idx,ShockIndex)*Coeff.By(CoreIV_idx,ShockIndex)';
    end;
    % fill matrix of regressor variance with copies of unconditional variance
    

%     lagZlagZ  =  YY+ g_0*g_0' + (g_0*g_1'*SumT_T(1,T)+g_1*g_0'*SumT_T(1,T))  + ( g_1*g_1' * SumTSq_T(1,1,T) );
%     lagZlag2Z =A*YY+ g_0*g_0' + (g_0*g_1'*SumT_T(2,T)+g_1*g_0'*SumT_T(1,T))  + ( g_1*g_1' * SumTSq_T(1,2,T) );
%     lag2Zlag2Z=  YY+ g_0*g_0' + (g_0*g_1'*SumT_T(2,T)+g_1*g_0'*SumT_T(2,T))  + ( g_1*g_1' * SumTSq_T(2,2,T) );
% 
%     RegressorVar=[lagZlagZ, lagZlag2Z, g_0+g_1*SumT_T(1,T), g_0*SumT_T(0,T) + g_1 * SumTSq_T(1,0,T); ...
%                     lagZlag2Z', lag2Zlag2Z, g_0+g_1*SumT_T(2,T), g_0*SumT_T(0,T) + g_1 * SumTSq_T(2,0,T); ...
%                     g_0'+g_1'*SumT_T(1,T), g_0'+g_1'*SumT_T(2,T), 1 , SumT_T(0,T) ; ...
%                     g_0'*SumT_T(0,T) + g_1' * SumTSq_T(1,0,T), g_0'*SumT_T(0,T) + g_1' * SumTSq_T(2,0,T), SumT_T(0,T) , SumTSq_T(0,0,T) ];

    if ModelStructure.ConstantTerms==2
        for ll1=1:OriginalData.k
            % pick contemporaneous component
            RegressorVarSimple((ll1-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY))= VarY ;
            RegressorVar((ll1-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY))= RegressorVarSimple((ll1-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY)) + ...
                Coeff.Mean(Core_idx)*Coeff.Mean(Core_idx)'+ ...
                Coeff.Mean(Core_idx)*Coeff.Trend(Core_idx)'*SumT_T(ll1,OriginalData.k+1)+Coeff.Trend(Core_idx)*Coeff.Mean(Core_idx)'*SumT_T(ll1,OriginalData.k+1)  +...
                Coeff.Trend(Core_idx)*Coeff.Trend(Core_idx)'*SumTSq_T(ll1,ll1,OriginalData.k+1) ;
            % CurrentCov is just a short-hand below
            for ll2=(ll1+1):OriginalData.k
                % covariance with lags depends on A matrix governing X dynamics and map back into Y going into VAR
                % use that a priori, everything has a zero mean. Trend is in Coeff.Trend
                % Divide all the deterministic terms by OriginalData.T or else multiply all stochastic terms by OriginalData.T
                CurrentCovSimple=Coeff.Ay(Core_idx,:) * Coeff.A^abs(ll1-ll2)*VarX * Coeff.Ay(Core_idx,:)' + ...
                    Coeff.Ay(Core_idx,:)* Coeff.A^(abs(ll1-ll2)-1)*Coeff.B(:,ShockIndex) * Coeff.By(Core_idx,ShockIndex)';
                CurrentCovAdded=  Coeff.Mean(Core_idx)*Coeff.Mean(Core_idx)'+ ...
                                Coeff.Mean(Core_idx)*Coeff.Trend(Core_idx)'*SumT_T(ll2,OriginalData.k+1)+Coeff.Trend(Core_idx)*Coeff.Mean(Core_idx)'*SumT_T(ll1,OriginalData.k+1)  +...
                                Coeff.Trend(Core_idx)*Coeff.Trend(Core_idx)'*SumTSq_T(ll1,ll2,OriginalData.k+1) ;

                RegressorVarSimple((ll1-1)*CurrentY+(1:CurrentY),(ll2-1)*CurrentY+(1:CurrentY))=CurrentCovSimple;
                RegressorVarSimple((ll2-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY))=CurrentCovSimple';

                % Note: potential discrepancy between this code and the tested code? Position of Coeff.Ay in covariance?
                RegressorVar((ll1-1)*CurrentY+(1:CurrentY),(ll2-1)*CurrentY+(1:CurrentY))=CurrentCovSimple+CurrentCovAdded;
                RegressorVar((ll2-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY))=CurrentCovSimple'+CurrentCovAdded';
            end;
            % Add constant term / trend(s) at the end
            RegressorVar((ll1-1)*CurrentY+(1:CurrentY), OriginalData.k*CurrentY+(1:2))= ...
                [Coeff.Mean(Core_idx)+Coeff.Trend(Core_idx)*SumT_T(ll1,OriginalData.k+1), Coeff.Mean(Core_idx)*SumT_T(0,OriginalData.k+1) + Coeff.Trend(Core_idx)* SumTSq_T(ll1,0,OriginalData.k+1)];
            RegressorVar(OriginalData.k*CurrentY+(1:2), (ll1-1)*CurrentY+(1:CurrentY))= ...
                [Coeff.Mean(Core_idx)+Coeff.Trend(Core_idx)*SumT_T(ll1,OriginalData.k+1), Coeff.Mean(Core_idx)*SumT_T(0,OriginalData.k+1) + Coeff.Trend(Core_idx)* SumTSq_T(ll1,0,OriginalData.k+1)]';

            % Now: Cov[X, Y]
            CurrentCovSimple = Coeff.Ay(Core_idx,:) * VarX * (Coeff.A')^ll1* Coeff.Ay(CoreLHS_idx,:)' + ...
                    Coeff.By(Core_idx,ShockIndex) * Coeff.B(:,ShockIndex)' * (Coeff.A')^(ll1-1) * Coeff.Ay(CoreLHS_idx,:)' ;
            CurrentCov = CurrentCovSimple + ...
                        Coeff.Mean(Core_idx)*Coeff.Mean(CoreLHS_idx)' + ...
                        ((Coeff.Mean(Core_idx)*Coeff.Trend(CoreLHS_idx)'*SumT_T(0,OriginalData.k+1)+Coeff.Trend(Core_idx)*Coeff.Mean(CoreLHS_idx)'*SumT_T(ll1,OriginalData.k+1)) ) + ...
                        Coeff.Trend(Core_idx)*Coeff.Trend(CoreLHS_idx)' * SumTSq_T(0,ll1,OriginalData.k+1) ;
    % Old version that works without periphery
    %         CurrentCov=Coeff.Ay(Core_idx,:) * VarX * (Coeff.A')^ll1* Coeff.Ay(CoreLHS_idx,:)' + ...
    %             Coeff.By(Core_idx,ShockIndex) * Coeff.B(:,ShockIndex)' * (Coeff.A')^(ll1-1) * Coeff.Ay(CoreLHS_idx,:)' + ...
    %                     Coeff.Mean(Core_idx)*Coeff.Mean(CoreLHS_idx)' + ...
    %                     ((Coeff.Mean(Core_idx)*Coeff.Trend(CoreLHS_idx)'*SumT_T(ll1,OriginalData.T)+Coeff.Trend(Core_idx)*Coeff.Mean(CoreLHS_idx)'*SumT_T(0,OriginalData.T)) )'+ ...
    %                     Coeff.Trend(Core_idx)*Coeff.Trend(CoreLHS_idx)' * SumTSq_T(0,ll1,OriginalData.T) ;
    %                 ((Coeff.Mean(CoreLHS_idx)*Coeff.Trend(Core_idx)'*SumT_T(0,OriginalData.T)+Coeff.Trend(CoreLHS_idx)*Coeff.Mean(Core_idx)'*SumT_T(ll1,OriginalData.T)) )

    %                     Coeff.Trend(Core_idx)*Coeff.Trend(CoreLHS_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6;
                    %g_0*g_0' + (g_0*g_1'*SumT_T(1,OriginalData.T)+g_1*g_0'*SumT_T(0,OriginalData.T))   + ( g_1*g_1' * SumTSq_T(0,1,OriginalData.T) );
            RegressorCovSimple((ll1-1)*CurrentY+(1:CurrentY),:) = CurrentCovSimple;
            RegressorCov((ll1-1)*CurrentY+(1:CurrentY),:) = CurrentCov;
    %     RegressorCov=[ZlagZ'; Zlag2Z'; g_0'+g_1'*SumT_T(0,T); g_0'*SumT_T(0,T) + g_1' * SumTSq_T(0,0,T)  ];
    %     ZlagZ   =A*YY+ g_0*g_0' + (g_0*g_1'*SumT_T(1,T)+g_1*g_0'*SumT_T(0,T))   + ( g_1*g_1' * SumTSq_T(0,1,T) );
    %     Zlag2Z   =A^2*YY + g_0*g_0' + (g_0*g_1'*SumT_T(2,T)+g_1*g_0'*SumT_T(0,T))   + ( g_1*g_1' * SumTSq_T(0,2,T) );
        end;
        % Add expected values of trend terms: Zero covariance with purely stochastic terms, so only the deterministic part is left
        RegressorVar(OriginalData.k*CurrentY+(1:2),OriginalData.k*CurrentY+(1:2))=[1 , SumT_T(0,OriginalData.k+1) ; SumT_T(0,OriginalData.k+1) , SumTSq_T(0,0,OriginalData.k+1) ];
        RegressorCov(OriginalData.k*CurrentY+(1:2),:) = ...
            [Coeff.Mean(CoreLHS_idx)'+Coeff.Trend(CoreLHS_idx)'*SumT_T(0,OriginalData.k+1); ...
                Coeff.Mean(CoreLHS_idx)'*SumT_T(0,OriginalData.k+1) + Coeff.Trend(CoreLHS_idx)' * SumTSq_T(0,0,OriginalData.k+1)  ];
    elseif ModelStructure.ConstantTerms==1
        for ll1=1:OriginalData.k
            % pick contemporaneous component
            RegressorVarSimple((ll1-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY))= VarY ;
            RegressorVar((ll1-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY))= RegressorVarSimple((ll1-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY)) + ...
                Coeff.Mean(Core_idx)*Coeff.Mean(Core_idx)'  ;
            % CurrentCov is just a short-hand below
            for ll2=(ll1+1):OriginalData.k
                % covariance with lags depends on A matrix governing X dynamics and map back into Y going into VAR
                % use that a priori, everything has a zero mean. Trend is in Coeff.Trend
                % Divide all the deterministic terms by OriginalData.T or else multiply all stochastic terms by OriginalData.T
                CurrentCovSimple=Coeff.Ay(Core_idx,:) * Coeff.A^abs(ll1-ll2)*VarX * Coeff.Ay(Core_idx,:)' + ...
                    Coeff.Ay(Core_idx,:)* Coeff.A^(abs(ll1-ll2)-1)*Coeff.B(:,ShockIndex) * Coeff.By(Core_idx,ShockIndex)';
                CurrentCovAdded=  Coeff.Mean(Core_idx)*Coeff.Mean(Core_idx)' ;

                RegressorVarSimple((ll1-1)*CurrentY+(1:CurrentY),(ll2-1)*CurrentY+(1:CurrentY))=CurrentCovSimple;
                RegressorVarSimple((ll2-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY))=CurrentCovSimple';

                % Note: potential discrepancy between this code and the tested code? Position of Coeff.Ay in covariance?
                RegressorVar((ll1-1)*CurrentY+(1:CurrentY),(ll2-1)*CurrentY+(1:CurrentY))=CurrentCovSimple+CurrentCovAdded;
                RegressorVar((ll2-1)*CurrentY+(1:CurrentY),(ll1-1)*CurrentY+(1:CurrentY))=CurrentCovSimple'+CurrentCovAdded';
            end;
            % Add constant term / trend(s) at the end
            RegressorVar((ll1-1)*CurrentY+(1:CurrentY), OriginalData.k*CurrentY+1 )= Coeff.Mean(Core_idx);
            RegressorVar(OriginalData.k*CurrentY+(1:1), (ll1-1)*CurrentY+(1:CurrentY))= Coeff.Mean(Core_idx)';

            % Now: Cov[X, Y]
            CurrentCovSimple = Coeff.Ay(Core_idx,:) * VarX * (Coeff.A')^ll1* Coeff.Ay(CoreLHS_idx,:)' + ...
                    Coeff.By(Core_idx,ShockIndex) * Coeff.B(:,ShockIndex)' * (Coeff.A')^(ll1-1) * Coeff.Ay(CoreLHS_idx,:)' ;
            CurrentCov = CurrentCovSimple + ...
                        Coeff.Mean(Core_idx)*Coeff.Mean(CoreLHS_idx)'  ;
    % Old version that works without periphery
    %         CurrentCov=Coeff.Ay(Core_idx,:) * VarX * (Coeff.A')^ll1* Coeff.Ay(CoreLHS_idx,:)' + ...
    %             Coeff.By(Core_idx,ShockIndex) * Coeff.B(:,ShockIndex)' * (Coeff.A')^(ll1-1) * Coeff.Ay(CoreLHS_idx,:)' + ...
    %                     Coeff.Mean(Core_idx)*Coeff.Mean(CoreLHS_idx)' + ...
    %                     ((Coeff.Mean(Core_idx)*Coeff.Trend(CoreLHS_idx)'*SumT_T(ll1,OriginalData.T)+Coeff.Trend(Core_idx)*Coeff.Mean(CoreLHS_idx)'*SumT_T(0,OriginalData.T)) )'+ ...
    %                     Coeff.Trend(Core_idx)*Coeff.Trend(CoreLHS_idx)' * SumTSq_T(0,ll1,OriginalData.T) ;
    %                 ((Coeff.Mean(CoreLHS_idx)*Coeff.Trend(Core_idx)'*SumT_T(0,OriginalData.T)+Coeff.Trend(CoreLHS_idx)*Coeff.Mean(Core_idx)'*SumT_T(ll1,OriginalData.T)) )

    %                     Coeff.Trend(Core_idx)*Coeff.Trend(CoreLHS_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6;
                    %g_0*g_0' + (g_0*g_1'*SumT_T(1,OriginalData.T)+g_1*g_0'*SumT_T(0,OriginalData.T))   + ( g_1*g_1' * SumTSq_T(0,1,OriginalData.T) );
            RegressorCovSimple((ll1-1)*CurrentY+(1:CurrentY),:) = CurrentCovSimple;
            RegressorCov((ll1-1)*CurrentY+(1:CurrentY),:) = CurrentCov;
    %     RegressorCov=[ZlagZ'; Zlag2Z'; g_0'+g_1'*SumT_T(0,T); g_0'*SumT_T(0,T) + g_1' * SumTSq_T(0,0,T)  ];
    %     ZlagZ   =A*YY+ g_0*g_0' + (g_0*g_1'*SumT_T(1,T)+g_1*g_0'*SumT_T(0,T))   + ( g_1*g_1' * SumTSq_T(0,1,T) );
    %     Zlag2Z   =A^2*YY + g_0*g_0' + (g_0*g_1'*SumT_T(2,T)+g_1*g_0'*SumT_T(0,T))   + ( g_1*g_1' * SumTSq_T(0,2,T) );
        end;
        % Add expected values of trend terms: Zero covariance with purely stochastic terms, so only the deterministic part is left
        RegressorVar(OriginalData.k*CurrentY+1,OriginalData.k*CurrentY+1)=1;
        RegressorCov(OriginalData.k*CurrentY+1,:) = Coeff.Mean(CoreLHS_idx)';
    end;
%     [Coeff.Trend(CoreLHS_idx)' * (OriginalData.T+1) / 2 ; Coeff.Trend(CoreLHS_idx)' * (OriginalData.T+1)*(2*OriginalData.T+1) / 6 ];
%     RegressorCov=[ZlagZ'; Zlag2Z'; g_0'+g_1'*SumT_T(0,OriginalData.T); g_0'*SumT_T(0,OriginalData.T) + g_1' * SumTSq_T(0,0,OriginalData.T)  ];

    % Take account of Core-Periphery exclusion restrictions
    if pp==0
        
        % OLS estimation of unrestricted (core) VAR
%         B_prior=(RegressorVar)\RegressorCov;
        B_priorSimple=RegressorVarSimple\RegressorCovSimple(1:OriginalData.k*CurrentY,:);
        Coeff.Trend_adj=Coeff.Trend(Core_idx);
        Coeff.Mean_adj=Coeff.Mean(Core_idx);

%         sort(abs(eig(K)))

%         SIG=VarY-RegressorCov(1:NowY*OriginalData.k,1:NowY)'*inv(RegressorVar(1:NowY*OriginalData.k,1:NowY*OriginalData.k))*RegressorCov(1:NowY*OriginalData.k,1:NowY)
        % variance-covariance matrix of residuals -- taking into account zero restrictions
%         SIG_alt=VarY+Coeff.Trend(Core_idx)*Coeff.Trend(Core_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6 ...
%             -BB'*RegressorVar*BB
%         if DO_SEPARATE_IV==1
%             S_prior=[VarY+Coeff.Trend(Core_idx)*Coeff.Trend(Core_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1)/6+...
%                     -B_prior'*(RegressorVar)*B_prior ...
%                     VarY_IV( 1:ModelStructure.m, (ModelStructure.m+1):end ) ; ...
%                     VarY_IV(end-(ModelStructure.mz-1):end,:)] ;
%                 
%             display('to do this, I need to use a simpler "RegressorVar", which already takes out the deterministic elements');
%             error('implement S_prior without including trend terms in the first place and without taking them out...');
        if ModelStructure.ConstantTerms==2
            for kk=1:OriginalData.k
                Coeff.Trend_adj=Coeff.Trend_adj-B_priorSimple(CurrentY*(kk-1)+(1:CurrentY),:)'*Coeff.Trend(Core_idx);
                Coeff.Mean_adj=Coeff.Mean_adj+B_priorSimple(CurrentY*(kk-1)+(1:CurrentY),:)'*(Coeff.Trend(Core_idx)*kk-Coeff.Mean(Core_idx));
            end;            
            
            B_prior=[B_priorSimple; Coeff.Mean_adj'; Coeff.Trend_adj'];

    %         display(rcond(RegressorVar));
    %         display([1-nancorr(B_prior(:),B_prior_alt(:)) norm(B_prior-B_prior_alt) ]);
    %         inv(eye(3)-Coeff.Ay)*B_prior(end,:)'
            K=[B_prior(1:CurrentY*OriginalData.k,:)'; eye(CurrentY*(OriginalData.k-1),CurrentY*OriginalData.k)];
    %         KSimple=[B_priorSimple'; eye(CurrentY*(OriginalData.k-1),CurrentY*OriginalData.k)];
            Prior_rho=max(abs(eig(K)));
            if ModelStructure.mz>0
                S_prior=[(VarY - B_prior'*(RegressorVar)*B_prior + ...
                    Coeff.Mean(Core_idx)*Coeff.Mean(Core_idx)'+ ...
                    Coeff.Mean(Core_idx)*Coeff.Trend(Core_idx)'*SumT_T(0,OriginalData.k+1)+Coeff.Trend(Core_idx)*Coeff.Mean(Core_idx)'*SumT_T(0,OriginalData.k+1)  +...
                    Coeff.Trend(Core_idx)*Coeff.Trend(Core_idx)'*SumTSq_T(0,0,OriginalData.k+1)) ...
                    VarY_IV( 1:ModelStructure.m, (ModelStructure.m+1):end ) ; ...
                    VarY_IV(end-(ModelStructure.mz-1):end,:)] ;
                Chol_cond_var=chol(Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2))*Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2))', 'lower');
                DSGE_Rotate=Chol_cond_var\Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2));
            else
                S_prior=[ (VarY - B_prior'*(RegressorVar)*B_prior + ...
                    Coeff.Mean(Core_idx)*Coeff.Mean(Core_idx)'+ ...
                    Coeff.Mean(Core_idx)*Coeff.Trend(Core_idx)'*SumT_T(0,OriginalData.k+1)+Coeff.Trend(Core_idx)*Coeff.Mean(Core_idx)'*SumT_T(0,OriginalData.k+1)  +...
                    Coeff.Trend(Core_idx)*Coeff.Trend(Core_idx)'*SumTSq_T(0,0,OriginalData.k+1)) ] ;
                DSGE_Rotate=[];
            end;
        elseif ModelStructure.ConstantTerms==1
            B_prior=[B_priorSimple; Coeff.Mean_adj' ];

    %         display(rcond(RegressorVar));
    %         display([1-nancorr(B_prior(:),B_prior_alt(:)) norm(B_prior-B_prior_alt) ]);
    %         inv(eye(3)-Coeff.Ay)*B_prior(end,:)'
            K=[B_prior(1:CurrentY*OriginalData.k,:)'; eye(CurrentY*(OriginalData.k-1),CurrentY*OriginalData.k)];
    %         KSimple=[B_priorSimple'; eye(CurrentY*(OriginalData.k-1),CurrentY*OriginalData.k)];
            Prior_rho=max(abs(eig(K)));
            if ModelStructure.mz>0
                S_prior=[(VarY - B_prior'*(RegressorVar)*B_prior + ...
                    Coeff.Mean(Core_idx)*Coeff.Mean(Core_idx)') ...
                    VarY_IV( 1:ModelStructure.m, (ModelStructure.m+1):end ) ; ...
                    VarY_IV(end-(ModelStructure.mz-1):end,:)] ;
                Chol_cond_var=chol(Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2))*Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2))', 'lower');
                DSGE_Rotate=Chol_cond_var\Coeff.By(ForecastErrIdx(:,1),ForecastErrIdx(:,2));
            else
                S_prior=[ (VarY - B_prior'*(RegressorVar)*B_prior + ...
                    Coeff.Mean(Core_idx)*Coeff.Mean(Core_idx)') ] ;
                DSGE_Rotate=[];
            end;
        end;
%             error('alternatively, add trends into the above specification and compare...');
%             
%             S_prior=[VarY+...
%                     -B_prior(1:end-2,:)'*RegressorVarSimple*B_prior(1:end-2,:) ...
%                     VarY_IV( 1:ModelStructure.m, (ModelStructure.m+1):end ) ; ...
%                     VarY_IV(end-(ModelStructure.mz-1):end,:)] ;            
%         else
%             S_prior=[VarY+Coeff.Trend(Core_idx)*Coeff.Trend(Core_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1)/6+...
%                     -B_prior'*(RegressorVar)*B_prior ] ;
%         end;

%         vec_VarX_cond = (eye(ModelStructure.mX^2)- kron(Coeff.A,Coeff.A)) \ reshape(Coeff.B(:,ForecastErrIdx(:,2))*Coeff.B(:,ForecastErrIdx(:,2))', ModelStructure.mX^2,1);
%         % bring into normal shape
%         VarX_cond=reshape(vec_VarX_cond,ModelStructure.mX,ModelStructure.mX);
%         % unconditional variance of Y via unconditional variance of X and increment
%         VarY_cond=Coeff.Ay(Core_idx,:) * VarX_cond * Coeff.Ay(Core_idx,:)' + Coeff.By(Core_idx,ForecastErrIdx(:,2))*Coeff.By(Core_idx,ForecastErrIdx(:,2))';
%         Chol_cond_var=chol(VarY_cond(ForecastErrIdx(:,1),ForecastErrIdx(:,1)), 'lower');
%         S_prior_Benchmark=[ Coeff.By(Core_idx,ShockIndex)*Coeff.By(Core_idx,ShockIndex)', VarY_IV( 1:ModelStructure.m, (ModelStructure.m+1):end ); VarY_IV(end-(ModelStructure.mz-1):end,:)] ;

%         prior_IRF_IV=my_IV_Identification(S_prior(1:ModelStructure.m,1:ModelStructure.m),S_prior(ModelStructure.m+(1:ModelStructure.mz),1:6),ModelStructure.mz);
%         rotated_prior_IRF=prior_IRF_IV*DSGE_Rotate*diag(sign(diag(prior_IRF_IV*DSGE_Rotate)));
%         save('tmp.mat','Chol_cond_var', 'VarY', 'Coeff', 'rotated_prior_IRF', 'prior_IRF_IV');

        Prior_eig_S_Prior=max(eig(S_prior));
        % note that Cov(IV,Y-BX)=Cov(IV,Y) since IV is iid and X is predetermined in the model
%         scatter(reshape(SIG(1:mC,1:mC),mC^2,1),VarY(:));
%         nancorr(reshape(SIG(1:mC,1:mC),mC^2,1),VarY(:))
        NXY=RegressorCov;
        NXX=RegressorVar;
    else
        
%         Yperiphery=Y(:,my_Exclusions);
% In the VAR, the extra columns are appended at the end
%         Xperiphery=[X Y(:,S_idx_Included)];
        % need to add VarY(1:mC,CoreLHS_idx)
        
        % new bottom right corner of X'X-analogue: the variance of the contemp. core variables
        if ModelStructure.ConstantTerms==2
            RegressorVar(CurrentY*OriginalData.k+ModelStructure.ConstantTerms+(1:ModelStructure.m),CurrentY*OriginalData.k+ModelStructure.ConstantTerms+(1:ModelStructure.m)) = ...
                VarY(1:ModelStructure.m,1:ModelStructure.m)' + ...
                Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Mean(Core_idx(1:ModelStructure.m))'+ ...
                Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Trend(Core_idx(1:ModelStructure.m))'*SumT_T(0,OriginalData.k+1)+...
                    Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Mean(Core_idx(1:ModelStructure.m))'*SumT_T(0,OriginalData.k+1)  +...
                Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Trend(Core_idx(1:ModelStructure.m))'*SumTSq_T(0,0,OriginalData.k+1) ;

            for ll1=1:OriginalData.k
                % CurrentCov is just a short-hand below
                % covariance with lags depends on A matrix governing X dynamics and map back into Y going into VAR

                % CurrentCov is the bottom left entry: the covariance of the current core with the lagged core + periphery
                CurrentCov=Coeff.Ay(Core_idx(1:ModelStructure.m),:) * (Coeff.A)^ll1 * VarX * Coeff.Ay(Core_idx,:)' + ...
                    Coeff.Ay(Core_idx(1:ModelStructure.m),:) * (Coeff.A)^(ll1-1) * Coeff.B(:,ShockIndex) * Coeff.By(Core_idx,ShockIndex)'  + ...
                    Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Mean(Core_idx)'+ ...
                    Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Trend(Core_idx)'*SumT_T(0,OriginalData.k+1)+ ...
                        Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Mean(Core_idx)'*SumT_T(ll1,OriginalData.k+1)  +...
                    Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Trend(Core_idx)'*SumTSq_T(ll1,0,OriginalData.k+1) ;
    %             Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Trend(Core_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6;
                % bottom left entry
                RegressorVar( (ModelStructure.m+ModelStructure.mP)*OriginalData.k+ModelStructure.ConstantTerms + (1:ModelStructure.m), (1:(ModelStructure.m+ModelStructure.mP))+(ll1-1)*(ModelStructure.m+ModelStructure.mP) ) = CurrentCov;
                % top right entry
                RegressorVar(  (1:(ModelStructure.m+ModelStructure.mP))+(ll1-1)*(ModelStructure.m+ModelStructure.mP),  (ModelStructure.m+ModelStructure.mP)*OriginalData.k+ModelStructure.ConstantTerms + (1:ModelStructure.m) )= CurrentCov';
            end;
            % contemporaneous covariance for added columns: Covariance of contemp. core with contemp. periphery
            RegressorCov(CurrentY*OriginalData.k+ModelStructure.ConstantTerms+(1:ModelStructure.m),:) = VarY(1:ModelStructure.m,CoreLHS_idx) + ...
                        Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Mean(CoreLHS_idx)' + ...
                        ( (Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Trend(CoreLHS_idx)'*SumT_T(0,OriginalData.k+1)+...
                             Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Mean(CoreLHS_idx)'*SumT_T(0,OriginalData.k+1)) ) + ...
                        Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Trend(CoreLHS_idx)' * SumTSq_T(0,0,OriginalData.k+1) ;
    %                         Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Trend(CoreLHS_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6;


    % % for debugging only
    %         AdHoc=mC-1;
    %         PeripheryBBshort=(RegressorVar(1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc,1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc))\RegressorCov(1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc,:);
    %         PeripherySIGshort=VarY(CoreLHS_idx,CoreLHS_idx)+Coeff.Trend(CoreLHS_idx)*Coeff.Trend(CoreLHS_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6  ...
    %             -PeripheryBBshort'*(RegressorVar(1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc,1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc))*PeripheryBBshort
    %         eig(PeripherySIGshort)
    %         PeripheryNXX=RegressorVar;

            PeripheryB_prior=(RegressorVar)\RegressorCov;
    %         PeripheryS_prior=VarY(CoreLHS_idx,CoreLHS_idx)+Coeff.Trend(CoreLHS_idx)*Coeff.Trend(CoreLHS_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6 ...
    %             -PeripheryB_prior'*(RegressorVar)*PeripheryB_prior;

            PeripheryS_prior=(VarY(CoreLHS_idx,CoreLHS_idx) - PeripheryB_prior'*(RegressorVar)*PeripheryB_prior + ...
                    Coeff.Mean(CoreLHS_idx)*Coeff.Mean(CoreLHS_idx)'+ ...
                    Coeff.Mean(CoreLHS_idx)*Coeff.Trend(CoreLHS_idx)'*SumT_T(0,OriginalData.k+1)+Coeff.Trend(CoreLHS_idx)*Coeff.Mean(CoreLHS_idx)'*SumT_T(0,OriginalData.k+1)  +...
                    Coeff.Trend(CoreLHS_idx)*Coeff.Trend(CoreLHS_idx)'*SumTSq_T(0,0,OriginalData.k+1)) ;          
        elseif ModelStructure.ConstantTerms==1
            RegressorVar(CurrentY*OriginalData.k+ModelStructure.ConstantTerms+(1:ModelStructure.m),CurrentY*OriginalData.k+ModelStructure.ConstantTerms+(1:ModelStructure.m)) = ...
                VarY(1:ModelStructure.m,1:ModelStructure.m)' + ...
                Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Mean(Core_idx(1:ModelStructure.m))';

            for ll1=1:OriginalData.k
                % CurrentCov is just a short-hand below
                % covariance with lags depends on A matrix governing X dynamics and map back into Y going into VAR

                % CurrentCov is the bottom left entry: the covariance of the current core with the lagged core + periphery
                CurrentCov=Coeff.Ay(Core_idx(1:ModelStructure.m),:) * (Coeff.A)^ll1 * VarX * Coeff.Ay(Core_idx,:)' + ...
                    Coeff.Ay(Core_idx(1:ModelStructure.m),:) * (Coeff.A)^(ll1-1) * Coeff.B(:,ShockIndex) * Coeff.By(Core_idx,ShockIndex)'  + ...
                    Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Mean(Core_idx)';
    %             Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Trend(Core_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6;
                % bottom left entry
                RegressorVar( (ModelStructure.m+ModelStructure.mP)*OriginalData.k+ModelStructure.ConstantTerms + (1:ModelStructure.m), (1:(ModelStructure.m+ModelStructure.mP))+(ll1-1)*(ModelStructure.m+ModelStructure.mP) ) = CurrentCov;
                % top right entry
                RegressorVar(  (1:(ModelStructure.m+ModelStructure.mP))+(ll1-1)*(ModelStructure.m+ModelStructure.mP),  (ModelStructure.m+ModelStructure.mP)*OriginalData.k+ModelStructure.ConstantTerms + (1:ModelStructure.m) )= CurrentCov';
            end;
            % contemporaneous covariance for added columns: Covariance of contemp. core with contemp. periphery
            RegressorCov(CurrentY*OriginalData.k+ModelStructure.ConstantTerms+(1:ModelStructure.m),:) = VarY(1:ModelStructure.m,CoreLHS_idx) + ...
                        Coeff.Mean(Core_idx(1:ModelStructure.m))*Coeff.Mean(CoreLHS_idx)' ;
    %                         Coeff.Trend(Core_idx(1:ModelStructure.m))*Coeff.Trend(CoreLHS_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6;


    % % for debugging only
    %         AdHoc=mC-1;
    %         PeripheryBBshort=(RegressorVar(1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc,1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc))\RegressorCov(1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc,:);
    %         PeripherySIGshort=VarY(CoreLHS_idx,CoreLHS_idx)+Coeff.Trend(CoreLHS_idx)*Coeff.Trend(CoreLHS_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6  ...
    %             -PeripheryBBshort'*(RegressorVar(1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc,1:m_all*OriginalData.k+ModelStructure.ConstantTerms+AdHoc))*PeripheryBBshort
    %         eig(PeripherySIGshort)
    %         PeripheryNXX=RegressorVar;

            PeripheryB_prior=(RegressorVar)\RegressorCov;
    %         PeripheryS_prior=VarY(CoreLHS_idx,CoreLHS_idx)+Coeff.Trend(CoreLHS_idx)*Coeff.Trend(CoreLHS_idx)'*(OriginalData.T+1)*(2*OriginalData.T+1) / 6 ...
    %             -PeripheryB_prior'*(RegressorVar)*PeripheryB_prior;

            PeripheryS_prior=(VarY(CoreLHS_idx,CoreLHS_idx) - PeripheryB_prior'*(RegressorVar)*PeripheryB_prior + ...
                    Coeff.Mean(CoreLHS_idx)*Coeff.Mean(CoreLHS_idx)' ) ;          
        end;
        
        PeripheryNXY=RegressorCov;
        PeripheryNXX=RegressorVar;
%         S_prior
%         eig(PeripheryS_prior)
        if min(eig(PeripheryS_prior))<0
            display('error with core-periphery: check whether covariance is computationally sound');
            display('check with simulated model');
%             error('non PD variance');
        end;

    end;
end;

